/* Generated by Cython 0.29.21 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "ParseTree.ParseNode",
        "sources": [
            "ParseTree/ParseNode.pyx"
        ]
    },
    "module_name": "ParseTree.ParseNode"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_21"
#define CYTHON_HEX_VERSION 0x001D15F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ParseTree__ParseNode
#define __PYX_HAVE_API__ParseTree__ParseNode
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "ParseTree/ParseNode.pyx",
  "stringsource",
  "Word.pxd",
  "ParseTree/Symbol.pxd",
  "ParseTree/ConstituentSpan.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_10Dictionary_4Word_Word;
struct __pyx_obj_9ParseTree_6Symbol_Symbol;
struct __pyx_obj_9ParseTree_15ConstituentSpan_ConstituentSpan;
struct __pyx_obj_9ParseTree_9ParseNode_ParseNode;
struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild;

/* "ParseTree/ParseNode.pxd":13
 *     cpdef ParseNode headLeaf(self)
 *     cpdef ParseNode headChild(self)
 *     cpdef addChild(self, ParseNode child, index=*)             # <<<<<<<<<<<<<<
 *     cpdef correctParents(self)
 *     cpdef removeXNodes(self)
 */
struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild {
  int __pyx_n;
  PyObject *index;
};

/* "Dictionary/Word.pxd":1
 * cdef class Word:             # <<<<<<<<<<<<<<
 * 
 *     cdef str name
 */
struct __pyx_obj_10Dictionary_4Word_Word {
  PyObject_HEAD
  struct __pyx_vtabstruct_10Dictionary_4Word_Word *__pyx_vtab;
  PyObject *name;
};


/* "ParseTree/Symbol.pxd":4
 * 
 * 
 * cdef class Symbol(Word):             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isVP(self)
 */
struct __pyx_obj_9ParseTree_6Symbol_Symbol {
  struct __pyx_obj_10Dictionary_4Word_Word __pyx_base;
};


/* "ParseTree/ConstituentSpan.pxd":4
 * 
 * 
 * cdef class ConstituentSpan:             # <<<<<<<<<<<<<<
 * 
 *     cdef Symbol constituent
 */
struct __pyx_obj_9ParseTree_15ConstituentSpan_ConstituentSpan {
  PyObject_HEAD
  struct __pyx_vtabstruct_9ParseTree_15ConstituentSpan_ConstituentSpan *__pyx_vtab;
  struct __pyx_obj_9ParseTree_6Symbol_Symbol *constituent;
  int start;
  int end;
};


/* "ParseTree/ParseNode.pxd":4
 * 
 * 
 * cdef class ParseNode:             # <<<<<<<<<<<<<<
 * 
 *     cdef list children
 */
struct __pyx_obj_9ParseTree_9ParseNode_ParseNode {
  PyObject_HEAD
  struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *__pyx_vtab;
  PyObject *children;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *parent;
  struct __pyx_obj_9ParseTree_6Symbol_Symbol *data;
};



/* "Dictionary/Word.pxd":1
 * cdef class Word:             # <<<<<<<<<<<<<<
 * 
 *     cdef str name
 */

struct __pyx_vtabstruct_10Dictionary_4Word_Word {
  int (*charCount)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*getName)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*setName)(struct __pyx_obj_10Dictionary_4Word_Word *, PyObject *, int __pyx_skip_dispatch);
  int (*isPunctuation)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
  PyObject *(*toCharacters)(struct __pyx_obj_10Dictionary_4Word_Word *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_10Dictionary_4Word_Word *__pyx_vtabptr_10Dictionary_4Word_Word;


/* "ParseTree/Symbol.pxd":4
 * 
 * 
 * cdef class Symbol(Word):             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isVP(self)
 */

struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol {
  struct __pyx_vtabstruct_10Dictionary_4Word_Word __pyx_base;
  int (*isVP)(struct __pyx_obj_9ParseTree_6Symbol_Symbol *, int __pyx_skip_dispatch);
  int (*isTerminal)(struct __pyx_obj_9ParseTree_6Symbol_Symbol *, int __pyx_skip_dispatch);
  int (*isChunkLabel)(struct __pyx_obj_9ParseTree_6Symbol_Symbol *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_6Symbol_Symbol *(*trimSymbol)(struct __pyx_obj_9ParseTree_6Symbol_Symbol *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *__pyx_vtabptr_9ParseTree_6Symbol_Symbol;


/* "ParseTree/ConstituentSpan.pxd":4
 * 
 * 
 * cdef class ConstituentSpan:             # <<<<<<<<<<<<<<
 * 
 *     cdef Symbol constituent
 */

struct __pyx_vtabstruct_9ParseTree_15ConstituentSpan_ConstituentSpan {
  int (*getStart)(struct __pyx_obj_9ParseTree_15ConstituentSpan_ConstituentSpan *, int __pyx_skip_dispatch);
  int (*getEnd)(struct __pyx_obj_9ParseTree_15ConstituentSpan_ConstituentSpan *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_6Symbol_Symbol *(*getConstituent)(struct __pyx_obj_9ParseTree_15ConstituentSpan_ConstituentSpan *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9ParseTree_15ConstituentSpan_ConstituentSpan *__pyx_vtabptr_9ParseTree_15ConstituentSpan_ConstituentSpan;


/* "ParseTree/ParseNode.pyx":7
 * 
 * 
 * cdef class ParseNode:             # <<<<<<<<<<<<<<
 * 
 *     ADJP = ["NNS", "QP", "NN", "$", "ADVP", "JJ", "VBN", "VBG", "ADJP", "JJR", "NP", "JJS", "DT", "FW", "RBR", "RBS",
 */

struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*__pyx___searchHeadChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, PyObject *, PyObject *, int, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*headLeaf)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*headChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*addChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch, struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild *__pyx_optional_args);
  PyObject *(*correctParents)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*removeXNodes)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*setChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*removeChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*leafCount)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*nodeCount)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*nodeCountWithMultipleChildren)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*numberOfChildren)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*getChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*firstChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*lastChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*isLastChild)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*getChildIndex)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*isDescendant)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*previousSibling)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*nextSibling)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*getParent)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  struct __pyx_obj_9ParseTree_6Symbol_Symbol *(*getData)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*setData)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_6Symbol_Symbol *, int __pyx_skip_dispatch);
  int (*wordCount)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, int __pyx_skip_dispatch);
  int (*isLeaf)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  int (*isDummyNode)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*moveLeft)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*moveRight)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*ancestorString)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch);
  PyObject *(*constituentSpanList)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *__pyx_vtabptr_9ParseTree_9ParseNode_ParseNode;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* pop_index.proto */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix);
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix);
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (likely(PyList_CheckExact(L) && __Pyx_fits_Py_ssize_t(ix, type, is_signed))) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    __Pyx_fits_Py_ssize_t(ix, type, is_signed) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#else
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)\
    __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
        __Pyx__PyObject_PopIndex(L, py_ix))
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyUnicodeContains.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_ContainsTF(PyObject* substring, PyObject* text, int eq) {
    int result = PyUnicode_Contains(text, substring);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* None.proto */
static CYTHON_INLINE Py_ssize_t __Pyx_mod_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode___searchHeadChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v_priorityList, PyObject *__pyx_v_direction, int __pyx_v_defaultCase, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_headLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_headChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_addChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch, struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_correctParents(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_removeXNodes(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_setChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_index, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_removeChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_leafCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCountWithMultipleChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_numberOfChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_getChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_i, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_firstChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_lastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isLastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_getChildIndex(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isDescendant(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_previousSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_nextSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_getParent(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_f_9ParseTree_9ParseNode_9ParseNode_getData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_setData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_wordCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_excludeStopWords, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isDummyNode(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_moveLeft(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_moveRight(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_ancestorString(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_constituentSpanList(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_startIndex, PyObject *__pyx_v_constituentSpanList, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'Dictionary.Word' */
static PyTypeObject *__pyx_ptype_10Dictionary_4Word_Word = 0;

/* Module declarations from 'ParseTree.Symbol' */
static PyTypeObject *__pyx_ptype_9ParseTree_6Symbol_Symbol = 0;

/* Module declarations from 'ParseTree.ConstituentSpan' */
static PyTypeObject *__pyx_ptype_9ParseTree_15ConstituentSpan_ConstituentSpan = 0;

/* Module declarations from 'ParseTree.ParseNode' */
static PyTypeObject *__pyx_ptype_9ParseTree_9ParseNode_ParseNode = 0;
static PyObject *__pyx_f_9ParseTree_9ParseNode___pyx_unpickle_ParseNode__set_state(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "ParseTree.ParseNode"
extern int __pyx_module_is_main_ParseTree__ParseNode;
int __pyx_module_is_main_ParseTree__ParseNode = 0;

/* Implementation of 'ParseTree.ParseNode' */
static PyObject *__pyx_builtin_range;
static const char __pyx_k_[] = "";
static const char __pyx_k_0[] = "0";
static const char __pyx_k_S[] = "S";
static const char __pyx_k_X[] = "X";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_s[] = "'s";
static const char __pyx_k_CC[] = "CC";
static const char __pyx_k_CD[] = "CD";
static const char __pyx_k_DT[] = "DT";
static const char __pyx_k_EX[] = "EX";
static const char __pyx_k_FW[] = "FW";
static const char __pyx_k_IN[] = "IN";
static const char __pyx_k_JJ[] = "JJ";
static const char __pyx_k_LS[] = "LS";
static const char __pyx_k_MD[] = "MD";
static const char __pyx_k_NN[] = "NN";
static const char __pyx_k_NP[] = "NP";
static const char __pyx_k_NX[] = "NX";
static const char __pyx_k_PP[] = "PP";
static const char __pyx_k_QP[] = "QP";
static const char __pyx_k_RB[] = "RB";
static const char __pyx_k_RP[] = "RP";
static const char __pyx_k_SQ[] = "SQ";
static const char __pyx_k_TO[] = "TO";
static const char __pyx_k_VB[] = "VB";
static const char __pyx_k_VP[] = "VP";
static const char __pyx_k_WP[] = "WP";
static const char __pyx_k__2[] = " ";
static const char __pyx_k__3[] = ")";
static const char __pyx_k__4[] = "(";
static const char __pyx_k__5[] = "*";
static const char __pyx_k__6[] = "$";
static const char __pyx_k__7[] = ") ";
static const char __pyx_k__8[] = ":";
static const char __pyx_k_an[] = "an";
static const char __pyx_k_as[] = "as";
static const char __pyx_k_at[] = "at";
static const char __pyx_k_be[] = "be";
static const char __pyx_k_by[] = "by";
static const char __pyx_k_do[] = "do";
static const char __pyx_k_in[] = "in";
static const char __pyx_k_is[] = "is";
static const char __pyx_k_of[] = "of";
static const char __pyx_k_on[] = "on";
static const char __pyx_k_or[] = "or";
static const char __pyx_k_re[] = "'re";
static const char __pyx_k_to[] = "to";
static const char __pyx_k_up[] = "up";
static const char __pyx_k_JJR[] = "JJR";
static const char __pyx_k_JJS[] = "JJS";
static const char __pyx_k_LST[] = "LST";
static const char __pyx_k_NAC[] = "NAC";
static const char __pyx_k_NCD[] = "NCD";
static const char __pyx_k_NNP[] = "NNP";
static const char __pyx_k_NNS[] = "NNS";
static const char __pyx_k_NP1[] = "NP1";
static const char __pyx_k_NP2[] = "NP2";
static const char __pyx_k_NP3[] = "NP3";
static const char __pyx_k_NP4[] = "NP4";
static const char __pyx_k_NP5[] = "NP5";
static const char __pyx_k_POS[] = "POS";
static const char __pyx_k_PRN[] = "PRN";
static const char __pyx_k_PRP[] = "PRP";
static const char __pyx_k_PRT[] = "PRT";
static const char __pyx_k_RBR[] = "RBR";
static const char __pyx_k_RBS[] = "RBS";
static const char __pyx_k_RRC[] = "RRC";
static const char __pyx_k_UCP[] = "UCP";
static const char __pyx_k_VBD[] = "VBD";
static const char __pyx_k_VBG[] = "VBG";
static const char __pyx_k_VBN[] = "VBN";
static const char __pyx_k_VBP[] = "VBP";
static const char __pyx_k_VBZ[] = "VBZ";
static const char __pyx_k_WDT[] = "WDT";
static const char __pyx_k_WRB[] = "WRB";
static const char __pyx_k_and[] = "and";
static const char __pyx_k_are[] = "are";
static const char __pyx_k_can[] = "can";
static const char __pyx_k_few[] = "few";
static const char __pyx_k_for[] = "for";
static const char __pyx_k_had[] = "had";
static const char __pyx_k_has[] = "has";
static const char __pyx_k_may[] = "may";
static const char __pyx_k_n_t[] = "n't";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_not[] = "not";
static const char __pyx_k_off[] = "off";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_str[] = "__str__";
static const char __pyx_k_the[] = "the";
static const char __pyx_k_was[] = "was";
static const char __pyx_k_ADJP[] = "ADJP";
static const char __pyx_k_ADVP[] = "ADVP";
static const char __pyx_k_FRAG[] = "FRAG";
static const char __pyx_k_INTJ[] = "INTJ";
static const char __pyx_k_LEFT[] = "LEFT";
static const char __pyx_k_NNPS[] = "NNPS";
static const char __pyx_k_NONE[] = "-NONE-";
static const char __pyx_k_SBAR[] = "SBAR";
static const char __pyx_k_SINV[] = "SINV";
static const char __pyx_k_WHNP[] = "WHNP";
static const char __pyx_k_WHPP[] = "WHPP";
static const char __pyx_k_WP_2[] = "WP$";
static const char __pyx_k_been[] = "been";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_down[] = "down";
static const char __pyx_k_have[] = "have";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_than[] = "than";
static const char __pyx_k_were[] = "were";
static const char __pyx_k_will[] = "will";
static const char __pyx_k_with[] = "with";
static const char __pyx_k_CONJP[] = "CONJP";
static const char __pyx_k_RIGHT[] = "RIGHT";
static const char __pyx_k_SBARQ[] = "SBARQ";
static const char __pyx_k_child[] = "child";
static const char __pyx_k_could[] = "could";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_might[] = "might";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_strip[] = "strip";
static const char __pyx_k_there[] = "there";
static const char __pyx_k_would[] = "would";
static const char __pyx_k_WHADJP[] = "WHADJP";
static const char __pyx_k_WHADVP[] = "WHADVP";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_isLeaf[] = "isLeaf";
static const char __pyx_k_parent[] = "parent";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_remove[] = "remove";
static const char __pyx_k_rindex[] = "rindex";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_getData[] = "getData";
static const char __pyx_k_getName[] = "getName";
static const char __pyx_k_setData[] = "setData";
static const char __pyx_k_addChild[] = "addChild";
static const char __pyx_k_children[] = "children";
static const char __pyx_k_getChild[] = "getChild";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_headLeaf[] = "headLeaf";
static const char __pyx_k_moveLeft[] = "moveLeft";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setChild[] = "setChild";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_ParseNode[] = "ParseNode";
static const char __pyx_k_direction[] = "direction";
static const char __pyx_k_getParent[] = "getParent";
static const char __pyx_k_headChild[] = "headChild";
static const char __pyx_k_lastChild[] = "lastChild";
static const char __pyx_k_leafCount[] = "leafCount";
static const char __pyx_k_moveRight[] = "moveRight";
static const char __pyx_k_nodeCount[] = "nodeCount";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_wordCount[] = "wordCount";
static const char __pyx_k_firstChild[] = "firstChild";
static const char __pyx_k_leftOrLine[] = "leftOrLine";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_startIndex[] = "startIndex";
static const char __pyx_k_startswith[] = "startswith";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_defaultCase[] = "defaultCase";
static const char __pyx_k_isDummyNode[] = "isDummyNode";
static const char __pyx_k_isLastChild[] = "isLastChild";
static const char __pyx_k_nextSibling[] = "nextSibling";
static const char __pyx_k_removeChild[] = "removeChild";
static const char __pyx_k_dataOrParent[] = "dataOrParent";
static const char __pyx_k_isDescendant[] = "isDescendant";
static const char __pyx_k_priorityList[] = "priorityList";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_removeXNodes[] = "removeXNodes";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_getChildIndex[] = "getChildIndex";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_rightOrIsLeaf[] = "rightOrIsLeaf";
static const char __pyx_k_ancestorString[] = "ancestorString";
static const char __pyx_k_correctParents[] = "correctParents";
static const char __pyx_k_previousSibling[] = "previousSibling";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_searchHeadChild[] = "__searchHeadChild";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_numberOfChildren[] = "numberOfChildren";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_ParseTree_ParseNode[] = "ParseTree.ParseNode";
static const char __pyx_k_SearchDirectionType[] = "SearchDirectionType";
static const char __pyx_k_constituentSpanList[] = "constituentSpanList";
static const char __pyx_k_isPunctuationSymbol[] = "isPunctuationSymbol";
static const char __pyx_k_pyx_unpickle_ParseNode[] = "__pyx_unpickle_ParseNode";
static const char __pyx_k_ParseTree_SearchDirectionType[] = "ParseTree.SearchDirectionType";
static const char __pyx_k_nodeCountWithMultipleChildren[] = "nodeCountWithMultipleChildren";
static const char __pyx_k_Incompatible_checksums_s_vs_0x67[] = "Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_kp_u_0;
static PyObject *__pyx_n_s_ADJP;
static PyObject *__pyx_n_u_ADJP;
static PyObject *__pyx_n_s_ADVP;
static PyObject *__pyx_n_u_ADVP;
static PyObject *__pyx_n_u_CC;
static PyObject *__pyx_n_u_CD;
static PyObject *__pyx_n_s_CONJP;
static PyObject *__pyx_n_u_CONJP;
static PyObject *__pyx_n_u_DT;
static PyObject *__pyx_n_u_EX;
static PyObject *__pyx_n_s_FRAG;
static PyObject *__pyx_n_u_FRAG;
static PyObject *__pyx_n_u_FW;
static PyObject *__pyx_n_u_IN;
static PyObject *__pyx_n_s_INTJ;
static PyObject *__pyx_n_u_INTJ;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x67;
static PyObject *__pyx_n_u_JJ;
static PyObject *__pyx_n_u_JJR;
static PyObject *__pyx_n_u_JJS;
static PyObject *__pyx_n_s_LEFT;
static PyObject *__pyx_n_u_LS;
static PyObject *__pyx_n_s_LST;
static PyObject *__pyx_n_u_LST;
static PyObject *__pyx_n_u_MD;
static PyObject *__pyx_n_s_NAC;
static PyObject *__pyx_n_u_NAC;
static PyObject *__pyx_n_u_NCD;
static PyObject *__pyx_n_u_NN;
static PyObject *__pyx_n_u_NNP;
static PyObject *__pyx_n_u_NNPS;
static PyObject *__pyx_n_u_NNS;
static PyObject *__pyx_kp_u_NONE;
static PyObject *__pyx_n_u_NP;
static PyObject *__pyx_n_s_NP1;
static PyObject *__pyx_n_s_NP2;
static PyObject *__pyx_n_s_NP3;
static PyObject *__pyx_n_s_NP4;
static PyObject *__pyx_n_s_NP5;
static PyObject *__pyx_n_u_NX;
static PyObject *__pyx_n_u_POS;
static PyObject *__pyx_n_s_PP;
static PyObject *__pyx_n_u_PP;
static PyObject *__pyx_n_s_PRN;
static PyObject *__pyx_n_u_PRN;
static PyObject *__pyx_n_u_PRP;
static PyObject *__pyx_n_s_PRT;
static PyObject *__pyx_n_u_PRT;
static PyObject *__pyx_n_s_ParseNode;
static PyObject *__pyx_n_s_ParseTree_ParseNode;
static PyObject *__pyx_n_s_ParseTree_SearchDirectionType;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_QP;
static PyObject *__pyx_n_u_QP;
static PyObject *__pyx_n_u_RB;
static PyObject *__pyx_n_u_RBR;
static PyObject *__pyx_n_u_RBS;
static PyObject *__pyx_n_s_RIGHT;
static PyObject *__pyx_n_u_RP;
static PyObject *__pyx_n_s_RRC;
static PyObject *__pyx_n_u_RRC;
static PyObject *__pyx_n_s_S;
static PyObject *__pyx_n_u_S;
static PyObject *__pyx_n_s_SBAR;
static PyObject *__pyx_n_u_SBAR;
static PyObject *__pyx_n_s_SBARQ;
static PyObject *__pyx_n_u_SBARQ;
static PyObject *__pyx_n_s_SINV;
static PyObject *__pyx_n_u_SINV;
static PyObject *__pyx_n_s_SQ;
static PyObject *__pyx_n_u_SQ;
static PyObject *__pyx_n_s_SearchDirectionType;
static PyObject *__pyx_n_u_TO;
static PyObject *__pyx_n_s_UCP;
static PyObject *__pyx_n_u_UCP;
static PyObject *__pyx_n_u_VB;
static PyObject *__pyx_n_u_VBD;
static PyObject *__pyx_n_u_VBG;
static PyObject *__pyx_n_u_VBN;
static PyObject *__pyx_n_u_VBP;
static PyObject *__pyx_n_u_VBZ;
static PyObject *__pyx_n_s_VP;
static PyObject *__pyx_n_u_VP;
static PyObject *__pyx_n_u_WDT;
static PyObject *__pyx_n_s_WHADJP;
static PyObject *__pyx_n_u_WHADJP;
static PyObject *__pyx_n_s_WHADVP;
static PyObject *__pyx_n_u_WHADVP;
static PyObject *__pyx_n_s_WHNP;
static PyObject *__pyx_n_u_WHNP;
static PyObject *__pyx_n_s_WHPP;
static PyObject *__pyx_n_u_WHPP;
static PyObject *__pyx_n_u_WP;
static PyObject *__pyx_kp_u_WP_2;
static PyObject *__pyx_n_u_WRB;
static PyObject *__pyx_n_u_X;
static PyObject *__pyx_kp_u__2;
static PyObject *__pyx_kp_u__3;
static PyObject *__pyx_kp_u__4;
static PyObject *__pyx_kp_u__5;
static PyObject *__pyx_kp_u__6;
static PyObject *__pyx_kp_u__7;
static PyObject *__pyx_kp_u__8;
static PyObject *__pyx_n_u_a;
static PyObject *__pyx_n_s_addChild;
static PyObject *__pyx_n_u_an;
static PyObject *__pyx_n_s_ancestorString;
static PyObject *__pyx_n_u_and;
static PyObject *__pyx_n_u_are;
static PyObject *__pyx_n_u_as;
static PyObject *__pyx_n_u_at;
static PyObject *__pyx_n_u_be;
static PyObject *__pyx_n_u_been;
static PyObject *__pyx_n_u_by;
static PyObject *__pyx_n_u_can;
static PyObject *__pyx_n_s_child;
static PyObject *__pyx_n_s_children;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_constituentSpanList;
static PyObject *__pyx_n_s_correctParents;
static PyObject *__pyx_n_u_could;
static PyObject *__pyx_n_s_dataOrParent;
static PyObject *__pyx_n_s_defaultCase;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_direction;
static PyObject *__pyx_n_u_do;
static PyObject *__pyx_n_u_down;
static PyObject *__pyx_n_u_few;
static PyObject *__pyx_n_s_firstChild;
static PyObject *__pyx_n_u_for;
static PyObject *__pyx_n_s_getChild;
static PyObject *__pyx_n_s_getChildIndex;
static PyObject *__pyx_n_s_getData;
static PyObject *__pyx_n_s_getName;
static PyObject *__pyx_n_s_getParent;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_u_had;
static PyObject *__pyx_n_u_has;
static PyObject *__pyx_n_u_have;
static PyObject *__pyx_n_s_headChild;
static PyObject *__pyx_n_s_headLeaf;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_u_in;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_u_is;
static PyObject *__pyx_n_s_isDescendant;
static PyObject *__pyx_n_s_isDummyNode;
static PyObject *__pyx_n_s_isLastChild;
static PyObject *__pyx_n_s_isLeaf;
static PyObject *__pyx_n_s_isPunctuationSymbol;
static PyObject *__pyx_n_s_lastChild;
static PyObject *__pyx_n_s_leafCount;
static PyObject *__pyx_n_s_leftOrLine;
static PyObject *__pyx_n_s_lower;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_u_may;
static PyObject *__pyx_n_u_might;
static PyObject *__pyx_n_s_moveLeft;
static PyObject *__pyx_n_s_moveRight;
static PyObject *__pyx_kp_u_n_t;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_nextSibling;
static PyObject *__pyx_n_s_nodeCount;
static PyObject *__pyx_n_s_nodeCountWithMultipleChildren;
static PyObject *__pyx_n_u_not;
static PyObject *__pyx_n_s_numberOfChildren;
static PyObject *__pyx_n_u_of;
static PyObject *__pyx_n_u_off;
static PyObject *__pyx_n_u_on;
static PyObject *__pyx_n_u_or;
static PyObject *__pyx_n_s_parent;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_previousSibling;
static PyObject *__pyx_n_s_priorityList;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_ParseNode;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_kp_u_re;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_remove;
static PyObject *__pyx_n_s_removeChild;
static PyObject *__pyx_n_s_removeXNodes;
static PyObject *__pyx_n_s_rightOrIsLeaf;
static PyObject *__pyx_n_s_rindex;
static PyObject *__pyx_kp_u_s;
static PyObject *__pyx_n_s_searchHeadChild;
static PyObject *__pyx_n_s_setChild;
static PyObject *__pyx_n_s_setData;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_startIndex;
static PyObject *__pyx_n_s_startswith;
static PyObject *__pyx_n_s_str;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_u_than;
static PyObject *__pyx_n_u_the;
static PyObject *__pyx_n_u_there;
static PyObject *__pyx_n_u_to;
static PyObject *__pyx_n_u_up;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_u_was;
static PyObject *__pyx_n_u_were;
static PyObject *__pyx_n_u_will;
static PyObject *__pyx_n_u_with;
static PyObject *__pyx_n_s_wordCount;
static PyObject *__pyx_n_u_would;
static int __pyx_pf_9ParseTree_9ParseNode_9ParseNode___init__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v_dataOrParent, PyObject *__pyx_v_leftOrLine, PyObject *__pyx_v_rightOrIsLeaf); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_2__searchHeadChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v_priorityList, PyObject *__pyx_v_direction, int __pyx_v_defaultCase); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_4headLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_6headChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_8addChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_10correctParents(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_12removeXNodes(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_14setChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_index, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_16removeChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_18leafCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_20nodeCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_22nodeCountWithMultipleChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_24numberOfChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_26getChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_i); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_28firstChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_30lastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_32isLastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_34getChildIndex(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_36isDescendant(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_38previousSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_40nextSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_42getParent(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_44getData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_46setData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_48wordCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_excludeStopWords); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_50isLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_52isDummyNode(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_54__str__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_56moveLeft(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_58moveRight(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_60ancestorString(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_62constituentSpanList(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_startIndex, PyObject *__pyx_v_constituentSpanList); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_64__reduce_cython__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_66__setstate_cython__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9ParseTree_9ParseNode___pyx_unpickle_ParseNode(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_9ParseTree_9ParseNode_ParseNode(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyUnicode_Type_strip = {0, &__pyx_n_s_strip, 0, 0, 0};
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_108795736;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_codeobj__10;
/* Late includes */

/* "ParseTree/ParseNode.pyx":40
 *     NP5 = ["JJ", "JJS", "RB", "QP"]
 * 
 *     def __init__(self, dataOrParent, leftOrLine=None, rightOrIsLeaf=None):             # <<<<<<<<<<<<<<
 *         """
 *         Another simple constructor for ParseNode. It takes inputs left and right children of this node, and the data.
 */

/* Python wrapper */
static int __pyx_pw_9ParseTree_9ParseNode_9ParseNode_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode___init__[] = "\n        Another simple constructor for ParseNode. It takes inputs left and right children of this node, and the data.\n        Sets the corresponding attributes with these inputs.\n\n        PARAMETERS\n        ----------\n        dataOrParent\n            Data for this node.\n        leftOrLine\n            Left child of this node.\n        rightOrIsLeaf\n            Right child of this node.\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode___init__;
#endif
static int __pyx_pw_9ParseTree_9ParseNode_9ParseNode_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_dataOrParent = 0;
  PyObject *__pyx_v_leftOrLine = 0;
  PyObject *__pyx_v_rightOrIsLeaf = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_dataOrParent,&__pyx_n_s_leftOrLine,&__pyx_n_s_rightOrIsLeaf,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dataOrParent)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_leftOrLine);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rightOrIsLeaf);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 40, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_dataOrParent = values[0];
    __pyx_v_leftOrLine = values[1];
    __pyx_v_rightOrIsLeaf = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 40, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode___init__(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), __pyx_v_dataOrParent, __pyx_v_leftOrLine, __pyx_v_rightOrIsLeaf);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9ParseTree_9ParseNode_9ParseNode___init__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v_dataOrParent, PyObject *__pyx_v_leftOrLine, PyObject *__pyx_v_rightOrIsLeaf) {
  int __pyx_v_parenthesisCount;
  int __pyx_v_i;
  PyObject *__pyx_v_childLine = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  long __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ParseTree/ParseNode.pyx":56
 *         cdef int parenthesisCount,i
 *         cdef str childLine
 *         self.children = []             # <<<<<<<<<<<<<<
 *         self.parent = None
 *         self.data = None
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->children);
  __Pyx_DECREF(__pyx_v_self->children);
  __pyx_v_self->children = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ParseTree/ParseNode.pyx":57
 *         cdef str childLine
 *         self.children = []
 *         self.parent = None             # <<<<<<<<<<<<<<
 *         self.data = None
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->parent);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->parent));
  __pyx_v_self->parent = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None);

  /* "ParseTree/ParseNode.pyx":58
 *         self.children = []
 *         self.parent = None
 *         self.data = None             # <<<<<<<<<<<<<<
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,
 *                                                                                                  ParseNode):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data));
  __pyx_v_self->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)Py_None);

  /* "ParseTree/ParseNode.pyx":59
 *         self.parent = None
 *         self.data = None
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,             # <<<<<<<<<<<<<<
 *                                                                                                  ParseNode):
 *             self.data = dataOrParent
 */
  __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_dataOrParent, __pyx_ptype_9ParseTree_6Symbol_Symbol); 
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_v_leftOrLine, __pyx_ptype_9ParseTree_9ParseNode_ParseNode); 
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }

  /* "ParseTree/ParseNode.pyx":60
 *         self.data = None
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,
 *                                                                                                  ParseNode):             # <<<<<<<<<<<<<<
 *             self.data = dataOrParent
 *             if rightOrIsLeaf is not None:
 */
  __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_rightOrIsLeaf, __pyx_ptype_9ParseTree_9ParseNode_ParseNode); 

  /* "ParseTree/ParseNode.pyx":59
 *         self.parent = None
 *         self.data = None
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,             # <<<<<<<<<<<<<<
 *                                                                                                  ParseNode):
 *             self.data = dataOrParent
 */
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ParseTree/ParseNode.pyx":61
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,
 *                                                                                                  ParseNode):
 *             self.data = dataOrParent             # <<<<<<<<<<<<<<
 *             if rightOrIsLeaf is not None:
 *                 self.children.append(leftOrLine)
 */
    if (!(likely(((__pyx_v_dataOrParent) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dataOrParent, __pyx_ptype_9ParseTree_6Symbol_Symbol))))) __PYX_ERR(0, 61, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_dataOrParent;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->data);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->data));
    __pyx_v_self->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":62
 *                                                                                                  ParseNode):
 *             self.data = dataOrParent
 *             if rightOrIsLeaf is not None:             # <<<<<<<<<<<<<<
 *                 self.children.append(leftOrLine)
 *                 leftOrLine.parent = self
 */
    __pyx_t_2 = (__pyx_v_rightOrIsLeaf != Py_None);
    __pyx_t_4 = (__pyx_t_2 != 0);
    if (__pyx_t_4) {

      /* "ParseTree/ParseNode.pyx":63
 *             self.data = dataOrParent
 *             if rightOrIsLeaf is not None:
 *                 self.children.append(leftOrLine)             # <<<<<<<<<<<<<<
 *                 leftOrLine.parent = self
 *                 self.children.append(rightOrIsLeaf)
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 63, __pyx_L1_error)
      }
      __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->children, __pyx_v_leftOrLine); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 63, __pyx_L1_error)

      /* "ParseTree/ParseNode.pyx":64
 *             if rightOrIsLeaf is not None:
 *                 self.children.append(leftOrLine)
 *                 leftOrLine.parent = self             # <<<<<<<<<<<<<<
 *                 self.children.append(rightOrIsLeaf)
 *                 rightOrIsLeaf.parent = self
 */
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_parent, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 64, __pyx_L1_error)

      /* "ParseTree/ParseNode.pyx":65
 *                 self.children.append(leftOrLine)
 *                 leftOrLine.parent = self
 *                 self.children.append(rightOrIsLeaf)             # <<<<<<<<<<<<<<
 *                 rightOrIsLeaf.parent = self
 *             else:
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 65, __pyx_L1_error)
      }
      __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->children, __pyx_v_rightOrIsLeaf); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 65, __pyx_L1_error)

      /* "ParseTree/ParseNode.pyx":66
 *                 leftOrLine.parent = self
 *                 self.children.append(rightOrIsLeaf)
 *                 rightOrIsLeaf.parent = self             # <<<<<<<<<<<<<<
 *             else:
 *                 if leftOrLine is not None:
 */
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_rightOrIsLeaf, __pyx_n_s_parent, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 66, __pyx_L1_error)

      /* "ParseTree/ParseNode.pyx":62
 *                                                                                                  ParseNode):
 *             self.data = dataOrParent
 *             if rightOrIsLeaf is not None:             # <<<<<<<<<<<<<<
 *                 self.children.append(leftOrLine)
 *                 leftOrLine.parent = self
 */
      goto __pyx_L7;
    }

    /* "ParseTree/ParseNode.pyx":68
 *                 rightOrIsLeaf.parent = self
 *             else:
 *                 if leftOrLine is not None:             # <<<<<<<<<<<<<<
 *                     self.children.append(leftOrLine)
 *                     leftOrLine.parent = self
 */
    /*else*/ {
      __pyx_t_4 = (__pyx_v_leftOrLine != Py_None);
      __pyx_t_2 = (__pyx_t_4 != 0);
      if (__pyx_t_2) {

        /* "ParseTree/ParseNode.pyx":69
 *             else:
 *                 if leftOrLine is not None:
 *                     self.children.append(leftOrLine)             # <<<<<<<<<<<<<<
 *                     leftOrLine.parent = self
 *         elif (isinstance(dataOrParent, ParseNode) or dataOrParent is None) and isinstance(leftOrLine, str) and isinstance(rightOrIsLeaf, bool):
 */
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
          __PYX_ERR(0, 69, __pyx_L1_error)
        }
        __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->children, __pyx_v_leftOrLine); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 69, __pyx_L1_error)

        /* "ParseTree/ParseNode.pyx":70
 *                 if leftOrLine is not None:
 *                     self.children.append(leftOrLine)
 *                     leftOrLine.parent = self             # <<<<<<<<<<<<<<
 *         elif (isinstance(dataOrParent, ParseNode) or dataOrParent is None) and isinstance(leftOrLine, str) and isinstance(rightOrIsLeaf, bool):
 *             parenthesisCount = 0
 */
        if (__Pyx_PyObject_SetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_parent, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 70, __pyx_L1_error)

        /* "ParseTree/ParseNode.pyx":68
 *                 rightOrIsLeaf.parent = self
 *             else:
 *                 if leftOrLine is not None:             # <<<<<<<<<<<<<<
 *                     self.children.append(leftOrLine)
 *                     leftOrLine.parent = self
 */
      }
    }
    __pyx_L7:;

    /* "ParseTree/ParseNode.pyx":59
 *         self.parent = None
 *         self.data = None
 *         if isinstance(dataOrParent, Symbol) and isinstance(leftOrLine, ParseNode) and isinstance(rightOrIsLeaf,             # <<<<<<<<<<<<<<
 *                                                                                                  ParseNode):
 *             self.data = dataOrParent
 */
    goto __pyx_L3;
  }

  /* "ParseTree/ParseNode.pyx":71
 *                     self.children.append(leftOrLine)
 *                     leftOrLine.parent = self
 *         elif (isinstance(dataOrParent, ParseNode) or dataOrParent is None) and isinstance(leftOrLine, str) and isinstance(rightOrIsLeaf, bool):             # <<<<<<<<<<<<<<
 *             parenthesisCount = 0
 *             childLine = ""
 */
  __pyx_t_4 = __Pyx_TypeCheck(__pyx_v_dataOrParent, __pyx_ptype_9ParseTree_9ParseNode_ParseNode); 
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (!__pyx_t_3) {
  } else {
    goto __pyx_L10_next_and;
  }
  __pyx_t_3 = (__pyx_v_dataOrParent == Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_L10_next_and:;
  __pyx_t_4 = PyUnicode_Check(__pyx_v_leftOrLine); 
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_1 = ((PyObject*)&PyBool_Type);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_3 = PyObject_IsInstance(__pyx_v_rightOrIsLeaf, __pyx_t_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ParseTree/ParseNode.pyx":72
 *                     leftOrLine.parent = self
 *         elif (isinstance(dataOrParent, ParseNode) or dataOrParent is None) and isinstance(leftOrLine, str) and isinstance(rightOrIsLeaf, bool):
 *             parenthesisCount = 0             # <<<<<<<<<<<<<<
 *             childLine = ""
 *             self.parent = dataOrParent
 */
    __pyx_v_parenthesisCount = 0;

    /* "ParseTree/ParseNode.pyx":73
 *         elif (isinstance(dataOrParent, ParseNode) or dataOrParent is None) and isinstance(leftOrLine, str) and isinstance(rightOrIsLeaf, bool):
 *             parenthesisCount = 0
 *             childLine = ""             # <<<<<<<<<<<<<<
 *             self.parent = dataOrParent
 *             if rightOrIsLeaf:
 */
    __Pyx_INCREF(__pyx_kp_u_);
    __pyx_v_childLine = __pyx_kp_u_;

    /* "ParseTree/ParseNode.pyx":74
 *             parenthesisCount = 0
 *             childLine = ""
 *             self.parent = dataOrParent             # <<<<<<<<<<<<<<
 *             if rightOrIsLeaf:
 *                 self.data = Symbol(leftOrLine)
 */
    if (!(likely(((__pyx_v_dataOrParent) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dataOrParent, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 74, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_dataOrParent;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->parent);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->parent));
    __pyx_v_self->parent = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":75
 *             childLine = ""
 *             self.parent = dataOrParent
 *             if rightOrIsLeaf:             # <<<<<<<<<<<<<<
 *                 self.data = Symbol(leftOrLine)
 *             else:
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_rightOrIsLeaf); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 75, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "ParseTree/ParseNode.pyx":76
 *             self.parent = dataOrParent
 *             if rightOrIsLeaf:
 *                 self.data = Symbol(leftOrLine)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.data = Symbol(leftOrLine[1: leftOrLine.index(" ")])
 */
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_9ParseTree_6Symbol_Symbol), __pyx_v_leftOrLine); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->data);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->data));
      __pyx_v_self->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "ParseTree/ParseNode.pyx":75
 *             childLine = ""
 *             self.parent = dataOrParent
 *             if rightOrIsLeaf:             # <<<<<<<<<<<<<<
 *                 self.data = Symbol(leftOrLine)
 *             else:
 */
      goto __pyx_L13;
    }

    /* "ParseTree/ParseNode.pyx":78
 *                 self.data = Symbol(leftOrLine)
 *             else:
 *                 self.data = Symbol(leftOrLine[1: leftOrLine.index(" ")])             # <<<<<<<<<<<<<<
 *                 if leftOrLine.index(")") == leftOrLine.rindex(")"):
 *                     self.children.append(ParseNode(self, leftOrLine[leftOrLine.index(" ") + 1: leftOrLine.index(")")],
 */
    /*else*/ {
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_kp_u__2) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_u__2);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetSlice(__pyx_v_leftOrLine, 1, 0, NULL, &__pyx_t_1, NULL, 1, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_9ParseTree_6Symbol_Symbol), __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->data);
      __Pyx_DECREF(((PyObject *)__pyx_v_self->data));
      __pyx_v_self->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "ParseTree/ParseNode.pyx":79
 *             else:
 *                 self.data = Symbol(leftOrLine[1: leftOrLine.index(" ")])
 *                 if leftOrLine.index(")") == leftOrLine.rindex(")"):             # <<<<<<<<<<<<<<
 *                     self.children.append(ParseNode(self, leftOrLine[leftOrLine.index(" ") + 1: leftOrLine.index(")")],
 *                                                    True))
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_7, __pyx_kp_u__3) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_u__3);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_rindex); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      __pyx_t_6 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_kp_u__3) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u__3);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyObject_RichCompare(__pyx_t_1, __pyx_t_6, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 79, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (__pyx_t_2) {

        /* "ParseTree/ParseNode.pyx":80
 *                 self.data = Symbol(leftOrLine[1: leftOrLine.index(" ")])
 *                 if leftOrLine.index(")") == leftOrLine.rindex(")"):
 *                     self.children.append(ParseNode(self, leftOrLine[leftOrLine.index(" ") + 1: leftOrLine.index(")")],             # <<<<<<<<<<<<<<
 *                                                    True))
 *                 else:
 */
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
          __PYX_ERR(0, 80, __pyx_L1_error)
        }
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_index); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_1 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        __pyx_t_7 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_1, __pyx_kp_u__2) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_kp_u__2);
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = __Pyx_PyInt_AddObjC(__pyx_t_7, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        __pyx_t_7 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_8, __pyx_kp_u__3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u__3);
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_v_leftOrLine, 0, 0, &__pyx_t_6, &__pyx_t_7, NULL, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "ParseTree/ParseNode.pyx":81
 *                 if leftOrLine.index(")") == leftOrLine.rindex(")"):
 *                     self.children.append(ParseNode(self, leftOrLine[leftOrLine.index(" ") + 1: leftOrLine.index(")")],
 *                                                    True))             # <<<<<<<<<<<<<<
 *                 else:
 *                     for i in range(leftOrLine.index(" ") + 1, len(leftOrLine)):
 */
        __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_INCREF(((PyObject *)__pyx_v_self));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
        PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)__pyx_v_self));
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1);
        __Pyx_INCREF(Py_True);
        __Pyx_GIVEREF(Py_True);
        PyTuple_SET_ITEM(__pyx_t_7, 2, Py_True);
        __pyx_t_1 = 0;

        /* "ParseTree/ParseNode.pyx":80
 *                 self.data = Symbol(leftOrLine[1: leftOrLine.index(" ")])
 *                 if leftOrLine.index(")") == leftOrLine.rindex(")"):
 *                     self.children.append(ParseNode(self, leftOrLine[leftOrLine.index(" ") + 1: leftOrLine.index(")")],             # <<<<<<<<<<<<<<
 *                                                    True))
 *                 else:
 */
        __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode), __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->children, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 80, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "ParseTree/ParseNode.pyx":79
 *             else:
 *                 self.data = Symbol(leftOrLine[1: leftOrLine.index(" ")])
 *                 if leftOrLine.index(")") == leftOrLine.rindex(")"):             # <<<<<<<<<<<<<<
 *                     self.children.append(ParseNode(self, leftOrLine[leftOrLine.index(" ") + 1: leftOrLine.index(")")],
 *                                                    True))
 */
        goto __pyx_L14;
      }

      /* "ParseTree/ParseNode.pyx":83
 *                                                    True))
 *                 else:
 *                     for i in range(leftOrLine.index(" ") + 1, len(leftOrLine)):             # <<<<<<<<<<<<<<
 *                         if leftOrLine[i] != " " or parenthesisCount > 0:
 *                             childLine = childLine + leftOrLine[i]
 */
      /*else*/ {
        __pyx_t_9 = PyObject_Length(__pyx_v_leftOrLine); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 83, __pyx_L1_error)
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_leftOrLine, __pyx_n_s_index); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 83, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_6, __pyx_kp_u__2) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u__2);
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 83, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_7); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 83, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_11 = __pyx_t_9;
        for (__pyx_t_12 = __pyx_t_10; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
          __pyx_v_i = __pyx_t_12;

          /* "ParseTree/ParseNode.pyx":84
 *                 else:
 *                     for i in range(leftOrLine.index(" ") + 1, len(leftOrLine)):
 *                         if leftOrLine[i] != " " or parenthesisCount > 0:             # <<<<<<<<<<<<<<
 *                             childLine = childLine + leftOrLine[i]
 *                         if leftOrLine[i] == "(":
 */
          __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_leftOrLine, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 84, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_t_7, __pyx_kp_u__2, Py_NE)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 84, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (!__pyx_t_4) {
          } else {
            __pyx_t_2 = __pyx_t_4;
            goto __pyx_L18_bool_binop_done;
          }
          __pyx_t_4 = ((__pyx_v_parenthesisCount > 0) != 0);
          __pyx_t_2 = __pyx_t_4;
          __pyx_L18_bool_binop_done:;
          if (__pyx_t_2) {

            /* "ParseTree/ParseNode.pyx":85
 *                     for i in range(leftOrLine.index(" ") + 1, len(leftOrLine)):
 *                         if leftOrLine[i] != " " or parenthesisCount > 0:
 *                             childLine = childLine + leftOrLine[i]             # <<<<<<<<<<<<<<
 *                         if leftOrLine[i] == "(":
 *                             parenthesisCount = parenthesisCount + 1
 */
            __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_leftOrLine, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 85, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_1 = PyNumber_Add(__pyx_v_childLine, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 85, __pyx_L1_error)
            __Pyx_DECREF_SET(__pyx_v_childLine, ((PyObject*)__pyx_t_1));
            __pyx_t_1 = 0;

            /* "ParseTree/ParseNode.pyx":84
 *                 else:
 *                     for i in range(leftOrLine.index(" ") + 1, len(leftOrLine)):
 *                         if leftOrLine[i] != " " or parenthesisCount > 0:             # <<<<<<<<<<<<<<
 *                             childLine = childLine + leftOrLine[i]
 *                         if leftOrLine[i] == "(":
 */
          }

          /* "ParseTree/ParseNode.pyx":86
 *                         if leftOrLine[i] != " " or parenthesisCount > 0:
 *                             childLine = childLine + leftOrLine[i]
 *                         if leftOrLine[i] == "(":             # <<<<<<<<<<<<<<
 *                             parenthesisCount = parenthesisCount + 1
 *                         elif leftOrLine[i] == ")":
 */
          __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_leftOrLine, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u__4, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 86, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (__pyx_t_2) {

            /* "ParseTree/ParseNode.pyx":87
 *                             childLine = childLine + leftOrLine[i]
 *                         if leftOrLine[i] == "(":
 *                             parenthesisCount = parenthesisCount + 1             # <<<<<<<<<<<<<<
 *                         elif leftOrLine[i] == ")":
 *                             parenthesisCount = parenthesisCount - 1
 */
            __pyx_v_parenthesisCount = (__pyx_v_parenthesisCount + 1);

            /* "ParseTree/ParseNode.pyx":86
 *                         if leftOrLine[i] != " " or parenthesisCount > 0:
 *                             childLine = childLine + leftOrLine[i]
 *                         if leftOrLine[i] == "(":             # <<<<<<<<<<<<<<
 *                             parenthesisCount = parenthesisCount + 1
 *                         elif leftOrLine[i] == ")":
 */
            goto __pyx_L20;
          }

          /* "ParseTree/ParseNode.pyx":88
 *                         if leftOrLine[i] == "(":
 *                             parenthesisCount = parenthesisCount + 1
 *                         elif leftOrLine[i] == ")":             # <<<<<<<<<<<<<<
 *                             parenthesisCount = parenthesisCount - 1
 *                         if parenthesisCount == 0 and len(childLine) != 0:
 */
          __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_leftOrLine, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u__3, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 88, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (__pyx_t_2) {

            /* "ParseTree/ParseNode.pyx":89
 *                             parenthesisCount = parenthesisCount + 1
 *                         elif leftOrLine[i] == ")":
 *                             parenthesisCount = parenthesisCount - 1             # <<<<<<<<<<<<<<
 *                         if parenthesisCount == 0 and len(childLine) != 0:
 *                             self.children.append(ParseNode(self, childLine.strip(), False))
 */
            __pyx_v_parenthesisCount = (__pyx_v_parenthesisCount - 1);

            /* "ParseTree/ParseNode.pyx":88
 *                         if leftOrLine[i] == "(":
 *                             parenthesisCount = parenthesisCount + 1
 *                         elif leftOrLine[i] == ")":             # <<<<<<<<<<<<<<
 *                             parenthesisCount = parenthesisCount - 1
 *                         if parenthesisCount == 0 and len(childLine) != 0:
 */
          }
          __pyx_L20:;

          /* "ParseTree/ParseNode.pyx":90
 *                         elif leftOrLine[i] == ")":
 *                             parenthesisCount = parenthesisCount - 1
 *                         if parenthesisCount == 0 and len(childLine) != 0:             # <<<<<<<<<<<<<<
 *                             self.children.append(ParseNode(self, childLine.strip(), False))
 *                             childLine = ""
 */
          __pyx_t_4 = ((__pyx_v_parenthesisCount == 0) != 0);
          if (__pyx_t_4) {
          } else {
            __pyx_t_2 = __pyx_t_4;
            goto __pyx_L22_bool_binop_done;
          }
          if (unlikely(__pyx_v_childLine == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
            __PYX_ERR(0, 90, __pyx_L1_error)
          }
          __pyx_t_13 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_childLine); if (unlikely(__pyx_t_13 == ((Py_ssize_t)-1))) __PYX_ERR(0, 90, __pyx_L1_error)
          __pyx_t_4 = ((__pyx_t_13 != 0) != 0);
          __pyx_t_2 = __pyx_t_4;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {

            /* "ParseTree/ParseNode.pyx":91
 *                             parenthesisCount = parenthesisCount - 1
 *                         if parenthesisCount == 0 and len(childLine) != 0:
 *                             self.children.append(ParseNode(self, childLine.strip(), False))             # <<<<<<<<<<<<<<
 *                             childLine = ""
 * 
 */
            if (unlikely(__pyx_v_self->children == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
              __PYX_ERR(0, 91, __pyx_L1_error)
            }
            __pyx_t_1 = __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyUnicode_Type_strip, __pyx_v_childLine); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 91, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_INCREF(((PyObject *)__pyx_v_self));
            __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
            PyTuple_SET_ITEM(__pyx_t_7, 0, ((PyObject *)__pyx_v_self));
            __Pyx_GIVEREF(__pyx_t_1);
            PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_1);
            __Pyx_INCREF(Py_False);
            __Pyx_GIVEREF(Py_False);
            PyTuple_SET_ITEM(__pyx_t_7, 2, Py_False);
            __pyx_t_1 = 0;
            __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode), __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->children, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 91, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "ParseTree/ParseNode.pyx":92
 *                         if parenthesisCount == 0 and len(childLine) != 0:
 *                             self.children.append(ParseNode(self, childLine.strip(), False))
 *                             childLine = ""             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode __searchHeadChild(self, list priorityList, object direction, bint defaultCase):
 */
            __Pyx_INCREF(__pyx_kp_u_);
            __Pyx_DECREF_SET(__pyx_v_childLine, __pyx_kp_u_);

            /* "ParseTree/ParseNode.pyx":90
 *                         elif leftOrLine[i] == ")":
 *                             parenthesisCount = parenthesisCount - 1
 *                         if parenthesisCount == 0 and len(childLine) != 0:             # <<<<<<<<<<<<<<
 *                             self.children.append(ParseNode(self, childLine.strip(), False))
 *                             childLine = ""
 */
          }
        }
      }
      __pyx_L14:;
    }
    __pyx_L13:;

    /* "ParseTree/ParseNode.pyx":71
 *                     self.children.append(leftOrLine)
 *                     leftOrLine.parent = self
 *         elif (isinstance(dataOrParent, ParseNode) or dataOrParent is None) and isinstance(leftOrLine, str) and isinstance(rightOrIsLeaf, bool):             # <<<<<<<<<<<<<<
 *             parenthesisCount = 0
 *             childLine = ""
 */
  }
  __pyx_L3:;

  /* "ParseTree/ParseNode.pyx":40
 *     NP5 = ["JJ", "JJS", "RB", "QP"]
 * 
 *     def __init__(self, dataOrParent, leftOrLine=None, rightOrIsLeaf=None):             # <<<<<<<<<<<<<<
 *         """
 *         Another simple constructor for ParseNode. It takes inputs left and right children of this node, and the data.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_childLine);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":94
 *                             childLine = ""
 * 
 *     cpdef ParseNode __searchHeadChild(self, list priorityList, object direction, bint defaultCase):             # <<<<<<<<<<<<<<
 *         """
 *         Extracts the head of the children of this current node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_3__searchHeadChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode___searchHeadChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v_priorityList, PyObject *__pyx_v_direction, int __pyx_v_defaultCase, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  PyObject *__pyx_v_item = 0;
  int __pyx_v_j;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__searchHeadChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_searchHeadChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_3__searchHeadChild)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_defaultCase); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 94, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_priorityList, __pyx_v_direction, __pyx_t_3};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_priorityList, __pyx_v_direction, __pyx_t_3};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 3+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(3+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 94, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(__pyx_v_priorityList);
          __Pyx_GIVEREF(__pyx_v_priorityList);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_priorityList);
          __Pyx_INCREF(__pyx_v_direction);
          __Pyx_GIVEREF(__pyx_v_direction);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_direction);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 94, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":117
 *         cdef str item
 *         cdef int j
 *         if direction == SearchDirectionType.LEFT:             # <<<<<<<<<<<<<<
 *             for item in priorityList:
 *                 for child in self.children:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_direction, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_8) {

    /* "ParseTree/ParseNode.pyx":118
 *         cdef int j
 *         if direction == SearchDirectionType.LEFT:
 *             for item in priorityList:             # <<<<<<<<<<<<<<
 *                 for child in self.children:
 *                     if child.getData().trimSymbol().getName() == item:
 */
    if (unlikely(__pyx_v_priorityList == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 118, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_priorityList; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
    for (;;) {
      if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 118, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 118, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_item, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "ParseTree/ParseNode.pyx":119
 *         if direction == SearchDirectionType.LEFT:
 *             for item in priorityList:
 *                 for child in self.children:             # <<<<<<<<<<<<<<
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 119, __pyx_L1_error)
      }
      __pyx_t_2 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_2); __pyx_t_10 = 0;
      for (;;) {
        if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_10); __Pyx_INCREF(__pyx_t_4); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 119, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 119, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
        if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 119, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_4));
        __pyx_t_4 = 0;

        /* "ParseTree/ParseNode.pyx":120
 *             for item in priorityList:
 *                 for child in self.children:
 *                     if child.getData().trimSymbol().getName() == item:             # <<<<<<<<<<<<<<
 *                         return child
 *             if defaultCase:
 */
        __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->getData(__pyx_v_child, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_7 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_4)->__pyx_base.__pyx_vtab)->trimSymbol(((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_4), 0)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_7)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_7), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_8 = (__Pyx_PyUnicode_Equals(__pyx_t_4, __pyx_v_item, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_11 = (__pyx_t_8 != 0);
        if (__pyx_t_11) {

          /* "ParseTree/ParseNode.pyx":121
 *                 for child in self.children:
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child             # <<<<<<<<<<<<<<
 *             if defaultCase:
 *                 return self.firstChild()
 */
          __Pyx_XDECREF(((PyObject *)__pyx_r));
          __Pyx_INCREF(((PyObject *)__pyx_v_child));
          __pyx_r = __pyx_v_child;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L0;

          /* "ParseTree/ParseNode.pyx":120
 *             for item in priorityList:
 *                 for child in self.children:
 *                     if child.getData().trimSymbol().getName() == item:             # <<<<<<<<<<<<<<
 *                         return child
 *             if defaultCase:
 */
        }

        /* "ParseTree/ParseNode.pyx":119
 *         if direction == SearchDirectionType.LEFT:
 *             for item in priorityList:
 *                 for child in self.children:             # <<<<<<<<<<<<<<
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 */
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "ParseTree/ParseNode.pyx":118
 *         cdef int j
 *         if direction == SearchDirectionType.LEFT:
 *             for item in priorityList:             # <<<<<<<<<<<<<<
 *                 for child in self.children:
 *                     if child.getData().trimSymbol().getName() == item:
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":122
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 *             if defaultCase:             # <<<<<<<<<<<<<<
 *                 return self.firstChild()
 *         elif direction == SearchDirectionType.RIGHT:
 */
    __pyx_t_11 = (__pyx_v_defaultCase != 0);
    if (__pyx_t_11) {

      /* "ParseTree/ParseNode.pyx":123
 *                         return child
 *             if defaultCase:
 *                 return self.firstChild()             # <<<<<<<<<<<<<<
 *         elif direction == SearchDirectionType.RIGHT:
 *             for item in priorityList:
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->firstChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
      __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":122
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 *             if defaultCase:             # <<<<<<<<<<<<<<
 *                 return self.firstChild()
 *         elif direction == SearchDirectionType.RIGHT:
 */
    }

    /* "ParseTree/ParseNode.pyx":117
 *         cdef str item
 *         cdef int j
 *         if direction == SearchDirectionType.LEFT:             # <<<<<<<<<<<<<<
 *             for item in priorityList:
 *                 for child in self.children:
 */
    goto __pyx_L3;
  }

  /* "ParseTree/ParseNode.pyx":124
 *             if defaultCase:
 *                 return self.firstChild()
 *         elif direction == SearchDirectionType.RIGHT:             # <<<<<<<<<<<<<<
 *             for item in priorityList:
 *                 for j in range(len(self.children), -1, -1):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_direction, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_11) {

    /* "ParseTree/ParseNode.pyx":125
 *                 return self.firstChild()
 *         elif direction == SearchDirectionType.RIGHT:
 *             for item in priorityList:             # <<<<<<<<<<<<<<
 *                 for j in range(len(self.children), -1, -1):
 *                     child = self.children[j]
 */
    if (unlikely(__pyx_v_priorityList == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 125, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_priorityList; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
    for (;;) {
      if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 125, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 125, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_item, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "ParseTree/ParseNode.pyx":126
 *         elif direction == SearchDirectionType.RIGHT:
 *             for item in priorityList:
 *                 for j in range(len(self.children), -1, -1):             # <<<<<<<<<<<<<<
 *                     child = self.children[j]
 *                     if child.getData().trimSymbol().getName() == item:
 */
      __pyx_t_2 = __pyx_v_self->children;
      __Pyx_INCREF(__pyx_t_2);
      if (unlikely(__pyx_t_2 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 126, __pyx_L1_error)
      }
      __pyx_t_10 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      for (__pyx_t_6 = __pyx_t_10; __pyx_t_6 > -1; __pyx_t_6-=1) {
        __pyx_v_j = __pyx_t_6;

        /* "ParseTree/ParseNode.pyx":127
 *             for item in priorityList:
 *                 for j in range(len(self.children), -1, -1):
 *                     child = self.children[j]             # <<<<<<<<<<<<<<
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 */
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 127, __pyx_L1_error)
        }
        __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_j, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 127, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "ParseTree/ParseNode.pyx":128
 *                 for j in range(len(self.children), -1, -1):
 *                     child = self.children[j]
 *                     if child.getData().trimSymbol().getName() == item:             # <<<<<<<<<<<<<<
 *                         return child
 *             if defaultCase:
 */
        __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->getData(__pyx_v_child, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_2)->__pyx_base.__pyx_vtab)->trimSymbol(((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_2), 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_4)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_4), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_v_item, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 128, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_8 = (__pyx_t_11 != 0);
        if (__pyx_t_8) {

          /* "ParseTree/ParseNode.pyx":129
 *                     child = self.children[j]
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child             # <<<<<<<<<<<<<<
 *             if defaultCase:
 *                 return self.lastChild()
 */
          __Pyx_XDECREF(((PyObject *)__pyx_r));
          __Pyx_INCREF(((PyObject *)__pyx_v_child));
          __pyx_r = __pyx_v_child;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L0;

          /* "ParseTree/ParseNode.pyx":128
 *                 for j in range(len(self.children), -1, -1):
 *                     child = self.children[j]
 *                     if child.getData().trimSymbol().getName() == item:             # <<<<<<<<<<<<<<
 *                         return child
 *             if defaultCase:
 */
        }
      }

      /* "ParseTree/ParseNode.pyx":125
 *                 return self.firstChild()
 *         elif direction == SearchDirectionType.RIGHT:
 *             for item in priorityList:             # <<<<<<<<<<<<<<
 *                 for j in range(len(self.children), -1, -1):
 *                     child = self.children[j]
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":130
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 *             if defaultCase:             # <<<<<<<<<<<<<<
 *                 return self.lastChild()
 *         return None
 */
    __pyx_t_8 = (__pyx_v_defaultCase != 0);
    if (__pyx_t_8) {

      /* "ParseTree/ParseNode.pyx":131
 *                         return child
 *             if defaultCase:
 *                 return self.lastChild()             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->lastChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
      __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":130
 *                     if child.getData().trimSymbol().getName() == item:
 *                         return child
 *             if defaultCase:             # <<<<<<<<<<<<<<
 *                 return self.lastChild()
 *         return None
 */
    }

    /* "ParseTree/ParseNode.pyx":124
 *             if defaultCase:
 *                 return self.firstChild()
 *         elif direction == SearchDirectionType.RIGHT:             # <<<<<<<<<<<<<<
 *             for item in priorityList:
 *                 for j in range(len(self.children), -1, -1):
 */
  }
  __pyx_L3:;

  /* "ParseTree/ParseNode.pyx":132
 *             if defaultCase:
 *                 return self.lastChild()
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode headLeaf(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":94
 *                             childLine = ""
 * 
 *     cpdef ParseNode __searchHeadChild(self, list priorityList, object direction, bint defaultCase):             # <<<<<<<<<<<<<<
 *         """
 *         Extracts the head of the children of this current node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__searchHeadChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_3__searchHeadChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_2__searchHeadChild[] = "\n        Extracts the head of the children of this current node.\n\n        PARAMETERS\n        ----------\n        priorityList : list\n            Depending on the pos of current node, the priorities among the children are given with this parameter\n        direction : SearchDirectionType\n            Depending on the pos of the current node, search direction is either from left to right, or from right to\n            left.\n        defaultCase : bool\n            If true, and no child appears in the priority list, returns first child on the left, or first child on the\n            right depending on the search direction.\n\n        RETURNS\n        -------\n        ParseNode\n            Head node of the children of the current node\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_3__searchHeadChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_priorityList = 0;
  PyObject *__pyx_v_direction = 0;
  int __pyx_v_defaultCase;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__searchHeadChild (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_priorityList,&__pyx_n_s_direction,&__pyx_n_s_defaultCase,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_priorityList)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_direction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__searchHeadChild", 1, 3, 3, 1); __PYX_ERR(0, 94, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_defaultCase)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__searchHeadChild", 1, 3, 3, 2); __PYX_ERR(0, 94, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__searchHeadChild") < 0)) __PYX_ERR(0, 94, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_priorityList = ((PyObject*)values[0]);
    __pyx_v_direction = values[1];
    __pyx_v_defaultCase = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_defaultCase == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 94, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__searchHeadChild", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 94, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__searchHeadChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_priorityList), (&PyList_Type), 1, "priorityList", 1))) __PYX_ERR(0, 94, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_2__searchHeadChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), __pyx_v_priorityList, __pyx_v_direction, __pyx_v_defaultCase);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_2__searchHeadChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v_priorityList, PyObject *__pyx_v_direction, int __pyx_v_defaultCase) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__searchHeadChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode___searchHeadChild(__pyx_v_self, __pyx_v_priorityList, __pyx_v_direction, __pyx_v_defaultCase, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__searchHeadChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":134
 *         return None
 * 
 *     cpdef ParseNode headLeaf(self):             # <<<<<<<<<<<<<<
 *         """
 *         If current node is not a leaf, it has one or more children, this method determines recursively the head of
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_5headLeaf(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_headLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_head = 0;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("headLeaf", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_headLeaf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_5headLeaf)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 134, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 134, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":146
 *         """
 *         cdef ParseNode head
 *         if len(self.children) > 0:             # <<<<<<<<<<<<<<
 *             head = self.headChild()
 *             if head is not None:
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 146, __pyx_L1_error)
  }
  __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = ((__pyx_t_5 > 0) != 0);
  if (__pyx_t_6) {

    /* "ParseTree/ParseNode.pyx":147
 *         cdef ParseNode head
 *         if len(self.children) > 0:
 *             head = self.headChild()             # <<<<<<<<<<<<<<
 *             if head is not None:
 *                 return head.headLeaf()
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->headChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_head = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":148
 *         if len(self.children) > 0:
 *             head = self.headChild()
 *             if head is not None:             # <<<<<<<<<<<<<<
 *                 return head.headLeaf()
 *             else:
 */
    __pyx_t_6 = (((PyObject *)__pyx_v_head) != Py_None);
    __pyx_t_7 = (__pyx_t_6 != 0);
    if (__pyx_t_7) {

      /* "ParseTree/ParseNode.pyx":149
 *             head = self.headChild()
 *             if head is not None:
 *                 return head.headLeaf()             # <<<<<<<<<<<<<<
 *             else:
 *                 return None
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_head->__pyx_vtab)->headLeaf(__pyx_v_head, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
      __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":148
 *         if len(self.children) > 0:
 *             head = self.headChild()
 *             if head is not None:             # <<<<<<<<<<<<<<
 *                 return head.headLeaf()
 *             else:
 */
    }

    /* "ParseTree/ParseNode.pyx":151
 *                 return head.headLeaf()
 *             else:
 *                 return None             # <<<<<<<<<<<<<<
 *         else:
 *             return self
 */
    /*else*/ {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); __Pyx_INCREF(Py_None);
      goto __pyx_L0;
    }

    /* "ParseTree/ParseNode.pyx":146
 *         """
 *         cdef ParseNode head
 *         if len(self.children) > 0:             # <<<<<<<<<<<<<<
 *             head = self.headChild()
 *             if head is not None:
 */
  }

  /* "ParseTree/ParseNode.pyx":153
 *                 return None
 *         else:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode headChild(self):
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = __pyx_v_self;
    goto __pyx_L0;
  }

  /* "ParseTree/ParseNode.pyx":134
 *         return None
 * 
 *     cpdef ParseNode headLeaf(self):             # <<<<<<<<<<<<<<
 *         """
 *         If current node is not a leaf, it has one or more children, this method determines recursively the head of
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.headLeaf", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_head);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_5headLeaf(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_4headLeaf[] = "\n        If current node is not a leaf, it has one or more children, this method determines recursively the head of\n        that (those) child(ren). Otherwise, it returns itself. In this way, this method returns the head of all leaf\n        successors.\n\n        RETURNS\n        -------\n        ParseNode\n            Head node of the descendant leaves of this current node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_5headLeaf(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("headLeaf (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_4headLeaf(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_4headLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("headLeaf", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_headLeaf(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.headLeaf", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":155
 *             return self
 * 
 *     cpdef ParseNode headChild(self):             # <<<<<<<<<<<<<<
 *         """
 *         Calls searchHeadChild to determine the head node of all children of this current node. The search direction and
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_7headChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_headChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_result = 0;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("headChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_headChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_7headChild)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 155, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 155, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":166
 *         """
 *         cdef ParseNode result
 *         if self.data.trimSymbol().__str__() == "ADJP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.ADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "ADVP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_ADJP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":167
 *         cdef ParseNode result
 *         if self.data.trimSymbol().__str__() == "ADJP":
 *             return self.__searchHeadChild(self.ADJP, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "ADVP":
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ADJP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":166
 *         """
 *         cdef ParseNode result
 *         if self.data.trimSymbol().__str__() == "ADJP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.ADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "ADVP":
 */
  }

  /* "ParseTree/ParseNode.pyx":168
 *         if self.data.trimSymbol().__str__() == "ADJP":
 *             return self.__searchHeadChild(self.ADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "ADVP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "CONJP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_ADVP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":169
 *             return self.__searchHeadChild(self.ADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "ADVP":
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "CONJP":
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ADVP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":168
 *         if self.data.trimSymbol().__str__() == "ADJP":
 *             return self.__searchHeadChild(self.ADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "ADVP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "CONJP":
 */
  }

  /* "ParseTree/ParseNode.pyx":170
 *         elif self.data.trimSymbol().__str__() == "ADVP":
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "CONJP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "FRAG":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_CONJP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":171
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "CONJP":
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "FRAG":
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_CONJP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":170
 *         elif self.data.trimSymbol().__str__() == "ADVP":
 *             return self.__searchHeadChild(self.ADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "CONJP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "FRAG":
 */
  }

  /* "ParseTree/ParseNode.pyx":172
 *         elif self.data.trimSymbol().__str__() == "CONJP":
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "FRAG":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "INTJ":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_FRAG, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":173
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "FRAG":
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "INTJ":
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_FRAG); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":172
 *         elif self.data.trimSymbol().__str__() == "CONJP":
 *             return self.__searchHeadChild(self.CONJP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "FRAG":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "INTJ":
 */
  }

  /* "ParseTree/ParseNode.pyx":174
 *         elif self.data.trimSymbol().__str__() == "FRAG":
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "INTJ":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "LST":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_INTJ, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":175
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "INTJ":
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "LST":
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_INTJ); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":174
 *         elif self.data.trimSymbol().__str__() == "FRAG":
 *             return self.__searchHeadChild(self.FRAG, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "INTJ":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "LST":
 */
  }

  /* "ParseTree/ParseNode.pyx":176
 *         elif self.data.trimSymbol().__str__() == "INTJ":
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "LST":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NAC":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_LST, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":177
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "LST":
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "NAC":
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_LST); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":176
 *         elif self.data.trimSymbol().__str__() == "INTJ":
 *             return self.__searchHeadChild(self.INTJ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "LST":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NAC":
 */
  }

  /* "ParseTree/ParseNode.pyx":178
 *         elif self.data.trimSymbol().__str__() == "LST":
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NAC":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_NAC, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":179
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NAC":
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "PP":
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_NAC); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":178
 *         elif self.data.trimSymbol().__str__() == "LST":
 *             return self.__searchHeadChild(self.LST, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NAC":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PP":
 */
  }

  /* "ParseTree/ParseNode.pyx":180
 *         elif self.data.trimSymbol().__str__() == "NAC":
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "PRN":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_PP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":181
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PP":
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "PRN":
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_PP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":180
 *         elif self.data.trimSymbol().__str__() == "NAC":
 *             return self.__searchHeadChild(self.NAC, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "PRN":
 */
  }

  /* "ParseTree/ParseNode.pyx":182
 *         elif self.data.trimSymbol().__str__() == "PP":
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "PRN":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PRT":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_PRN, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":183
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "PRN":
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "PRT":
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_PRN); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":182
 *         elif self.data.trimSymbol().__str__() == "PP":
 *             return self.__searchHeadChild(self.PP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "PRN":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PRT":
 */
  }

  /* "ParseTree/ParseNode.pyx":184
 *         elif self.data.trimSymbol().__str__() == "PRN":
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PRT":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "QP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_PRT, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":185
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PRT":
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "QP":
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_PRT); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":184
 *         elif self.data.trimSymbol().__str__() == "PRN":
 *             return self.__searchHeadChild(self.PRN, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "PRT":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "QP":
 */
  }

  /* "ParseTree/ParseNode.pyx":186
 *         elif self.data.trimSymbol().__str__() == "PRT":
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "QP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "RRC":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_QP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":187
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "QP":
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "RRC":
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_QP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":186
 *         elif self.data.trimSymbol().__str__() == "PRT":
 *             return self.__searchHeadChild(self.PRT, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "QP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "RRC":
 */
  }

  /* "ParseTree/ParseNode.pyx":188
 *         elif self.data.trimSymbol().__str__() == "QP":
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "RRC":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "S":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_RRC, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":189
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "RRC":
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "S":
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_RRC); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":188
 *         elif self.data.trimSymbol().__str__() == "QP":
 *             return self.__searchHeadChild(self.QP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "RRC":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "S":
 */
  }

  /* "ParseTree/ParseNode.pyx":190
 *         elif self.data.trimSymbol().__str__() == "RRC":
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "S":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBAR":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_S, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":191
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "S":
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "SBAR":
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_S); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":190
 *         elif self.data.trimSymbol().__str__() == "RRC":
 *             return self.__searchHeadChild(self.RRC, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "S":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBAR":
 */
  }

  /* "ParseTree/ParseNode.pyx":192
 *         elif self.data.trimSymbol().__str__() == "S":
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBAR":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBARQ":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_SBAR, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":193
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBAR":
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "SBARQ":
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_SBAR); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":192
 *         elif self.data.trimSymbol().__str__() == "S":
 *             return self.__searchHeadChild(self.S, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBAR":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBARQ":
 */
  }

  /* "ParseTree/ParseNode.pyx":194
 *         elif self.data.trimSymbol().__str__() == "SBAR":
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBARQ":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SINV":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_SBARQ, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":195
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBARQ":
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "SINV":
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_SBARQ); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":194
 *         elif self.data.trimSymbol().__str__() == "SBAR":
 *             return self.__searchHeadChild(self.SBAR, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SBARQ":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SINV":
 */
  }

  /* "ParseTree/ParseNode.pyx":196
 *         elif self.data.trimSymbol().__str__() == "SBARQ":
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SINV":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SQ":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_SINV, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":197
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SINV":
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "SQ":
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_SINV); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":196
 *         elif self.data.trimSymbol().__str__() == "SBARQ":
 *             return self.__searchHeadChild(self.SBARQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SINV":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SQ":
 */
  }

  /* "ParseTree/ParseNode.pyx":198
 *         elif self.data.trimSymbol().__str__() == "SINV":
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SQ":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "UCP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_SQ, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":199
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SQ":
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "UCP":
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_SQ); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":198
 *         elif self.data.trimSymbol().__str__() == "SINV":
 *             return self.__searchHeadChild(self.SINV, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "SQ":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "UCP":
 */
  }

  /* "ParseTree/ParseNode.pyx":200
 *         elif self.data.trimSymbol().__str__() == "SQ":
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "UCP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "VP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_UCP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":201
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "UCP":
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "VP":
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_UCP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":200
 *         elif self.data.trimSymbol().__str__() == "SQ":
 *             return self.__searchHeadChild(self.SQ, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "UCP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "VP":
 */
  }

  /* "ParseTree/ParseNode.pyx":202
 *         elif self.data.trimSymbol().__str__() == "UCP":
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "VP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADJP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_VP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":203
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "VP":
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "WHADJP":
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_VP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":202
 *         elif self.data.trimSymbol().__str__() == "UCP":
 *             return self.__searchHeadChild(self.UCP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "VP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADJP":
 */
  }

  /* "ParseTree/ParseNode.pyx":204
 *         elif self.data.trimSymbol().__str__() == "VP":
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADJP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADVP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_WHADJP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":205
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADJP":
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "WHADVP":
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_WHADJP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":204
 *         elif self.data.trimSymbol().__str__() == "VP":
 *             return self.__searchHeadChild(self.VP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADJP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADVP":
 */
  }

  /* "ParseTree/ParseNode.pyx":206
 *         elif self.data.trimSymbol().__str__() == "WHADJP":
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADVP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "WHNP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_WHADVP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":207
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADVP":
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "WHNP":
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_WHADVP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":206
 *         elif self.data.trimSymbol().__str__() == "WHADJP":
 *             return self.__searchHeadChild(self.WHADJP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHADVP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "WHNP":
 */
  }

  /* "ParseTree/ParseNode.pyx":208
 *         elif self.data.trimSymbol().__str__() == "WHADVP":
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "WHNP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHPP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_WHNP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":209
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "WHNP":
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "WHPP":
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_WHNP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":208
 *         elif self.data.trimSymbol().__str__() == "WHADVP":
 *             return self.__searchHeadChild(self.WHADVP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "WHNP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHPP":
 */
  }

  /* "ParseTree/ParseNode.pyx":210
 *         elif self.data.trimSymbol().__str__() == "WHNP":
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHPP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NP":
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_WHPP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":211
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHPP":
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)             # <<<<<<<<<<<<<<
 *         elif self.data.trimSymbol().__str__() == "NP":
 *             if self.lastChild().getData().getName() == "POS":
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_WHPP); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_2, 1, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":210
 *         elif self.data.trimSymbol().__str__() == "WHNP":
 *             return self.__searchHeadChild(self.WHNP, SearchDirectionType.LEFT, True)
 *         elif self.data.trimSymbol().__str__() == "WHPP":             # <<<<<<<<<<<<<<
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NP":
 */
  }

  /* "ParseTree/ParseNode.pyx":212
 *         elif self.data.trimSymbol().__str__() == "WHPP":
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NP":             # <<<<<<<<<<<<<<
 *             if self.lastChild().getData().getName() == "POS":
 *                 return self.lastChild()
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->trimSymbol(__pyx_v_self->data, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_NP, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "ParseTree/ParseNode.pyx":213
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NP":
 *             if self.lastChild().getData().getName() == "POS":             # <<<<<<<<<<<<<<
 *                 return self.lastChild()
 *             else:
 */
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->lastChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3)->__pyx_vtab)->getData(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_1), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_t_3, __pyx_n_u_POS, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {

      /* "ParseTree/ParseNode.pyx":214
 *         elif self.data.trimSymbol().__str__() == "NP":
 *             if self.lastChild().getData().getName() == "POS":
 *                 return self.lastChild()             # <<<<<<<<<<<<<<
 *             else:
 *                 result = self.__searchHeadChild(self.NP1, SearchDirectionType.RIGHT, False)
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->lastChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3);
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":213
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NP":
 *             if self.lastChild().getData().getName() == "POS":             # <<<<<<<<<<<<<<
 *                 return self.lastChild()
 *             else:
 */
    }

    /* "ParseTree/ParseNode.pyx":216
 *                 return self.lastChild()
 *             else:
 *                 result = self.__searchHeadChild(self.NP1, SearchDirectionType.RIGHT, False)             # <<<<<<<<<<<<<<
 *                 if result is not None:
 *                     return result
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_NP1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_result = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "ParseTree/ParseNode.pyx":217
 *             else:
 *                 result = self.__searchHeadChild(self.NP1, SearchDirectionType.RIGHT, False)
 *                 if result is not None:             # <<<<<<<<<<<<<<
 *                     return result
 *                 else:
 */
      __pyx_t_6 = (((PyObject *)__pyx_v_result) != Py_None);
      __pyx_t_5 = (__pyx_t_6 != 0);
      if (__pyx_t_5) {

        /* "ParseTree/ParseNode.pyx":218
 *                 result = self.__searchHeadChild(self.NP1, SearchDirectionType.RIGHT, False)
 *                 if result is not None:
 *                     return result             # <<<<<<<<<<<<<<
 *                 else:
 *                     result = self.__searchHeadChild(self.NP2, SearchDirectionType.LEFT, False)
 */
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(((PyObject *)__pyx_v_result));
        __pyx_r = __pyx_v_result;
        goto __pyx_L0;

        /* "ParseTree/ParseNode.pyx":217
 *             else:
 *                 result = self.__searchHeadChild(self.NP1, SearchDirectionType.RIGHT, False)
 *                 if result is not None:             # <<<<<<<<<<<<<<
 *                     return result
 *                 else:
 */
      }

      /* "ParseTree/ParseNode.pyx":220
 *                     return result
 *                 else:
 *                     result = self.__searchHeadChild(self.NP2, SearchDirectionType.LEFT, False)             # <<<<<<<<<<<<<<
 *                     if result is not None:
 *                         return result
 */
      /*else*/ {
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_NP2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_LEFT); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_3, 0, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "ParseTree/ParseNode.pyx":221
 *                 else:
 *                     result = self.__searchHeadChild(self.NP2, SearchDirectionType.LEFT, False)
 *                     if result is not None:             # <<<<<<<<<<<<<<
 *                         return result
 *                     else:
 */
        __pyx_t_5 = (((PyObject *)__pyx_v_result) != Py_None);
        __pyx_t_6 = (__pyx_t_5 != 0);
        if (__pyx_t_6) {

          /* "ParseTree/ParseNode.pyx":222
 *                     result = self.__searchHeadChild(self.NP2, SearchDirectionType.LEFT, False)
 *                     if result is not None:
 *                         return result             # <<<<<<<<<<<<<<
 *                     else:
 *                         result = self.__searchHeadChild(self.NP3, SearchDirectionType.RIGHT, False)
 */
          __Pyx_XDECREF(((PyObject *)__pyx_r));
          __Pyx_INCREF(((PyObject *)__pyx_v_result));
          __pyx_r = __pyx_v_result;
          goto __pyx_L0;

          /* "ParseTree/ParseNode.pyx":221
 *                 else:
 *                     result = self.__searchHeadChild(self.NP2, SearchDirectionType.LEFT, False)
 *                     if result is not None:             # <<<<<<<<<<<<<<
 *                         return result
 *                     else:
 */
        }

        /* "ParseTree/ParseNode.pyx":224
 *                         return result
 *                     else:
 *                         result = self.__searchHeadChild(self.NP3, SearchDirectionType.RIGHT, False)             # <<<<<<<<<<<<<<
 *                         if result is not None:
 *                             return result
 */
        /*else*/ {
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_NP3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 224, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 224, __pyx_L1_error)
          __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 224, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_2), __pyx_t_1, 0, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_3));
          __pyx_t_3 = 0;

          /* "ParseTree/ParseNode.pyx":225
 *                     else:
 *                         result = self.__searchHeadChild(self.NP3, SearchDirectionType.RIGHT, False)
 *                         if result is not None:             # <<<<<<<<<<<<<<
 *                             return result
 *                         else:
 */
          __pyx_t_6 = (((PyObject *)__pyx_v_result) != Py_None);
          __pyx_t_5 = (__pyx_t_6 != 0);
          if (__pyx_t_5) {

            /* "ParseTree/ParseNode.pyx":226
 *                         result = self.__searchHeadChild(self.NP3, SearchDirectionType.RIGHT, False)
 *                         if result is not None:
 *                             return result             # <<<<<<<<<<<<<<
 *                         else:
 *                             result = self.__searchHeadChild(self.NP4, SearchDirectionType.RIGHT, False)
 */
            __Pyx_XDECREF(((PyObject *)__pyx_r));
            __Pyx_INCREF(((PyObject *)__pyx_v_result));
            __pyx_r = __pyx_v_result;
            goto __pyx_L0;

            /* "ParseTree/ParseNode.pyx":225
 *                     else:
 *                         result = self.__searchHeadChild(self.NP3, SearchDirectionType.RIGHT, False)
 *                         if result is not None:             # <<<<<<<<<<<<<<
 *                             return result
 *                         else:
 */
          }

          /* "ParseTree/ParseNode.pyx":228
 *                             return result
 *                         else:
 *                             result = self.__searchHeadChild(self.NP4, SearchDirectionType.RIGHT, False)             # <<<<<<<<<<<<<<
 *                             if result is not None:
 *                                 return result
 */
          /*else*/ {
            __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_NP4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 228, __pyx_L1_error)
            __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 228, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_3), __pyx_t_2, 0, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1));
            __pyx_t_1 = 0;

            /* "ParseTree/ParseNode.pyx":229
 *                         else:
 *                             result = self.__searchHeadChild(self.NP4, SearchDirectionType.RIGHT, False)
 *                             if result is not None:             # <<<<<<<<<<<<<<
 *                                 return result
 *                             else:
 */
            __pyx_t_5 = (((PyObject *)__pyx_v_result) != Py_None);
            __pyx_t_6 = (__pyx_t_5 != 0);
            if (__pyx_t_6) {

              /* "ParseTree/ParseNode.pyx":230
 *                             result = self.__searchHeadChild(self.NP4, SearchDirectionType.RIGHT, False)
 *                             if result is not None:
 *                                 return result             # <<<<<<<<<<<<<<
 *                             else:
 *                                 result = self.__searchHeadChild(self.NP5, SearchDirectionType.RIGHT, False)
 */
              __Pyx_XDECREF(((PyObject *)__pyx_r));
              __Pyx_INCREF(((PyObject *)__pyx_v_result));
              __pyx_r = __pyx_v_result;
              goto __pyx_L0;

              /* "ParseTree/ParseNode.pyx":229
 *                         else:
 *                             result = self.__searchHeadChild(self.NP4, SearchDirectionType.RIGHT, False)
 *                             if result is not None:             # <<<<<<<<<<<<<<
 *                                 return result
 *                             else:
 */
            }

            /* "ParseTree/ParseNode.pyx":232
 *                                 return result
 *                             else:
 *                                 result = self.__searchHeadChild(self.NP5, SearchDirectionType.RIGHT, False)             # <<<<<<<<<<<<<<
 *                                 if result is not None:
 *                                     return result
 */
            /*else*/ {
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_NP5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_1);
              if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 232, __pyx_L1_error)
              __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 232, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_RIGHT); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 232, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->__pyx___searchHeadChild(__pyx_v_self, ((PyObject*)__pyx_t_1), __pyx_t_3, 0, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 232, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF_SET(__pyx_v_result, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
              __pyx_t_2 = 0;

              /* "ParseTree/ParseNode.pyx":233
 *                             else:
 *                                 result = self.__searchHeadChild(self.NP5, SearchDirectionType.RIGHT, False)
 *                                 if result is not None:             # <<<<<<<<<<<<<<
 *                                     return result
 *                                 else:
 */
              __pyx_t_6 = (((PyObject *)__pyx_v_result) != Py_None);
              __pyx_t_5 = (__pyx_t_6 != 0);
              if (__pyx_t_5) {

                /* "ParseTree/ParseNode.pyx":234
 *                                 result = self.__searchHeadChild(self.NP5, SearchDirectionType.RIGHT, False)
 *                                 if result is not None:
 *                                     return result             # <<<<<<<<<<<<<<
 *                                 else:
 *                                     return self.lastChild()
 */
                __Pyx_XDECREF(((PyObject *)__pyx_r));
                __Pyx_INCREF(((PyObject *)__pyx_v_result));
                __pyx_r = __pyx_v_result;
                goto __pyx_L0;

                /* "ParseTree/ParseNode.pyx":233
 *                             else:
 *                                 result = self.__searchHeadChild(self.NP5, SearchDirectionType.RIGHT, False)
 *                                 if result is not None:             # <<<<<<<<<<<<<<
 *                                     return result
 *                                 else:
 */
              }

              /* "ParseTree/ParseNode.pyx":236
 *                                     return result
 *                                 else:
 *                                     return self.lastChild()             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
              /*else*/ {
                __Pyx_XDECREF(((PyObject *)__pyx_r));
                __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->lastChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_2);
                __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
                __pyx_t_2 = 0;
                goto __pyx_L0;
              }
            }
          }
        }
      }
    }

    /* "ParseTree/ParseNode.pyx":212
 *         elif self.data.trimSymbol().__str__() == "WHPP":
 *             return self.__searchHeadChild(self.WHPP, SearchDirectionType.RIGHT, True)
 *         elif self.data.trimSymbol().__str__() == "NP":             # <<<<<<<<<<<<<<
 *             if self.lastChild().getData().getName() == "POS":
 *                 return self.lastChild()
 */
  }

  /* "ParseTree/ParseNode.pyx":237
 *                                 else:
 *                                     return self.lastChild()
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cpdef addChild(self, ParseNode child, index=None):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":155
 *             return self
 * 
 *     cpdef ParseNode headChild(self):             # <<<<<<<<<<<<<<
 *         """
 *         Calls searchHeadChild to determine the head node of all children of this current node. The search direction and
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.headChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_7headChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_6headChild[] = "\n        Calls searchHeadChild to determine the head node of all children of this current node. The search direction and\n        the search priority list is determined according to the symbol in this current parent node.\n\n        RETURNS\n        -------\n        ParseNode\n            Head node among its children of this current node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_7headChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("headChild (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_6headChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_6headChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("headChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_headChild(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.headChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":239
 *         return None
 * 
 *     cpdef addChild(self, ParseNode child, index=None):             # <<<<<<<<<<<<<<
 *         """
 *         Adds a child node at the end of the children node list.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_9addChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_addChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch, struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild *__pyx_optional_args) {
  PyObject *__pyx_v_index = ((PyObject *)Py_None);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("addChild", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_index = __pyx_optional_args->index;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_addChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_9addChild)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_child), __pyx_v_index};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 239, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_child), __pyx_v_index};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 239, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 239, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_child));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_child));
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_child));
          __Pyx_INCREF(__pyx_v_index);
          __Pyx_GIVEREF(__pyx_v_index);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_index);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 239, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":250
 *             Position to be added.
 *         """
 *         if index is None:             # <<<<<<<<<<<<<<
 *             self.children.append(child)
 *         else:
 */
  __pyx_t_7 = (__pyx_v_index == Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "ParseTree/ParseNode.pyx":251
 *         """
 *         if index is None:
 *             self.children.append(child)             # <<<<<<<<<<<<<<
 *         else:
 *             self.children.insert(index, child)
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 251, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_self->children, ((PyObject *)__pyx_v_child)); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 251, __pyx_L1_error)

    /* "ParseTree/ParseNode.pyx":250
 *             Position to be added.
 *         """
 *         if index is None:             # <<<<<<<<<<<<<<
 *             self.children.append(child)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "ParseTree/ParseNode.pyx":253
 *             self.children.append(child)
 *         else:
 *             self.children.insert(index, child)             # <<<<<<<<<<<<<<
 *         child.parent = self
 * 
 */
  /*else*/ {
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "insert");
      __PYX_ERR(0, 253, __pyx_L1_error)
    }
    __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 253, __pyx_L1_error)
    __pyx_t_9 = PyList_Insert(__pyx_v_self->children, __pyx_t_10, ((PyObject *)__pyx_v_child)); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 253, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "ParseTree/ParseNode.pyx":254
 *         else:
 *             self.children.insert(index, child)
 *         child.parent = self             # <<<<<<<<<<<<<<
 * 
 *     cpdef correctParents(self):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_child->parent);
  __Pyx_DECREF(((PyObject *)__pyx_v_child->parent));
  __pyx_v_child->parent = __pyx_v_self;

  /* "ParseTree/ParseNode.pyx":239
 *         return None
 * 
 *     cpdef addChild(self, ParseNode child, index=None):             # <<<<<<<<<<<<<<
 *         """
 *         Adds a child node at the end of the children node list.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.addChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_9addChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_8addChild[] = "\n        Adds a child node at the end of the children node list.\n\n        PARAMETERS\n        ----------\n        child : ParseNode\n            Child node to be added.\n        index\n            Position to be added.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_9addChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  PyObject *__pyx_v_index = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("addChild (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child,&__pyx_n_s_index,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_child)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "addChild") < 0)) __PYX_ERR(0, 239, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_child = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)values[0]);
    __pyx_v_index = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("addChild", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 239, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.addChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "child", 0))) __PYX_ERR(0, 239, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_8addChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), __pyx_v_child, __pyx_v_index);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_8addChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("addChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.index = __pyx_v_index;
  __pyx_t_1 = __pyx_vtabptr_9ParseTree_9ParseNode_ParseNode->addChild(__pyx_v_self, __pyx_v_child, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.addChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":256
 *         child.parent = self
 * 
 *     cpdef correctParents(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to restore the parents of all nodes below this node in the hierarchy.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_11correctParents(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_correctParents(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("correctParents", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correctParents); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_11correctParents)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":261
 *         """
 *         cdef ParseNode child
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.parent = self
 *             child.correctParents()
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 261, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 261, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ParseTree/ParseNode.pyx":262
 *         cdef ParseNode child
 *         for child in self.children:
 *             child.parent = self             # <<<<<<<<<<<<<<
 *             child.correctParents()
 * 
 */
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_child->parent);
    __Pyx_DECREF(((PyObject *)__pyx_v_child->parent));
    __pyx_v_child->parent = __pyx_v_self;

    /* "ParseTree/ParseNode.pyx":263
 *         for child in self.children:
 *             child.parent = self
 *             child.correctParents()             # <<<<<<<<<<<<<<
 * 
 *     cpdef removeXNodes(self):
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->correctParents(__pyx_v_child, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 263, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ParseTree/ParseNode.pyx":261
 *         """
 *         cdef ParseNode child
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.parent = self
 *             child.correctParents()
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ParseTree/ParseNode.pyx":256
 *         child.parent = self
 * 
 *     cpdef correctParents(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to restore the parents of all nodes below this node in the hierarchy.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.correctParents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_11correctParents(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_10correctParents[] = "\n        Recursive method to restore the parents of all nodes below this node in the hierarchy.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_11correctParents(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("correctParents (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_10correctParents(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_10correctParents(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("correctParents", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_correctParents(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.correctParents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":265
 *             child.correctParents()
 * 
 *     cpdef removeXNodes(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to remove all nodes starting with the symbol X. If the node is removed, its children are
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_13removeXNodes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_removeXNodes(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_i;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeXNodes", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_removeXNodes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_13removeXNodes)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":272
 *         cdef int i
 *         cdef ParseNode child
 *         i = 0             # <<<<<<<<<<<<<<
 *         while i < len(self.children):
 *             if self.children[i].getData().getName().startswith("X"):
 */
  __pyx_v_i = 0;

  /* "ParseTree/ParseNode.pyx":273
 *         cdef ParseNode child
 *         i = 0
 *         while i < len(self.children):             # <<<<<<<<<<<<<<
 *             if self.children[i].getData().getName().startswith("X"):
 *                 self.children.insert(i + 1, self.children[i].children)
 */
  while (1) {
    __pyx_t_1 = __pyx_v_self->children;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 273, __pyx_L1_error)
    }
    __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = ((__pyx_v_i < __pyx_t_5) != 0);
    if (!__pyx_t_6) break;

    /* "ParseTree/ParseNode.pyx":274
 *         i = 0
 *         while i < len(self.children):
 *             if self.children[i].getData().getName().startswith("X"):             # <<<<<<<<<<<<<<
 *                 self.children.insert(i + 1, self.children[i].children)
 *                 self.children.pop(i)
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 274, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_getData); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_getName); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_startswith); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_2, __pyx_n_u_X) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_n_u_X);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "ParseTree/ParseNode.pyx":275
 *         while i < len(self.children):
 *             if self.children[i].getData().getName().startswith("X"):
 *                 self.children.insert(i + 1, self.children[i].children)             # <<<<<<<<<<<<<<
 *                 self.children.pop(i)
 *             else:
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "insert");
        __PYX_ERR(0, 275, __pyx_L1_error)
      }
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 275, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_children); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 275, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = PyList_Insert(__pyx_v_self->children, (__pyx_v_i + 1), __pyx_t_7); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 275, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "ParseTree/ParseNode.pyx":276
 *             if self.children[i].getData().getName().startswith("X"):
 *                 self.children.insert(i + 1, self.children[i].children)
 *                 self.children.pop(i)             # <<<<<<<<<<<<<<
 *             else:
 *                 i = i + 1
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
        __PYX_ERR(0, 276, __pyx_L1_error)
      }
      __pyx_t_7 = __Pyx_PyList_PopIndex(__pyx_v_self->children, Py_None, __pyx_v_i, 1, int, __Pyx_PyInt_From_int); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 276, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "ParseTree/ParseNode.pyx":274
 *         i = 0
 *         while i < len(self.children):
 *             if self.children[i].getData().getName().startswith("X"):             # <<<<<<<<<<<<<<
 *                 self.children.insert(i + 1, self.children[i].children)
 *                 self.children.pop(i)
 */
      goto __pyx_L5;
    }

    /* "ParseTree/ParseNode.pyx":278
 *                 self.children.pop(i)
 *             else:
 *                 i = i + 1             # <<<<<<<<<<<<<<
 *         for child in self.children:
 *             child.removeXNodes()
 */
    /*else*/ {
      __pyx_v_i = (__pyx_v_i + 1);
    }
    __pyx_L5:;
  }

  /* "ParseTree/ParseNode.pyx":279
 *             else:
 *                 i = i + 1
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.removeXNodes()
 * 
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 279, __pyx_L1_error)
  }
  __pyx_t_7 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_7); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_7)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 279, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_7, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":280
 *                 i = i + 1
 *         for child in self.children:
 *             child.removeXNodes()             # <<<<<<<<<<<<<<
 * 
 *     cpdef setChild(self, int index, ParseNode child):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->removeXNodes(__pyx_v_child, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":279
 *             else:
 *                 i = i + 1
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.removeXNodes()
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "ParseTree/ParseNode.pyx":265
 *             child.correctParents()
 * 
 *     cpdef removeXNodes(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to remove all nodes starting with the symbol X. If the node is removed, its children are
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.removeXNodes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_13removeXNodes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_12removeXNodes[] = "\n        Recursive method to remove all nodes starting with the symbol X. If the node is removed, its children are\n        connected to the next sibling of the deleted node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_13removeXNodes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("removeXNodes (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_12removeXNodes(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_12removeXNodes(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeXNodes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_removeXNodes(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.removeXNodes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":282
 *             child.removeXNodes()
 * 
 *     cpdef setChild(self, int index, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Replaces a child node at the given specific with a new child node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_15setChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_setChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_index, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_15setChild)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 282, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, ((PyObject *)__pyx_v_child)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 282, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, ((PyObject *)__pyx_v_child)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 282, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 282, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
          __Pyx_INCREF(((PyObject *)__pyx_v_child));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_child));
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, ((PyObject *)__pyx_v_child));
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 282, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":293
 *             Child node to be replaced.
 *         """
 *         self.children[index] = child             # <<<<<<<<<<<<<<
 * 
 *     cpdef removeChild(self, ParseNode child):
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 293, __pyx_L1_error)
  }
  if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, __pyx_v_index, ((PyObject *)__pyx_v_child), int, 1, __Pyx_PyInt_From_int, 1, 1, 1) < 0)) __PYX_ERR(0, 293, __pyx_L1_error)

  /* "ParseTree/ParseNode.pyx":282
 *             child.removeXNodes()
 * 
 *     cpdef setChild(self, int index, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Replaces a child node at the given specific with a new child node.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.setChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_15setChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_14setChild[] = "\n        Replaces a child node at the given specific with a new child node.\n\n        PARAMETERS\n        ----------\n        index : int\n            Index where the new child node replaces the old one.\n        child : ParseNode\n            Child node to be replaced.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_15setChild(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_index;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setChild (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_child,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_child)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("setChild", 1, 2, 2, 1); __PYX_ERR(0, 282, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "setChild") < 0)) __PYX_ERR(0, 282, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_index == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 282, __pyx_L3_error)
    __pyx_v_child = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("setChild", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 282, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.setChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "child", 0))) __PYX_ERR(0, 282, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_14setChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), __pyx_v_index, __pyx_v_child);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_14setChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_index, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_setChild(__pyx_v_self, __pyx_v_index, __pyx_v_child, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.setChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":295
 *         self.children[index] = child
 * 
 *     cpdef removeChild(self, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Removes a given child from children node list.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_17removeChild(PyObject *__pyx_v_self, PyObject *__pyx_v_child); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_removeChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_removeChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_17removeChild)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_child)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_child));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 295, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":304
 *             Child node to be deleted.
 *         """
 *         self.children.remove(child)             # <<<<<<<<<<<<<<
 * 
 *     cpdef int leafCount(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->children, __pyx_n_s_remove); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_child)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_child));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ParseTree/ParseNode.pyx":295
 *         self.children[index] = child
 * 
 *     cpdef removeChild(self, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Removes a given child from children node list.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.removeChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_17removeChild(PyObject *__pyx_v_self, PyObject *__pyx_v_child); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_16removeChild[] = "\n        Removes a given child from children node list.\n\n        PARAMETERS\n        ----------\n        child : ParseNode\n            Child node to be deleted.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_17removeChild(PyObject *__pyx_v_self, PyObject *__pyx_v_child) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("removeChild (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "child", 0))) __PYX_ERR(0, 295, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_16removeChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_16removeChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("removeChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_removeChild(__pyx_v_self, __pyx_v_child, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.removeChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":306
 *         self.children.remove(child)
 * 
 *     cpdef int leafCount(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to calculate the number of all leaf nodes in the subtree rooted with this current node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_19leafCount(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_leafCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_total;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("leafCount", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_leafCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_19leafCount)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 306, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 306, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":317
 *         cdef int total
 *         cdef ParseNode child
 *         if len(self.children) == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 317, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((__pyx_t_6 == 0) != 0);
  if (__pyx_t_7) {

    /* "ParseTree/ParseNode.pyx":318
 *         cdef ParseNode child
 *         if len(self.children) == 0:
 *             return 1             # <<<<<<<<<<<<<<
 *         else:
 *             total = 0
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":317
 *         cdef int total
 *         cdef ParseNode child
 *         if len(self.children) == 0:             # <<<<<<<<<<<<<<
 *             return 1
 *         else:
 */
  }

  /* "ParseTree/ParseNode.pyx":320
 *             return 1
 *         else:
 *             total = 0             # <<<<<<<<<<<<<<
 *             for child in self.children:
 *                 total += child.leafCount()
 */
  /*else*/ {
    __pyx_v_total = 0;

    /* "ParseTree/ParseNode.pyx":321
 *         else:
 *             total = 0
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 total += child.leafCount()
 *             return total
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 321, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 321, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 321, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "ParseTree/ParseNode.pyx":322
 *             total = 0
 *             for child in self.children:
 *                 total += child.leafCount()             # <<<<<<<<<<<<<<
 *             return total
 * 
 */
      __pyx_v_total = (__pyx_v_total + ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->leafCount(__pyx_v_child, 0));

      /* "ParseTree/ParseNode.pyx":321
 *         else:
 *             total = 0
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 total += child.leafCount()
 *             return total
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":323
 *             for child in self.children:
 *                 total += child.leafCount()
 *             return total             # <<<<<<<<<<<<<<
 * 
 *     cpdef int nodeCount(self):
 */
    __pyx_r = __pyx_v_total;
    goto __pyx_L0;
  }

  /* "ParseTree/ParseNode.pyx":306
 *         self.children.remove(child)
 * 
 *     cpdef int leafCount(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to calculate the number of all leaf nodes in the subtree rooted with this current node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.leafCount", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_19leafCount(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_18leafCount[] = "\n        Recursive method to calculate the number of all leaf nodes in the subtree rooted with this current node.\n\n        RETURNS\n        -------\n        int\n            Number of all leaf nodes in the current subtree.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_19leafCount(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("leafCount (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_18leafCount(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_18leafCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("leafCount", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_9ParseTree_9ParseNode_9ParseNode_leafCount(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.leafCount", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":325
 *             return total
 * 
 *     cpdef int nodeCount(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to calculate the number of all nodes in the subtree rooted with this current node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_21nodeCount(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_total;
  PyObject *__pyx_v_child = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nodeCount", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nodeCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_21nodeCount)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 325, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":335
 *         """
 *         cdef int total
 *         if len(self.children) > 0:             # <<<<<<<<<<<<<<
 *             total = 1
 *             for child in self.children:
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 335, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((__pyx_t_6 > 0) != 0);
  if (__pyx_t_7) {

    /* "ParseTree/ParseNode.pyx":336
 *         cdef int total
 *         if len(self.children) > 0:
 *             total = 1             # <<<<<<<<<<<<<<
 *             for child in self.children:
 *                 total += child.nodeCount()
 */
    __pyx_v_total = 1;

    /* "ParseTree/ParseNode.pyx":337
 *         if len(self.children) > 0:
 *             total = 1
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 total += child.nodeCount()
 *             return total
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 337, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 337, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 337, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_child, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "ParseTree/ParseNode.pyx":338
 *             total = 1
 *             for child in self.children:
 *                 total += child.nodeCount()             # <<<<<<<<<<<<<<
 *             return total
 *         else:
 */
      __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_total); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 338, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_child, __pyx_n_s_nodeCount); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 338, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_3 = (__pyx_t_8) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 338, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 338, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 338, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_total = __pyx_t_5;

      /* "ParseTree/ParseNode.pyx":337
 *         if len(self.children) > 0:
 *             total = 1
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 total += child.nodeCount()
 *             return total
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":339
 *             for child in self.children:
 *                 total += child.nodeCount()
 *             return total             # <<<<<<<<<<<<<<
 *         else:
 *             return 1
 */
    __pyx_r = __pyx_v_total;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":335
 *         """
 *         cdef int total
 *         if len(self.children) > 0:             # <<<<<<<<<<<<<<
 *             total = 1
 *             for child in self.children:
 */
  }

  /* "ParseTree/ParseNode.pyx":341
 *             return total
 *         else:
 *             return 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef int nodeCountWithMultipleChildren(self):
 */
  /*else*/ {
    __pyx_r = 1;
    goto __pyx_L0;
  }

  /* "ParseTree/ParseNode.pyx":325
 *             return total
 * 
 *     cpdef int nodeCount(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to calculate the number of all nodes in the subtree rooted with this current node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.nodeCount", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_child);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_21nodeCount(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_20nodeCount[] = "\n        Recursive method to calculate the number of all nodes in the subtree rooted with this current node.\n\n        RETURNS\n        -------\n        int\n            Number of all nodes in the current subtree.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_21nodeCount(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("nodeCount (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_20nodeCount(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_20nodeCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nodeCount", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCount(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.nodeCount", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":343
 *             return 1
 * 
 *     cpdef int nodeCountWithMultipleChildren(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to calculate the number of all nodes, which have more than one children, in the subtree rooted
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_23nodeCountWithMultipleChildren(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCountWithMultipleChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_total;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nodeCountWithMultipleChildren", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nodeCountWithMultipleChildren); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_23nodeCountWithMultipleChildren)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 343, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 343, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":355
 *         cdef int total
 *         cdef ParseNode child
 *         if len(self.children) > 1:             # <<<<<<<<<<<<<<
 *             total = 1
 *             for child in self.children:
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 355, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 355, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((__pyx_t_6 > 1) != 0);
  if (__pyx_t_7) {

    /* "ParseTree/ParseNode.pyx":356
 *         cdef ParseNode child
 *         if len(self.children) > 1:
 *             total = 1             # <<<<<<<<<<<<<<
 *             for child in self.children:
 *                 total += child.nodeCountWithMultipleChildren()
 */
    __pyx_v_total = 1;

    /* "ParseTree/ParseNode.pyx":357
 *         if len(self.children) > 1:
 *             total = 1
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 total += child.nodeCountWithMultipleChildren()
 *             return total
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 357, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 357, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "ParseTree/ParseNode.pyx":358
 *             total = 1
 *             for child in self.children:
 *                 total += child.nodeCountWithMultipleChildren()             # <<<<<<<<<<<<<<
 *             return total
 *         else:
 */
      __pyx_v_total = (__pyx_v_total + ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->nodeCountWithMultipleChildren(__pyx_v_child, 0));

      /* "ParseTree/ParseNode.pyx":357
 *         if len(self.children) > 1:
 *             total = 1
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 total += child.nodeCountWithMultipleChildren()
 *             return total
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":359
 *             for child in self.children:
 *                 total += child.nodeCountWithMultipleChildren()
 *             return total             # <<<<<<<<<<<<<<
 *         else:
 *             return 0
 */
    __pyx_r = __pyx_v_total;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":355
 *         cdef int total
 *         cdef ParseNode child
 *         if len(self.children) > 1:             # <<<<<<<<<<<<<<
 *             total = 1
 *             for child in self.children:
 */
  }

  /* "ParseTree/ParseNode.pyx":361
 *             return total
 *         else:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef int numberOfChildren(self):
 */
  /*else*/ {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "ParseTree/ParseNode.pyx":343
 *             return 1
 * 
 *     cpdef int nodeCountWithMultipleChildren(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive method to calculate the number of all nodes, which have more than one children, in the subtree rooted
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.nodeCountWithMultipleChildren", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_23nodeCountWithMultipleChildren(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_22nodeCountWithMultipleChildren[] = "\n        Recursive method to calculate the number of all nodes, which have more than one children, in the subtree rooted\n        with this current node.\n\n        RETURNS\n        -------\n        int\n            Number of all nodes, which have more than one children, in the current subtree.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_23nodeCountWithMultipleChildren(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("nodeCountWithMultipleChildren (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_22nodeCountWithMultipleChildren(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_22nodeCountWithMultipleChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nodeCountWithMultipleChildren", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCountWithMultipleChildren(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.nodeCountWithMultipleChildren", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":363
 *             return 0
 * 
 *     cpdef int numberOfChildren(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns number of children of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_25numberOfChildren(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_numberOfChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("numberOfChildren", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_numberOfChildren); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 363, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_25numberOfChildren)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 363, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 363, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":372
 *             Number of children of this node.
 *         """
 *         return len(self.children)             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode getChild(self, int i):
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 372, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 372, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_6;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":363
 *             return 0
 * 
 *     cpdef int numberOfChildren(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns number of children of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.numberOfChildren", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_25numberOfChildren(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_24numberOfChildren[] = "\n        Returns number of children of this node.\n\n        RETURNS\n        -------\n        int\n            Number of children of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_25numberOfChildren(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("numberOfChildren (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_24numberOfChildren(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_24numberOfChildren(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("numberOfChildren", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_9ParseTree_9ParseNode_9ParseNode_numberOfChildren(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.numberOfChildren", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":374
 *         return len(self.children)
 * 
 *     cpdef ParseNode getChild(self, int i):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the i'th child of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_27getChild(PyObject *__pyx_v_self, PyObject *__pyx_arg_i); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_getChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_i, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 374, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_27getChild)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 374, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 374, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 374, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":388
 *             i'th child of this node.
 *         """
 *         return self.children[i]             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode firstChild(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 388, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 388, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":374
 *         return len(self.children)
 * 
 *     cpdef ParseNode getChild(self, int i):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the i'th child of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_27getChild(PyObject *__pyx_v_self, PyObject *__pyx_arg_i); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_26getChild[] = "\n        Returns the i'th child of this node.\n\n        PARAMETERS\n        ----------\n        i : int\n            Index of the retrieved node.\n\n        RETURNS\n        -------\n        ParseNode\n            i'th child of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_27getChild(PyObject *__pyx_v_self, PyObject *__pyx_arg_i) {
  int __pyx_v_i;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getChild (wrapper)", 0);
  assert(__pyx_arg_i); {
    __pyx_v_i = __Pyx_PyInt_As_int(__pyx_arg_i); if (unlikely((__pyx_v_i == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 374, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_26getChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((int)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_26getChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_i) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_getChild(__pyx_v_self, __pyx_v_i, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":390
 *         return self.children[i]
 * 
 *     cpdef ParseNode firstChild(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the first child of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_29firstChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_firstChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("firstChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_firstChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_29firstChild)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 390, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":399
 *             First child of this node.
 *         """
 *         return self.children[0]             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode lastChild(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 399, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 399, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":390
 *         return self.children[i]
 * 
 *     cpdef ParseNode firstChild(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the first child of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.firstChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_29firstChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_28firstChild[] = "\n        Returns the first child of this node.\n\n        RETURNS\n        -------\n        ParseNode\n            First child of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_29firstChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("firstChild (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_28firstChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_28firstChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("firstChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_firstChild(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.firstChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":401
 *         return self.children[0]
 * 
 *     cpdef ParseNode lastChild(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the last child of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_31lastChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_lastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lastChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_lastChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_31lastChild)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 401, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 401, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":410
 *             Last child of this node.
 *         """
 *         return self.children[len(self.children) - 1]             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isLastChild(self, ParseNode child):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 410, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 410, __pyx_L1_error)
  }
  __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_5 - 1);
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_t_6, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 410, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":401
 *         return self.children[0]
 * 
 *     cpdef ParseNode lastChild(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the last child of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.lastChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_31lastChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_30lastChild[] = "\n        Returns the last child of this node.\n\n        RETURNS\n        -------\n        ParseNode\n            Last child of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_31lastChild(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lastChild (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_30lastChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_30lastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lastChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_lastChild(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.lastChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":412
 *         return self.children[len(self.children) - 1]
 * 
 *     cpdef bint isLastChild(self, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Checks if the given node is the last child of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_33isLastChild(PyObject *__pyx_v_self, PyObject *__pyx_v_child); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isLastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isLastChild", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_isLastChild); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_33isLastChild)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_child)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_child));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":426
 *             True, if child is the last child of this node, false otherwise.
 *         """
 *         return self.children[len(self.children) - 1] == child             # <<<<<<<<<<<<<<
 * 
 *     cpdef int getChildIndex(self, ParseNode child):
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 426, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 426, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = (__pyx_t_6 - 1);
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_t_7, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, ((PyObject *)__pyx_v_child), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 426, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":412
 *         return self.children[len(self.children) - 1]
 * 
 *     cpdef bint isLastChild(self, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Checks if the given node is the last child of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.isLastChild", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_33isLastChild(PyObject *__pyx_v_self, PyObject *__pyx_v_child); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_32isLastChild[] = "\n        Checks if the given node is the last child of this node.\n\n        PARAMETERS\n        ----------\n        child : ParseNode\n            To be checked node.\n\n        RETURNS\n        -------\n        bool\n            True, if child is the last child of this node, false otherwise.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_33isLastChild(PyObject *__pyx_v_self, PyObject *__pyx_v_child) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isLastChild (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "child", 0))) __PYX_ERR(0, 412, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_32isLastChild(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_32isLastChild(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isLastChild", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_9ParseTree_9ParseNode_9ParseNode_isLastChild(__pyx_v_self, __pyx_v_child, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.isLastChild", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":428
 *         return self.children[len(self.children) - 1] == child
 * 
 *     cpdef int getChildIndex(self, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the index of the given child of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_35getChildIndex(PyObject *__pyx_v_self, PyObject *__pyx_v_child); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_getChildIndex(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getChildIndex", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getChildIndex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 428, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_35getChildIndex)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_child)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_child));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 428, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":437
 *             Index of the child of this node.
 *         """
 *         return self.children.index(child)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isDescendant(self, ParseNode node):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->children, __pyx_n_s_index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, ((PyObject *)__pyx_v_child)) : __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_child));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":428
 *         return self.children[len(self.children) - 1] == child
 * 
 *     cpdef int getChildIndex(self, ParseNode child):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the index of the given child of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.getChildIndex", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_35getChildIndex(PyObject *__pyx_v_self, PyObject *__pyx_v_child); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_34getChildIndex[] = "\n        Returns the index of the given child of this node.\n\n        RETURNS\n        -------\n        int\n            Index of the child of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_35getChildIndex(PyObject *__pyx_v_self, PyObject *__pyx_v_child) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getChildIndex (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "child", 0))) __PYX_ERR(0, 428, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_34getChildIndex(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_34getChildIndex(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getChildIndex", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_9ParseTree_9ParseNode_9ParseNode_getChildIndex(__pyx_v_self, __pyx_v_child, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getChildIndex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":439
 *         return self.children.index(child)
 * 
 *     cpdef bint isDescendant(self, ParseNode node):             # <<<<<<<<<<<<<<
 *         """
 *         Returns true if the given node is a descendant of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_37isDescendant(PyObject *__pyx_v_self, PyObject *__pyx_v_node); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isDescendant(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isDescendant", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_isDescendant); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 439, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_37isDescendant)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_node)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_node));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 439, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":449
 *         """
 *         cdef ParseNode child
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             if child == node:
 *                 return True
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 449, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
  for (;;) {
    if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 449, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ParseTree/ParseNode.pyx":450
 *         cdef ParseNode child
 *         for child in self.children:
 *             if child == node:             # <<<<<<<<<<<<<<
 *                 return True
 *             elif child.isDescendant(node):
 */
    __pyx_t_2 = PyObject_RichCompare(((PyObject *)__pyx_v_child), ((PyObject *)__pyx_v_node), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_5) {

      /* "ParseTree/ParseNode.pyx":451
 *         for child in self.children:
 *             if child == node:
 *                 return True             # <<<<<<<<<<<<<<
 *             elif child.isDescendant(node):
 *                 return True
 */
      __pyx_r = 1;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":450
 *         cdef ParseNode child
 *         for child in self.children:
 *             if child == node:             # <<<<<<<<<<<<<<
 *                 return True
 *             elif child.isDescendant(node):
 */
    }

    /* "ParseTree/ParseNode.pyx":452
 *             if child == node:
 *                 return True
 *             elif child.isDescendant(node):             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
 */
    __pyx_t_5 = (((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->isDescendant(__pyx_v_child, __pyx_v_node, 0) != 0);
    if (__pyx_t_5) {

      /* "ParseTree/ParseNode.pyx":453
 *                 return True
 *             elif child.isDescendant(node):
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
      __pyx_r = 1;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":452
 *             if child == node:
 *                 return True
 *             elif child.isDescendant(node):             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
 */
    }

    /* "ParseTree/ParseNode.pyx":449
 *         """
 *         cdef ParseNode child
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             if child == node:
 *                 return True
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ParseTree/ParseNode.pyx":454
 *             elif child.isDescendant(node):
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode previousSibling(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":439
 *         return self.children.index(child)
 * 
 *     cpdef bint isDescendant(self, ParseNode node):             # <<<<<<<<<<<<<<
 *         """
 *         Returns true if the given node is a descendant of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.isDescendant", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_37isDescendant(PyObject *__pyx_v_self, PyObject *__pyx_v_node); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_36isDescendant[] = "\n        Returns true if the given node is a descendant of this node.\n\n        RETURNS\n        -------\n        bool\n            True if the given node is descendant of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_37isDescendant(PyObject *__pyx_v_self, PyObject *__pyx_v_node) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isDescendant (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_node), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "node", 0))) __PYX_ERR(0, 439, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_36isDescendant(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_node));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_36isDescendant(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isDescendant", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_9ParseTree_9ParseNode_9ParseNode_isDescendant(__pyx_v_self, __pyx_v_node, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.isDescendant", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":456
 *         return False
 * 
 *     cpdef ParseNode previousSibling(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the previous sibling (sister) of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_39previousSibling(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_previousSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_i;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  long __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("previousSibling", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_previousSibling); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_39previousSibling)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 456, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 456, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":467
 *         """
 *         cdef int i
 *         for i in range(1, len(self.parent.children)):             # <<<<<<<<<<<<<<
 *             if self.parent.children[i] == self:
 *                 return self.parent.children[i - 1]
 */
  __pyx_t_1 = __pyx_v_self->parent->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 467, __pyx_L1_error)
  }
  __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 1; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ParseTree/ParseNode.pyx":468
 *         cdef int i
 *         for i in range(1, len(self.parent.children)):
 *             if self.parent.children[i] == self:             # <<<<<<<<<<<<<<
 *                 return self.parent.children[i - 1]
 *         return None
 */
    if (unlikely(__pyx_v_self->parent->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 468, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->parent->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, ((PyObject *)__pyx_v_self), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 468, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 468, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_8) {

      /* "ParseTree/ParseNode.pyx":469
 *         for i in range(1, len(self.parent.children)):
 *             if self.parent.children[i] == self:
 *                 return self.parent.children[i - 1]             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      if (unlikely(__pyx_v_self->parent->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 469, __pyx_L1_error)
      }
      __pyx_t_9 = (__pyx_v_i - 1);
      __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->parent->children, __pyx_t_9, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 469, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
      __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":468
 *         cdef int i
 *         for i in range(1, len(self.parent.children)):
 *             if self.parent.children[i] == self:             # <<<<<<<<<<<<<<
 *                 return self.parent.children[i - 1]
 *         return None
 */
    }
  }

  /* "ParseTree/ParseNode.pyx":470
 *             if self.parent.children[i] == self:
 *                 return self.parent.children[i - 1]
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode nextSibling(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":456
 *         return False
 * 
 *     cpdef ParseNode previousSibling(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the previous sibling (sister) of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.previousSibling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_39previousSibling(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_38previousSibling[] = "\n        Returns the previous sibling (sister) of this node.\n\n        RETURNS\n        -------\n        ParseNode\n            If this is the first child of its parent, returns null. Otherwise, returns the previous sibling of this\n            node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_39previousSibling(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("previousSibling (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_38previousSibling(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_38previousSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("previousSibling", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_previousSibling(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.previousSibling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":472
 *         return None
 * 
 *     cpdef ParseNode nextSibling(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the next sibling (sister) of this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_41nextSibling(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_nextSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_v_i;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  long __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nextSibling", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_nextSibling); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_41nextSibling)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 472, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":482
 *         """
 *         cdef int i
 *         for i in range(0, len(self.parent.children) - 1):             # <<<<<<<<<<<<<<
 *             if self.parent.children[i] == self:
 *                 return self.parent.children[i + 1]
 */
  __pyx_t_1 = __pyx_v_self->parent->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 482, __pyx_L1_error)
  }
  __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_5 - 1);
  __pyx_t_5 = __pyx_t_6;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_5; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ParseTree/ParseNode.pyx":483
 *         cdef int i
 *         for i in range(0, len(self.parent.children) - 1):
 *             if self.parent.children[i] == self:             # <<<<<<<<<<<<<<
 *                 return self.parent.children[i + 1]
 *         return None
 */
    if (unlikely(__pyx_v_self->parent->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 483, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->parent->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, ((PyObject *)__pyx_v_self), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_8) {

      /* "ParseTree/ParseNode.pyx":484
 *         for i in range(0, len(self.parent.children) - 1):
 *             if self.parent.children[i] == self:
 *                 return self.parent.children[i + 1]             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      if (unlikely(__pyx_v_self->parent->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 484, __pyx_L1_error)
      }
      __pyx_t_9 = (__pyx_v_i + 1);
      __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->parent->children, __pyx_t_9, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 484, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
      __pyx_t_2 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":483
 *         cdef int i
 *         for i in range(0, len(self.parent.children) - 1):
 *             if self.parent.children[i] == self:             # <<<<<<<<<<<<<<
 *                 return self.parent.children[i + 1]
 *         return None
 */
    }
  }

  /* "ParseTree/ParseNode.pyx":485
 *             if self.parent.children[i] == self:
 *                 return self.parent.children[i + 1]
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cpdef ParseNode getParent(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":472
 *         return None
 * 
 *     cpdef ParseNode nextSibling(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the next sibling (sister) of this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.nextSibling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_41nextSibling(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_40nextSibling[] = "\n        Returns the next sibling (sister) of this node.\n\n        RETURNS\n        -------\n        ParseNode\n            If this is the last child of its parent, returns null. Otherwise, returns the next sibling of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_41nextSibling(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("nextSibling (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_40nextSibling(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_40nextSibling(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("nextSibling", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_nextSibling(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.nextSibling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":487
 *         return None
 * 
 *     cpdef ParseNode getParent(self):             # <<<<<<<<<<<<<<
 *         """
 *         Accessor for the parent attribute.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_43getParent(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_f_9ParseTree_9ParseNode_9ParseNode_getParent(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getParent", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getParent); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_43getParent)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 487, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":496
 *             Parent of this node.
 *         """
 *         return self.parent             # <<<<<<<<<<<<<<
 * 
 *     cpdef Symbol getData(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->parent));
  __pyx_r = __pyx_v_self->parent;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":487
 *         return None
 * 
 *     cpdef ParseNode getParent(self):             # <<<<<<<<<<<<<<
 *         """
 *         Accessor for the parent attribute.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getParent", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_43getParent(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_42getParent[] = "\n        Accessor for the parent attribute.\n\n        RETURNS\n        -------\n        ParseNode\n            Parent of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_43getParent(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getParent (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_42getParent(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_42getParent(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getParent", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_getParent(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getParent", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":498
 *         return self.parent
 * 
 *     cpdef Symbol getData(self):             # <<<<<<<<<<<<<<
 *         """
 *         Accessor for the data attribute.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_45getData(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_f_9ParseTree_9ParseNode_9ParseNode_getData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getData", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_getData); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 498, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_45getData)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 498, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_6Symbol_Symbol))))) __PYX_ERR(0, 498, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":507
 *             Data of this node.
 *         """
 *         return self.data             # <<<<<<<<<<<<<<
 * 
 *     cpdef setData(self, Symbol data):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data));
  __pyx_r = __pyx_v_self->data;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":498
 *         return self.parent
 * 
 *     cpdef Symbol getData(self):             # <<<<<<<<<<<<<<
 *         """
 *         Accessor for the data attribute.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_45getData(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_44getData[] = "\n        Accessor for the data attribute.\n\n        RETURNS\n        -------\n        Symbol\n            Data of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_45getData(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("getData (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_44getData(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_44getData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("getData", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_9ParseTree_9ParseNode_9ParseNode_getData(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.getData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":509
 *         return self.data
 * 
 *     cpdef setData(self, Symbol data):             # <<<<<<<<<<<<<<
 *         """
 *         Mutator of the data attribute.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_47setData(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_setData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setData", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setData); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_47setData)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_data)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_data));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 509, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":518
 *             Data to be set.
 *         """
 *         self.data = data             # <<<<<<<<<<<<<<
 * 
 *     cpdef int wordCount(self, bint excludeStopWords):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_data));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_data));
  __Pyx_GOTREF(__pyx_v_self->data);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->data));
  __pyx_v_self->data = __pyx_v_data;

  /* "ParseTree/ParseNode.pyx":509
 *         return self.data
 * 
 *     cpdef setData(self, Symbol data):             # <<<<<<<<<<<<<<
 *         """
 *         Mutator of the data attribute.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.setData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_47setData(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_46setData[] = "\n        Mutator of the data attribute.\n\n        PARAMETERS\n        ----------\n        data : Symbol\n            Data to be set.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_47setData(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setData (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), __pyx_ptype_9ParseTree_6Symbol_Symbol, 1, "data", 0))) __PYX_ERR(0, 509, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_46setData(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_46setData(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_6Symbol_Symbol *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setData", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_setData(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.setData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":520
 *         self.data = data
 * 
 *     cpdef int wordCount(self, bint excludeStopWords):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive function to count the number of words in the subtree rooted at this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_49wordCount(PyObject *__pyx_v_self, PyObject *__pyx_arg_excludeStopWords); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_wordCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_excludeStopWords, int __pyx_skip_dispatch) {
  int __pyx_v_total;
  PyObject *__pyx_v_child = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wordCount", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_wordCount); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_49wordCount)) {
        __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_excludeStopWords); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 520, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        __pyx_t_2 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_5, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3);
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 520, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 520, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_6;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":535
 *         """
 *         cdef int total
 *         if len(self.children) == 0:             # <<<<<<<<<<<<<<
 *             if not excludeStopWords:
 *                 total = 1
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 535, __pyx_L1_error)
  }
  __pyx_t_7 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((__pyx_t_7 == 0) != 0);
  if (__pyx_t_8) {

    /* "ParseTree/ParseNode.pyx":536
 *         cdef int total
 *         if len(self.children) == 0:
 *             if not excludeStopWords:             # <<<<<<<<<<<<<<
 *                 total = 1
 *             else:
 */
    __pyx_t_8 = ((!(__pyx_v_excludeStopWords != 0)) != 0);
    if (__pyx_t_8) {

      /* "ParseTree/ParseNode.pyx":537
 *         if len(self.children) == 0:
 *             if not excludeStopWords:
 *                 total = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 if Word.isPunctuationSymbol(self.data.getName()) or \
 */
      __pyx_v_total = 1;

      /* "ParseTree/ParseNode.pyx":536
 *         cdef int total
 *         if len(self.children) == 0:
 *             if not excludeStopWords:             # <<<<<<<<<<<<<<
 *                 total = 1
 *             else:
 */
      goto __pyx_L4;
    }

    /* "ParseTree/ParseNode.pyx":539
 *                 total = 1
 *             else:
 *                 if Word.isPunctuationSymbol(self.data.getName()) or \             # <<<<<<<<<<<<<<
 *                         "*" in self.data.getName().lower() or self.data.getName().lower() == "at" or self.data.getName().lower() == "the" or \
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_10Dictionary_4Word_Word), __pyx_n_s_isPunctuationSymbol); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_9) {
      } else {
        __pyx_t_8 = __pyx_t_9;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":540
 *             else:
 *                 if Word.isPunctuationSymbol(self.data.getName()) or \
 *                         "*" in self.data.getName().lower() or self.data.getName().lower() == "at" or self.data.getName().lower() == "the" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \
 *                         self.data.getName().lower() == "not" or self.data.getName().lower() == "is" or self.data.getName().lower() == "was" or \
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_kp_u__5, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_10 = (__pyx_t_9 != 0);
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_at, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_the, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 540, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":541
 *                 if Word.isPunctuationSymbol(self.data.getName()) or \
 *                         "*" in self.data.getName().lower() or self.data.getName().lower() == "at" or self.data.getName().lower() == "the" or \
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "not" or self.data.getName().lower() == "is" or self.data.getName().lower() == "was" or \
 *                         self.data.getName().lower() == "were" or self.data.getName().lower() == "have" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_to, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_a, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_an, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":542
 *                         "*" in self.data.getName().lower() or self.data.getName().lower() == "at" or self.data.getName().lower() == "the" or \
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \
 *                         self.data.getName().lower() == "not" or self.data.getName().lower() == "is" or self.data.getName().lower() == "was" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "were" or self.data.getName().lower() == "have" or \
 *                         self.data.getName().lower() == "had" or self.data.getName().lower() == "has" or \
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_not, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_is, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_was, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 542, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":543
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \
 *                         self.data.getName().lower() == "not" or self.data.getName().lower() == "is" or self.data.getName().lower() == "was" or \
 *                         self.data.getName().lower() == "were" or self.data.getName().lower() == "have" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "had" or self.data.getName().lower() == "has" or \
 *                         self.data.getName().lower() == "by" or self.data.getName().lower() == "at" or self.data.getName().lower() == "'re" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_were, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_have, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":544
 *                         self.data.getName().lower() == "not" or self.data.getName().lower() == "is" or self.data.getName().lower() == "was" or \
 *                         self.data.getName().lower() == "were" or self.data.getName().lower() == "have" or \
 *                         self.data.getName().lower() == "had" or self.data.getName().lower() == "has" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "by" or self.data.getName().lower() == "at" or self.data.getName().lower() == "'re" or \
 *                         self.data.getName().lower() == "on" or self.data.getName().lower() == "off" or self.data.getName().lower() == "'s" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_had, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_has, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 544, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":545
 *                         self.data.getName().lower() == "were" or self.data.getName().lower() == "have" or \
 *                         self.data.getName().lower() == "had" or self.data.getName().lower() == "has" or \
 *                         self.data.getName().lower() == "by" or self.data.getName().lower() == "at" or self.data.getName().lower() == "'re" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "on" or self.data.getName().lower() == "off" or self.data.getName().lower() == "'s" or \
 *                         self.data.getName().lower() == "n't" or self.data.getName().lower() == "can" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_by, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_at, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u_re, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":546
 *                         self.data.getName().lower() == "had" or self.data.getName().lower() == "has" or \
 *                         self.data.getName().lower() == "by" or self.data.getName().lower() == "at" or self.data.getName().lower() == "'re" or \
 *                         self.data.getName().lower() == "on" or self.data.getName().lower() == "off" or self.data.getName().lower() == "'s" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "n't" or self.data.getName().lower() == "can" or \
 *                         self.data.getName().lower() == "could" or self.data.getName().lower() == "may" or \
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_on, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_off, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u_s, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 546, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":547
 *                         self.data.getName().lower() == "by" or self.data.getName().lower() == "at" or self.data.getName().lower() == "'re" or \
 *                         self.data.getName().lower() == "on" or self.data.getName().lower() == "off" or self.data.getName().lower() == "'s" or \
 *                         self.data.getName().lower() == "n't" or self.data.getName().lower() == "can" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "could" or self.data.getName().lower() == "may" or \
 *                         self.data.getName().lower() == "might" or self.data.getName().lower() == "will" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u_n_t, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_can, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 547, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":548
 *                         self.data.getName().lower() == "on" or self.data.getName().lower() == "off" or self.data.getName().lower() == "'s" or \
 *                         self.data.getName().lower() == "n't" or self.data.getName().lower() == "can" or \
 *                         self.data.getName().lower() == "could" or self.data.getName().lower() == "may" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "might" or self.data.getName().lower() == "will" or \
 *                         self.data.getName().lower() == "would" or self.data.getName().lower() == "as" or\
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_could, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_may, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":549
 *                         self.data.getName().lower() == "n't" or self.data.getName().lower() == "can" or \
 *                         self.data.getName().lower() == "could" or self.data.getName().lower() == "may" or \
 *                         self.data.getName().lower() == "might" or self.data.getName().lower() == "will" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "would" or self.data.getName().lower() == "as" or\
 *                         self.data.getName().lower() == "with" or self.data.getName().lower() == "for" or self.data.getName().lower() == "in" or\
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_might, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_will, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 549, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":550
 *                         self.data.getName().lower() == "could" or self.data.getName().lower() == "may" or \
 *                         self.data.getName().lower() == "might" or self.data.getName().lower() == "will" or \
 *                         self.data.getName().lower() == "would" or self.data.getName().lower() == "as" or\             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "with" or self.data.getName().lower() == "for" or self.data.getName().lower() == "in" or\
 *                         self.data.getName().lower() == "will" or self.data.getName().lower() == "would" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_would, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_as, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 550, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":551
 *                         self.data.getName().lower() == "might" or self.data.getName().lower() == "will" or \
 *                         self.data.getName().lower() == "would" or self.data.getName().lower() == "as" or\
 *                         self.data.getName().lower() == "with" or self.data.getName().lower() == "for" or self.data.getName().lower() == "in" or\             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "will" or self.data.getName().lower() == "would" or \
 *                         self.data.getName().lower() == "than" or self.data.getName().lower() == "$" or \
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_with, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_for, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_in, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 551, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":552
 *                         self.data.getName().lower() == "would" or self.data.getName().lower() == "as" or\
 *                         self.data.getName().lower() == "with" or self.data.getName().lower() == "for" or self.data.getName().lower() == "in" or\
 *                         self.data.getName().lower() == "will" or self.data.getName().lower() == "would" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "than" or self.data.getName().lower() == "$" or \
 *                         self.data.getName().lower() == "and" or self.data.getName().lower() == "or" or self.data.getName().lower() == "of" or \
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_will, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_would, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":553
 *                         self.data.getName().lower() == "with" or self.data.getName().lower() == "for" or self.data.getName().lower() == "in" or\
 *                         self.data.getName().lower() == "will" or self.data.getName().lower() == "would" or \
 *                         self.data.getName().lower() == "than" or self.data.getName().lower() == "$" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "and" or self.data.getName().lower() == "or" or self.data.getName().lower() == "of" or \
 *                         self.data.getName().lower() == "are" or self.data.getName().lower() == "be" or \
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_than, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u__6, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":554
 *                         self.data.getName().lower() == "will" or self.data.getName().lower() == "would" or \
 *                         self.data.getName().lower() == "than" or self.data.getName().lower() == "$" or \
 *                         self.data.getName().lower() == "and" or self.data.getName().lower() == "or" or self.data.getName().lower() == "of" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "are" or self.data.getName().lower() == "be" or \
 *                         self.data.getName().lower() == "been" or self.data.getName().lower() == "do" or self.data.getName().lower() == "few" or\
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_and, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_or, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_of, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 554, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":555
 *                         self.data.getName().lower() == "than" or self.data.getName().lower() == "$" or \
 *                         self.data.getName().lower() == "and" or self.data.getName().lower() == "or" or self.data.getName().lower() == "of" or \
 *                         self.data.getName().lower() == "are" or self.data.getName().lower() == "be" or \             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "been" or self.data.getName().lower() == "do" or self.data.getName().lower() == "few" or\
 *                         self.data.getName().lower() == "there" or self.data.getName().lower() == "up" or self.data.getName().lower() == "down":
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_are, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_be, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 555, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":556
 *                         self.data.getName().lower() == "and" or self.data.getName().lower() == "or" or self.data.getName().lower() == "of" or \
 *                         self.data.getName().lower() == "are" or self.data.getName().lower() == "be" or \
 *                         self.data.getName().lower() == "been" or self.data.getName().lower() == "do" or self.data.getName().lower() == "few" or\             # <<<<<<<<<<<<<<
 *                         self.data.getName().lower() == "there" or self.data.getName().lower() == "up" or self.data.getName().lower() == "down":
 *                     total = 0
 */
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_been, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_do, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_few, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }

      /* "ParseTree/ParseNode.pyx":557
 *                         self.data.getName().lower() == "are" or self.data.getName().lower() == "be" or \
 *                         self.data.getName().lower() == "been" or self.data.getName().lower() == "do" or self.data.getName().lower() == "few" or\
 *                         self.data.getName().lower() == "there" or self.data.getName().lower() == "up" or self.data.getName().lower() == "down":             # <<<<<<<<<<<<<<
 *                     total = 0
 *                 else:
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_there, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_up, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!__pyx_t_10) {
      } else {
        __pyx_t_8 = __pyx_t_10;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_lower); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_10 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_n_u_down, Py_EQ)); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = __pyx_t_10;
      __pyx_L6_bool_binop_done:;

      /* "ParseTree/ParseNode.pyx":539
 *                 total = 1
 *             else:
 *                 if Word.isPunctuationSymbol(self.data.getName()) or \             # <<<<<<<<<<<<<<
 *                         "*" in self.data.getName().lower() or self.data.getName().lower() == "at" or self.data.getName().lower() == "the" or \
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \
 */
      if (__pyx_t_8) {

        /* "ParseTree/ParseNode.pyx":558
 *                         self.data.getName().lower() == "been" or self.data.getName().lower() == "do" or self.data.getName().lower() == "few" or\
 *                         self.data.getName().lower() == "there" or self.data.getName().lower() == "up" or self.data.getName().lower() == "down":
 *                     total = 0             # <<<<<<<<<<<<<<
 *                 else:
 *                     total = 1
 */
        __pyx_v_total = 0;

        /* "ParseTree/ParseNode.pyx":539
 *                 total = 1
 *             else:
 *                 if Word.isPunctuationSymbol(self.data.getName()) or \             # <<<<<<<<<<<<<<
 *                         "*" in self.data.getName().lower() or self.data.getName().lower() == "at" or self.data.getName().lower() == "the" or \
 *                         self.data.getName().lower() == "to" or self.data.getName().lower() == "a" or self.data.getName().lower() == "an" or \
 */
        goto __pyx_L5;
      }

      /* "ParseTree/ParseNode.pyx":560
 *                     total = 0
 *                 else:
 *                     total = 1             # <<<<<<<<<<<<<<
 *         else:
 *             total = 0
 */
      /*else*/ {
        __pyx_v_total = 1;
      }
      __pyx_L5:;
    }
    __pyx_L4:;

    /* "ParseTree/ParseNode.pyx":535
 *         """
 *         cdef int total
 *         if len(self.children) == 0:             # <<<<<<<<<<<<<<
 *             if not excludeStopWords:
 *                 total = 1
 */
    goto __pyx_L3;
  }

  /* "ParseTree/ParseNode.pyx":562
 *                     total = 1
 *         else:
 *             total = 0             # <<<<<<<<<<<<<<
 *         for child in self.children:
 *             total += child.wordCount(excludeStopWords)
 */
  /*else*/ {
    __pyx_v_total = 0;
  }
  __pyx_L3:;

  /* "ParseTree/ParseNode.pyx":563
 *         else:
 *             total = 0
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             total += child.wordCount(excludeStopWords)
 *         return total
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 563, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
  for (;;) {
    if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 563, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 563, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_child, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "ParseTree/ParseNode.pyx":564
 *             total = 0
 *         for child in self.children:
 *             total += child.wordCount(excludeStopWords)             # <<<<<<<<<<<<<<
 *         return total
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_total); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_child, __pyx_n_s_wordCount); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_excludeStopWords); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_11, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_total = __pyx_t_6;

    /* "ParseTree/ParseNode.pyx":563
 *         else:
 *             total = 0
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             total += child.wordCount(excludeStopWords)
 *         return total
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ParseTree/ParseNode.pyx":565
 *         for child in self.children:
 *             total += child.wordCount(excludeStopWords)
 *         return total             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isLeaf(self):
 */
  __pyx_r = __pyx_v_total;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":520
 *         self.data = data
 * 
 *     cpdef int wordCount(self, bint excludeStopWords):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive function to count the number of words in the subtree rooted at this node.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.wordCount", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_child);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_49wordCount(PyObject *__pyx_v_self, PyObject *__pyx_arg_excludeStopWords); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_48wordCount[] = "\n        Recursive function to count the number of words in the subtree rooted at this node.\n\n        PARAMETERS\n        ----------\n        excludeStopWords : bool\n            If true, stop words are not counted.\n\n        RETURNS\n        -------\n        int\n            Number of words in the subtree rooted at this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_49wordCount(PyObject *__pyx_v_self, PyObject *__pyx_arg_excludeStopWords) {
  int __pyx_v_excludeStopWords;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("wordCount (wrapper)", 0);
  assert(__pyx_arg_excludeStopWords); {
    __pyx_v_excludeStopWords = __Pyx_PyObject_IsTrue(__pyx_arg_excludeStopWords); if (unlikely((__pyx_v_excludeStopWords == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 520, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.wordCount", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_48wordCount(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((int)__pyx_v_excludeStopWords));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_48wordCount(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_excludeStopWords) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("wordCount", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_9ParseTree_9ParseNode_9ParseNode_wordCount(__pyx_v_self, __pyx_v_excludeStopWords, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.wordCount", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":567
 *         return total
 * 
 *     cpdef bint isLeaf(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns True if this node is leaf, False otherwise.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_51isLeaf(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isLeaf", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_isLeaf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_51isLeaf)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 567, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 567, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":576
 *             True if this node is leaf, False otherwise.
 *         """
 *         return len(self.children) == 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint isDummyNode(self):
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 576, __pyx_L1_error)
  }
  __pyx_t_6 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = (__pyx_t_6 == 0);
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":567
 *         return total
 * 
 *     cpdef bint isLeaf(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns True if this node is leaf, False otherwise.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.isLeaf", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_51isLeaf(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_50isLeaf[] = "\n        Returns True if this node is leaf, False otherwise.\n\n        RETURNS\n        -------\n        bool\n            True if this node is leaf, False otherwise.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_51isLeaf(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isLeaf (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_50isLeaf(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_50isLeaf(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isLeaf", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_9ParseTree_9ParseNode_9ParseNode_isLeaf(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.isLeaf", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":578
 *         return len(self.children) == 0
 * 
 *     cpdef bint isDummyNode(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns True if this node does not contain a meaningful data, False otherwise.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_53isDummyNode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_9ParseTree_9ParseNode_9ParseNode_isDummyNode(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isDummyNode", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_isDummyNode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_53isDummyNode)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 578, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 578, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":587
 *             True if this node does not contain a meaningful data, False otherwise.
 *         """
 *         return "*" in self.getData().getName() or (self.getData().getName() == "0" and             # <<<<<<<<<<<<<<
 *                                                    self.parent.getData().getName() == "-NONE-")
 * 
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->getData(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_1), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 587, __pyx_L1_error)
  }
  __pyx_t_6 = (__Pyx_PyUnicode_ContainsTF(__pyx_kp_u__5, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_5 = __pyx_t_7;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->getData(__pyx_v_self, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_2)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_2), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_kp_u_0, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_7 != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }

  /* "ParseTree/ParseNode.pyx":588
 *         """
 *         return "*" in self.getData().getName() or (self.getData().getName() == "0" and
 *                                                    self.parent.getData().getName() == "-NONE-")             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self) -> str:
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->parent->__pyx_vtab)->getData(__pyx_v_self->parent, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_1), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_2, __pyx_kp_u_NONE, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_5 = __pyx_t_7;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "ParseTree/ParseNode.pyx":578
 *         return len(self.children) == 0
 * 
 *     cpdef bint isDummyNode(self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns True if this node does not contain a meaningful data, False otherwise.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("ParseTree.ParseNode.ParseNode.isDummyNode", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_53isDummyNode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_52isDummyNode[] = "\n        Returns True if this node does not contain a meaningful data, False otherwise.\n\n        RETURNS\n        -------\n        bool\n            True if this node does not contain a meaningful data, False otherwise.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_53isDummyNode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("isDummyNode (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_52isDummyNode(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_52isDummyNode(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("isDummyNode", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_9ParseTree_9ParseNode_9ParseNode_isDummyNode(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.isDummyNode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":590
 *                                                    self.parent.getData().getName() == "-NONE-")
 * 
 *     def __str__(self) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Recursive function to convert the subtree rooted at this node to a string.
 */

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_55__str__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_54__str__[] = "\n        Recursive function to convert the subtree rooted at this node to a string.\n\n        RETURNS\n        -------\n        str\n            A string which contains all words in the subtree rooted at this node.\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode_54__str__;
#endif
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_55__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_54__str__(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_54__str__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_v_st = NULL;
  PyObject *__pyx_v_child = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "ParseTree/ParseNode.pyx":599
 *             A string which contains all words in the subtree rooted at this node.
 *         """
 *         if len(self.children) < 2:             # <<<<<<<<<<<<<<
 *             if len(self.children) < 1:
 *                 return self.getData().getName()
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 599, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = ((__pyx_t_2 < 2) != 0);
  if (__pyx_t_3) {

    /* "ParseTree/ParseNode.pyx":600
 *         """
 *         if len(self.children) < 2:
 *             if len(self.children) < 1:             # <<<<<<<<<<<<<<
 *                 return self.getData().getName()
 *             else:
 */
    __pyx_t_1 = __pyx_v_self->children;
    __Pyx_INCREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 600, __pyx_L1_error)
    }
    __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = ((__pyx_t_2 < 1) != 0);
    if (__pyx_t_3) {

      /* "ParseTree/ParseNode.pyx":601
 *         if len(self.children) < 2:
 *             if len(self.children) < 1:
 *                 return self.getData().getName()             # <<<<<<<<<<<<<<
 *             else:
 *                 return "(" + self.data.getName() + " " + self.firstChild().__str__() + ")"
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->getData(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1)->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_t_1), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_r = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":600
 *         """
 *         if len(self.children) < 2:
 *             if len(self.children) < 1:             # <<<<<<<<<<<<<<
 *                 return self.getData().getName()
 *             else:
 */
    }

    /* "ParseTree/ParseNode.pyx":603
 *                 return self.getData().getName()
 *             else:
 *                 return "(" + self.data.getName() + " " + self.firstChild().__str__() + ")"             # <<<<<<<<<<<<<<
 *         else:
 *             st = "(" + self.data.getName()
 */
    /*else*/ {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = __Pyx_PyUnicode_ConcatSafe(__pyx_kp_u__4, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_u__2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->firstChild(__pyx_v_self, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_str); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_6);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyNumber_Add(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyNumber_Add(__pyx_t_6, __pyx_kp_u__3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_r = __pyx_t_1;
      __pyx_t_1 = 0;
      goto __pyx_L0;
    }

    /* "ParseTree/ParseNode.pyx":599
 *             A string which contains all words in the subtree rooted at this node.
 *         """
 *         if len(self.children) < 2:             # <<<<<<<<<<<<<<
 *             if len(self.children) < 1:
 *                 return self.getData().getName()
 */
  }

  /* "ParseTree/ParseNode.pyx":605
 *                 return "(" + self.data.getName() + " " + self.firstChild().__str__() + ")"
 *         else:
 *             st = "(" + self.data.getName()             # <<<<<<<<<<<<<<
 *             for child in self.children:
 *                 st += " " + child.__str__()
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyUnicode_ConcatSafe(__pyx_kp_u__4, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_st = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "ParseTree/ParseNode.pyx":606
 *         else:
 *             st = "(" + self.data.getName()
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 st += " " + child.__str__()
 *             return st + ") "
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 606, __pyx_L1_error)
    }
    __pyx_t_6 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_6); __pyx_t_2 = 0;
    for (;;) {
      if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_6)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_1 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_2); __Pyx_INCREF(__pyx_t_1); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 606, __pyx_L1_error)
      #else
      __pyx_t_1 = PySequence_ITEM(__pyx_t_6, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_child, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "ParseTree/ParseNode.pyx":607
 *             st = "(" + self.data.getName()
 *             for child in self.children:
 *                 st += " " + child.__str__()             # <<<<<<<<<<<<<<
 *             return st + ") "
 * 
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_child, __pyx_n_s_str); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyNumber_Add(__pyx_kp_u__2, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_st, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 607, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_st, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "ParseTree/ParseNode.pyx":606
 *         else:
 *             st = "(" + self.data.getName()
 *             for child in self.children:             # <<<<<<<<<<<<<<
 *                 st += " " + child.__str__()
 *             return st + ") "
 */
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "ParseTree/ParseNode.pyx":608
 *             for child in self.children:
 *                 st += " " + child.__str__()
 *             return st + ") "             # <<<<<<<<<<<<<<
 * 
 *     cpdef moveLeft(self, ParseNode node):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_6 = PyNumber_Add(__pyx_v_st, __pyx_kp_u__7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "ParseTree/ParseNode.pyx":590
 *                                                    self.parent.getData().getName() == "-NONE-")
 * 
 *     def __str__(self) -> str:             # <<<<<<<<<<<<<<
 *         """
 *         Recursive function to convert the subtree rooted at this node to a string.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_st);
  __Pyx_XDECREF(__pyx_v_child);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":610
 *             return st + ") "
 * 
 *     cpdef moveLeft(self, ParseNode node):             # <<<<<<<<<<<<<<
 *         """
 *         Swaps the given child node of this node with the previous sibling of that given node. If the given node is the
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_57moveLeft(PyObject *__pyx_v_self, PyObject *__pyx_v_node); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_moveLeft(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node, int __pyx_skip_dispatch) {
  int __pyx_v_i;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("moveLeft", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_moveLeft); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 610, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_57moveLeft)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_node)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_node));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 610, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":622
 *         cdef int i
 *         cdef ParseNode child
 *         for i in range(len(self.children)):             # <<<<<<<<<<<<<<
 *             if self.children[i] == node:
 *                 if i == 0:
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 622, __pyx_L1_error)
  }
  __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 622, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ParseTree/ParseNode.pyx":623
 *         cdef ParseNode child
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:             # <<<<<<<<<<<<<<
 *                 if i == 0:
 *                     self.children[0], self.children[len(self.children) - 1] = \
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 623, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, ((PyObject *)__pyx_v_node), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 623, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_8) {

      /* "ParseTree/ParseNode.pyx":624
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:
 *                 if i == 0:             # <<<<<<<<<<<<<<
 *                     self.children[0], self.children[len(self.children) - 1] = \
 *                         self.children[len(self.children) - 1], self.children[0]
 */
      __pyx_t_8 = ((__pyx_v_i == 0) != 0);
      if (__pyx_t_8) {

        /* "ParseTree/ParseNode.pyx":626
 *                 if i == 0:
 *                     self.children[0], self.children[len(self.children) - 1] = \
 *                         self.children[len(self.children) - 1], self.children[0]             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.children[i], self.children[(i - 1) % len(self.children)] = \
 */
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 626, __pyx_L1_error)
        }
        __pyx_t_2 = __pyx_v_self->children;
        __Pyx_INCREF(__pyx_t_2);
        if (unlikely(__pyx_t_2 == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(0, 626, __pyx_L1_error)
        }
        __pyx_t_9 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 626, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_10 = (__pyx_t_9 - 1);
        __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_t_10, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 626, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 626, __pyx_L1_error)
        }
        __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);

        /* "ParseTree/ParseNode.pyx":625
 *             if self.children[i] == node:
 *                 if i == 0:
 *                     self.children[0], self.children[len(self.children) - 1] = \             # <<<<<<<<<<<<<<
 *                         self.children[len(self.children) - 1], self.children[0]
 *                 else:
 */
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 625, __pyx_L1_error)
        }
        if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, 0, __pyx_t_2, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 625, __pyx_L1_error)
        }
        __pyx_t_2 = __pyx_v_self->children;
        __Pyx_INCREF(__pyx_t_2);
        if (unlikely(__pyx_t_2 == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(0, 625, __pyx_L1_error)
        }
        __pyx_t_10 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_9 = (__pyx_t_10 - 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, __pyx_t_9, __pyx_t_1, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1) < 0)) __PYX_ERR(0, 625, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "ParseTree/ParseNode.pyx":624
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:
 *                 if i == 0:             # <<<<<<<<<<<<<<
 *                     self.children[0], self.children[len(self.children) - 1] = \
 *                         self.children[len(self.children) - 1], self.children[0]
 */
        goto __pyx_L6;
      }

      /* "ParseTree/ParseNode.pyx":629
 *                 else:
 *                     self.children[i], self.children[(i - 1) % len(self.children)] = \
 *                         self.children[(i - 1) % len(self.children)], self.children[i]             # <<<<<<<<<<<<<<
 *                 return
 *         for child in self.children:
 */
      /*else*/ {
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 629, __pyx_L1_error)
        }
        __pyx_t_11 = (__pyx_v_i - 1);
        __pyx_t_1 = __pyx_v_self->children;
        __Pyx_INCREF(__pyx_t_1);
        if (unlikely(__pyx_t_1 == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(0, 629, __pyx_L1_error)
        }
        __pyx_t_9 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 629, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(__pyx_t_9 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
          __PYX_ERR(0, 629, __pyx_L1_error)
        }
        __pyx_t_10 = __Pyx_mod_Py_ssize_t(__pyx_t_11, __pyx_t_9);
        __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_t_10, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 629, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 629, __pyx_L1_error)
        }
        __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 629, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);

        /* "ParseTree/ParseNode.pyx":628
 *                         self.children[len(self.children) - 1], self.children[0]
 *                 else:
 *                     self.children[i], self.children[(i - 1) % len(self.children)] = \             # <<<<<<<<<<<<<<
 *                         self.children[(i - 1) % len(self.children)], self.children[i]
 *                 return
 */
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 628, __pyx_L1_error)
        }
        if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, __pyx_v_i, __pyx_t_1, int, 1, __Pyx_PyInt_From_int, 1, 1, 1) < 0)) __PYX_ERR(0, 628, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(__pyx_v_self->children == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 628, __pyx_L1_error)
        }
        __pyx_t_11 = (__pyx_v_i - 1);
        __pyx_t_1 = __pyx_v_self->children;
        __Pyx_INCREF(__pyx_t_1);
        if (unlikely(__pyx_t_1 == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
          __PYX_ERR(0, 628, __pyx_L1_error)
        }
        __pyx_t_10 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 628, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(__pyx_t_10 == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
          __PYX_ERR(0, 628, __pyx_L1_error)
        }
        __pyx_t_9 = __Pyx_mod_Py_ssize_t(__pyx_t_11, __pyx_t_10);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, __pyx_t_9, __pyx_t_2, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1) < 0)) __PYX_ERR(0, 628, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __pyx_L6:;

      /* "ParseTree/ParseNode.pyx":630
 *                     self.children[i], self.children[(i - 1) % len(self.children)] = \
 *                         self.children[(i - 1) % len(self.children)], self.children[i]
 *                 return             # <<<<<<<<<<<<<<
 *         for child in self.children:
 *             child.moveLeft(node)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":623
 *         cdef ParseNode child
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:             # <<<<<<<<<<<<<<
 *                 if i == 0:
 *                     self.children[0], self.children[len(self.children) - 1] = \
 */
    }
  }

  /* "ParseTree/ParseNode.pyx":631
 *                         self.children[(i - 1) % len(self.children)], self.children[i]
 *                 return
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.moveLeft(node)
 * 
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 631, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_2); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 631, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 631, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":632
 *                 return
 *         for child in self.children:
 *             child.moveLeft(node)             # <<<<<<<<<<<<<<
 * 
 *     cpdef moveRight(self, ParseNode node):
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->moveLeft(__pyx_v_child, __pyx_v_node, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 632, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "ParseTree/ParseNode.pyx":631
 *                         self.children[(i - 1) % len(self.children)], self.children[i]
 *                 return
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.moveLeft(node)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ParseTree/ParseNode.pyx":610
 *             return st + ") "
 * 
 *     cpdef moveLeft(self, ParseNode node):             # <<<<<<<<<<<<<<
 *         """
 *         Swaps the given child node of this node with the previous sibling of that given node. If the given node is the
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.moveLeft", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_57moveLeft(PyObject *__pyx_v_self, PyObject *__pyx_v_node); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_56moveLeft[] = "\n        Swaps the given child node of this node with the previous sibling of that given node. If the given node is the\n        leftmost child, it swaps with the last node.\n\n        PARAMETERS\n        ----------\n        node : ParseNode\n            Node to be swapped.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_57moveLeft(PyObject *__pyx_v_self, PyObject *__pyx_v_node) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("moveLeft (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_node), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "node", 0))) __PYX_ERR(0, 610, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_56moveLeft(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_node));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_56moveLeft(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("moveLeft", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_moveLeft(__pyx_v_self, __pyx_v_node, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.moveLeft", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":634
 *             child.moveLeft(node)
 * 
 *     cpdef moveRight(self, ParseNode node):             # <<<<<<<<<<<<<<
 *         """
 *         Swaps the given child node of this node with the next sibling of that given node. If the given node is the
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_59moveRight(PyObject *__pyx_v_self, PyObject *__pyx_v_node); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_moveRight(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node, int __pyx_skip_dispatch) {
  int __pyx_v_i;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_child = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  long __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("moveRight", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_moveRight); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_59moveRight)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_node)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_node));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":646
 *         cdef int i
 *         cdef ParseNode child
 *         for i in range(len(self.children)):             # <<<<<<<<<<<<<<
 *             if self.children[i] == node:
 *                 self.children[i], self.children[(i + 1) % len(self.children)] = \
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 646, __pyx_L1_error)
  }
  __pyx_t_5 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __pyx_t_5;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_i = __pyx_t_7;

    /* "ParseTree/ParseNode.pyx":647
 *         cdef ParseNode child
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:             # <<<<<<<<<<<<<<
 *                 self.children[i], self.children[(i + 1) % len(self.children)] = \
 *                     self.children[(i + 1) % len(self.children)], self.children[i]
 */
    if (unlikely(__pyx_v_self->children == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 647, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, ((PyObject *)__pyx_v_node), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_8) {

      /* "ParseTree/ParseNode.pyx":649
 *             if self.children[i] == node:
 *                 self.children[i], self.children[(i + 1) % len(self.children)] = \
 *                     self.children[(i + 1) % len(self.children)], self.children[i]             # <<<<<<<<<<<<<<
 *                 return
 *         for child in self.children:
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 649, __pyx_L1_error)
      }
      __pyx_t_9 = (__pyx_v_i + 1);
      __pyx_t_2 = __pyx_v_self->children;
      __Pyx_INCREF(__pyx_t_2);
      if (unlikely(__pyx_t_2 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 649, __pyx_L1_error)
      }
      __pyx_t_10 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(__pyx_t_10 == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 649, __pyx_L1_error)
      }
      __pyx_t_11 = __Pyx_mod_Py_ssize_t(__pyx_t_9, __pyx_t_10);
      __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_t_11, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 649, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->children, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "ParseTree/ParseNode.pyx":648
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:
 *                 self.children[i], self.children[(i + 1) % len(self.children)] = \             # <<<<<<<<<<<<<<
 *                     self.children[(i + 1) % len(self.children)], self.children[i]
 *                 return
 */
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 648, __pyx_L1_error)
      }
      if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, __pyx_v_i, __pyx_t_2, int, 1, __Pyx_PyInt_From_int, 1, 1, 1) < 0)) __PYX_ERR(0, 648, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(__pyx_v_self->children == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 648, __pyx_L1_error)
      }
      __pyx_t_9 = (__pyx_v_i + 1);
      __pyx_t_2 = __pyx_v_self->children;
      __Pyx_INCREF(__pyx_t_2);
      if (unlikely(__pyx_t_2 == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 648, __pyx_L1_error)
      }
      __pyx_t_11 = PyList_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 648, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(__pyx_t_11 == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        __PYX_ERR(0, 648, __pyx_L1_error)
      }
      __pyx_t_10 = __Pyx_mod_Py_ssize_t(__pyx_t_9, __pyx_t_11);
      if (unlikely(__Pyx_SetItemInt(__pyx_v_self->children, __pyx_t_10, __pyx_t_1, Py_ssize_t, 1, PyInt_FromSsize_t, 1, 1, 1) < 0)) __PYX_ERR(0, 648, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "ParseTree/ParseNode.pyx":650
 *                 self.children[i], self.children[(i + 1) % len(self.children)] = \
 *                     self.children[(i + 1) % len(self.children)], self.children[i]
 *                 return             # <<<<<<<<<<<<<<
 *         for child in self.children:
 *             child.moveRight(node)
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "ParseTree/ParseNode.pyx":647
 *         cdef ParseNode child
 *         for i in range(len(self.children)):
 *             if self.children[i] == node:             # <<<<<<<<<<<<<<
 *                 self.children[i], self.children[(i + 1) % len(self.children)] = \
 *                     self.children[(i + 1) % len(self.children)], self.children[i]
 */
    }
  }

  /* "ParseTree/ParseNode.pyx":651
 *                     self.children[(i + 1) % len(self.children)], self.children[i]
 *                 return
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.moveRight(node)
 * 
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 651, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 651, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 651, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 651, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_child, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "ParseTree/ParseNode.pyx":652
 *                 return
 *         for child in self.children:
 *             child.moveRight(node)             # <<<<<<<<<<<<<<
 * 
 *     cpdef str ancestorString(self):
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_child->__pyx_vtab)->moveRight(__pyx_v_child, __pyx_v_node, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 652, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ParseTree/ParseNode.pyx":651
 *                     self.children[(i + 1) % len(self.children)], self.children[i]
 *                 return
 *         for child in self.children:             # <<<<<<<<<<<<<<
 *             child.moveRight(node)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ParseTree/ParseNode.pyx":634
 *             child.moveLeft(node)
 * 
 *     cpdef moveRight(self, ParseNode node):             # <<<<<<<<<<<<<<
 *         """
 *         Swaps the given child node of this node with the next sibling of that given node. If the given node is the
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.moveRight", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_59moveRight(PyObject *__pyx_v_self, PyObject *__pyx_v_node); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_58moveRight[] = "\n        Swaps the given child node of this node with the next sibling of that given node. If the given node is the\n        rightmost child, it swaps with the first node.\n\n        PARAMETERS\n        ----------\n        node : ParseNode\n            Node to be swapped.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_59moveRight(PyObject *__pyx_v_self, PyObject *__pyx_v_node) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("moveRight (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_node), __pyx_ptype_9ParseTree_9ParseNode_ParseNode, 1, "node", 0))) __PYX_ERR(0, 634, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_58moveRight(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_node));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_58moveRight(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_node) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("moveRight", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_moveRight(__pyx_v_self, __pyx_v_node, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.moveRight", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":654
 *             child.moveRight(node)
 * 
 *     cpdef str ancestorString(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive function to concatenate the data of the all ascendant nodes of this node to a string.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_61ancestorString(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_ancestorString(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ancestorString", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ancestorString); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_61ancestorString)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 654, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":663
 *             A string which contains all data of all the ascendant nodes of this node.
 *         """
 *         if self.parent is None:             # <<<<<<<<<<<<<<
 *             return self.data.getName()
 *         else:
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_self->parent) == Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "ParseTree/ParseNode.pyx":664
 *         """
 *         if self.parent is None:
 *             return self.data.getName()             # <<<<<<<<<<<<<<
 *         else:
 *             return self.parent.ancestorString() + self.data.getName()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ParseTree/ParseNode.pyx":663
 *             A string which contains all data of all the ascendant nodes of this node.
 *         """
 *         if self.parent is None:             # <<<<<<<<<<<<<<
 *             return self.data.getName()
 *         else:
 */
  }

  /* "ParseTree/ParseNode.pyx":666
 *             return self.data.getName()
 *         else:
 *             return self.parent.ancestorString() + self.data.getName()             # <<<<<<<<<<<<<<
 * 
 *     cpdef constituentSpanList(self, int startIndex, list constituentSpanList):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->parent->__pyx_vtab)->ancestorString(__pyx_v_self->parent, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = ((struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol *)__pyx_v_self->data->__pyx_base.__pyx_vtab)->__pyx_base.getName(((struct __pyx_obj_10Dictionary_4Word_Word *)__pyx_v_self->data), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyUnicode_ConcatSafe(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "ParseTree/ParseNode.pyx":654
 *             child.moveRight(node)
 * 
 *     cpdef str ancestorString(self):             # <<<<<<<<<<<<<<
 *         """
 *         Recursive function to concatenate the data of the all ascendant nodes of this node to a string.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.ancestorString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_61ancestorString(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_60ancestorString[] = "\n        Recursive function to concatenate the data of the all ascendant nodes of this node to a string.\n\n        RETURNS\n        -------\n        str\n            A string which contains all data of all the ascendant nodes of this node.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_61ancestorString(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ancestorString (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_60ancestorString(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_60ancestorString(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("ancestorString", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_ancestorString(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.ancestorString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ParseTree/ParseNode.pyx":668
 *             return self.parent.ancestorString() + self.data.getName()
 * 
 *     cpdef constituentSpanList(self, int startIndex, list constituentSpanList):             # <<<<<<<<<<<<<<
 *         """
 *         Construct recursively the constituent span list of a subtree rooted at this node.
 */

static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_63constituentSpanList(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9ParseTree_9ParseNode_9ParseNode_constituentSpanList(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_startIndex, PyObject *__pyx_v_constituentSpanList, int __pyx_skip_dispatch) {
  int __pyx_v_total;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_parseNode = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("constituentSpanList", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_constituentSpanList); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_63constituentSpanList)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_startIndex); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 668, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_constituentSpanList};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 668, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_3, __pyx_v_constituentSpanList};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 668, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 668, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_3);
          __Pyx_INCREF(__pyx_v_constituentSpanList);
          __Pyx_GIVEREF(__pyx_v_constituentSpanList);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_constituentSpanList);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 668, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ParseTree/ParseNode.pyx":681
 *         cdef int total
 *         cdef ParseNode parseNode
 *         if len(self.children) > 0:             # <<<<<<<<<<<<<<
 *             constituentSpanList.append(ConstituentSpan(self.data, startIndex, startIndex + self.leafCount()))
 *         total = 0
 */
  __pyx_t_1 = __pyx_v_self->children;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 681, __pyx_L1_error)
  }
  __pyx_t_8 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = ((__pyx_t_8 > 0) != 0);
  if (__pyx_t_9) {

    /* "ParseTree/ParseNode.pyx":682
 *         cdef ParseNode parseNode
 *         if len(self.children) > 0:
 *             constituentSpanList.append(ConstituentSpan(self.data, startIndex, startIndex + self.leafCount()))             # <<<<<<<<<<<<<<
 *         total = 0
 *         for parseNode in self.children:
 */
    if (unlikely(__pyx_v_constituentSpanList == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 682, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_startIndex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyInt_From_int((__pyx_v_startIndex + ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self->__pyx_vtab)->leafCount(__pyx_v_self, 0))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->data));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->data));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_self->data));
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9ParseTree_15ConstituentSpan_ConstituentSpan), __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_constituentSpanList, __pyx_t_2); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 682, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "ParseTree/ParseNode.pyx":681
 *         cdef int total
 *         cdef ParseNode parseNode
 *         if len(self.children) > 0:             # <<<<<<<<<<<<<<
 *             constituentSpanList.append(ConstituentSpan(self.data, startIndex, startIndex + self.leafCount()))
 *         total = 0
 */
  }

  /* "ParseTree/ParseNode.pyx":683
 *         if len(self.children) > 0:
 *             constituentSpanList.append(ConstituentSpan(self.data, startIndex, startIndex + self.leafCount()))
 *         total = 0             # <<<<<<<<<<<<<<
 *         for parseNode in self.children:
 *             parseNode.constituentSpanList(startIndex + total, constituentSpanList)
 */
  __pyx_v_total = 0;

  /* "ParseTree/ParseNode.pyx":684
 *             constituentSpanList.append(ConstituentSpan(self.data, startIndex, startIndex + self.leafCount()))
 *         total = 0
 *         for parseNode in self.children:             # <<<<<<<<<<<<<<
 *             parseNode.constituentSpanList(startIndex + total, constituentSpanList)
 *             total = total + parseNode.leafCount()
 */
  if (unlikely(__pyx_v_self->children == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 684, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_v_self->children; __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
  for (;;) {
    if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 684, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_parseNode, ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "ParseTree/ParseNode.pyx":685
 *         total = 0
 *         for parseNode in self.children:
 *             parseNode.constituentSpanList(startIndex + total, constituentSpanList)             # <<<<<<<<<<<<<<
 *             total = total + parseNode.leafCount()
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_parseNode->__pyx_vtab)->constituentSpanList(__pyx_v_parseNode, (__pyx_v_startIndex + __pyx_v_total), __pyx_v_constituentSpanList, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "ParseTree/ParseNode.pyx":686
 *         for parseNode in self.children:
 *             parseNode.constituentSpanList(startIndex + total, constituentSpanList)
 *             total = total + parseNode.leafCount()             # <<<<<<<<<<<<<<
 */
    __pyx_v_total = (__pyx_v_total + ((struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode *)__pyx_v_parseNode->__pyx_vtab)->leafCount(__pyx_v_parseNode, 0));

    /* "ParseTree/ParseNode.pyx":684
 *             constituentSpanList.append(ConstituentSpan(self.data, startIndex, startIndex + self.leafCount()))
 *         total = 0
 *         for parseNode in self.children:             # <<<<<<<<<<<<<<
 *             parseNode.constituentSpanList(startIndex + total, constituentSpanList)
 *             total = total + parseNode.leafCount()
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ParseTree/ParseNode.pyx":668
 *             return self.parent.ancestorString() + self.data.getName()
 * 
 *     cpdef constituentSpanList(self, int startIndex, list constituentSpanList):             # <<<<<<<<<<<<<<
 *         """
 *         Construct recursively the constituent span list of a subtree rooted at this node.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.constituentSpanList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_parseNode);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_63constituentSpanList(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9ParseTree_9ParseNode_9ParseNode_62constituentSpanList[] = "\n        Construct recursively the constituent span list of a subtree rooted at this node.\n\n        PARAMETERS\n        ----------\n        startIndex : int\n            Start index of the leftmost leaf node of this subtree.\n        constituentSpanList: list\n            Returned span list.\n        ";
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_63constituentSpanList(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_startIndex;
  PyObject *__pyx_v_constituentSpanList = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("constituentSpanList (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_startIndex,&__pyx_n_s_constituentSpanList,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_startIndex)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_constituentSpanList)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("constituentSpanList", 1, 2, 2, 1); __PYX_ERR(0, 668, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "constituentSpanList") < 0)) __PYX_ERR(0, 668, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_startIndex = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_startIndex == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 668, __pyx_L3_error)
    __pyx_v_constituentSpanList = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("constituentSpanList", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 668, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.constituentSpanList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_constituentSpanList), (&PyList_Type), 1, "constituentSpanList", 1))) __PYX_ERR(0, 668, __pyx_L1_error)
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_62constituentSpanList(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), __pyx_v_startIndex, __pyx_v_constituentSpanList);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_62constituentSpanList(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, int __pyx_v_startIndex, PyObject *__pyx_v_constituentSpanList) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("constituentSpanList", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode_9ParseNode_constituentSpanList(__pyx_v_self, __pyx_v_startIndex, __pyx_v_constituentSpanList, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.constituentSpanList", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_65__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_65__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_64__reduce_cython__(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_64__reduce_cython__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.children, self.data, self.parent)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->children);
  __Pyx_GIVEREF(__pyx_v_self->children);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->children);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->data));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->data));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->data));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->parent));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->parent));
  PyTuple_SET_ITEM(__pyx_t_1, 2, ((PyObject *)__pyx_v_self->parent));
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.children, self.data, self.parent)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.children, self.data, self.parent)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.children is not None or self.data is not None or self.parent is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.children, self.data, self.parent)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.children is not None or self.data is not None or self.parent is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, None), state
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_self->children != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_2 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_3 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->data) != Py_None);
    __pyx_t_2 = (__pyx_t_5 != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_2 = (((PyObject *)__pyx_v_self->parent) != Py_None);
    __pyx_t_5 = (__pyx_t_2 != 0);
    __pyx_t_3 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.children is not None or self.data is not None or self.parent is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.children is not None or self.data is not None or self.parent is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_ParseNode); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_108795736);
    __Pyx_GIVEREF(__pyx_int_108795736);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_108795736);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.children is not None or self.data is not None or self.parent is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, None), state
 *     else:
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ParseNode__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_ParseNode); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_108795736);
    __Pyx_GIVEREF(__pyx_int_108795736);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_108795736);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_6 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ParseNode__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_67__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9ParseTree_9ParseNode_9ParseNode_67__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode_9ParseNode_66__setstate_cython__(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode_9ParseNode_66__setstate_cython__(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ParseNode__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9ParseTree_9ParseNode___pyx_unpickle_ParseNode__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ParseNode, (type(self), 0x67c1758, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ParseNode__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ParseTree.ParseNode.ParseNode.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ParseNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9ParseTree_9ParseNode_1__pyx_unpickle_ParseNode(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9ParseTree_9ParseNode_1__pyx_unpickle_ParseNode = {"__pyx_unpickle_ParseNode", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9ParseTree_9ParseNode_1__pyx_unpickle_ParseNode, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9ParseTree_9ParseNode_1__pyx_unpickle_ParseNode(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ParseNode (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ParseNode", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ParseNode", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ParseNode") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ParseNode", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ParseTree.ParseNode.__pyx_unpickle_ParseNode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9ParseTree_9ParseNode___pyx_unpickle_ParseNode(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9ParseTree_9ParseNode___pyx_unpickle_ParseNode(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ParseNode", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x67c1758:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x67c1758) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x67c1758:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)
 *     __pyx_result = ParseNode.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x67c1758:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ParseNode.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x67, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x67c1758:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)
 *     __pyx_result = ParseNode.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)
 *     __pyx_result = ParseNode.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = ParseNode.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9ParseTree_9ParseNode___pyx_unpickle_ParseNode__set_state(((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x67c1758 = (children, data, parent))" % __pyx_checksum)
 *     __pyx_result = ParseNode.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ParseNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ParseTree.ParseNode.__pyx_unpickle_ParseNode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9ParseTree_9ParseNode___pyx_unpickle_ParseNode__set_state(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ParseNode__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->children);
  __Pyx_DECREF(__pyx_v___pyx_result->children);
  __pyx_v___pyx_result->children = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_6Symbol_Symbol))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->data);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->data));
  __pyx_v___pyx_result->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9ParseTree_9ParseNode_ParseNode))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->parent);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->parent));
  __pyx_v___pyx_result->parent = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 3) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ParseNode__set_state(<ParseNode> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ParseNode__set_state(ParseNode __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.children = __pyx_state[0]; __pyx_result.data = __pyx_state[1]; __pyx_result.parent = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ParseTree.ParseNode.__pyx_unpickle_ParseNode__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_9ParseTree_9ParseNode_ParseNode __pyx_vtable_9ParseTree_9ParseNode_ParseNode;

static PyObject *__pyx_tp_new_9ParseTree_9ParseNode_ParseNode(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)o);
  p->__pyx_vtab = __pyx_vtabptr_9ParseTree_9ParseNode_ParseNode;
  p->children = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->parent = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); Py_INCREF(Py_None);
  p->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9ParseTree_9ParseNode_ParseNode(PyObject *o) {
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *p = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->children);
  Py_CLEAR(p->parent);
  Py_CLEAR(p->data);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_9ParseTree_9ParseNode_ParseNode(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *p = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)o;
  if (p->children) {
    e = (*v)(p->children, a); if (e) return e;
  }
  if (p->parent) {
    e = (*v)(((PyObject *)p->parent), a); if (e) return e;
  }
  if (p->data) {
    e = (*v)(((PyObject *)p->data), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9ParseTree_9ParseNode_ParseNode(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *p = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)o;
  tmp = ((PyObject*)p->children);
  p->children = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->parent);
  p->parent = ((struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->data);
  p->data = ((struct __pyx_obj_9ParseTree_6Symbol_Symbol *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_9ParseTree_9ParseNode_ParseNode[] = {
  {"__searchHeadChild", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_3__searchHeadChild, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_2__searchHeadChild},
  {"headLeaf", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_5headLeaf, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_4headLeaf},
  {"headChild", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_7headChild, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_6headChild},
  {"addChild", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_9addChild, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_8addChild},
  {"correctParents", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_11correctParents, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_10correctParents},
  {"removeXNodes", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_13removeXNodes, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_12removeXNodes},
  {"setChild", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_15setChild, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_14setChild},
  {"removeChild", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_17removeChild, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_16removeChild},
  {"leafCount", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_19leafCount, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_18leafCount},
  {"nodeCount", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_21nodeCount, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_20nodeCount},
  {"nodeCountWithMultipleChildren", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_23nodeCountWithMultipleChildren, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_22nodeCountWithMultipleChildren},
  {"numberOfChildren", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_25numberOfChildren, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_24numberOfChildren},
  {"getChild", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_27getChild, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_26getChild},
  {"firstChild", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_29firstChild, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_28firstChild},
  {"lastChild", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_31lastChild, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_30lastChild},
  {"isLastChild", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_33isLastChild, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_32isLastChild},
  {"getChildIndex", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_35getChildIndex, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_34getChildIndex},
  {"isDescendant", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_37isDescendant, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_36isDescendant},
  {"previousSibling", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_39previousSibling, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_38previousSibling},
  {"nextSibling", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_41nextSibling, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_40nextSibling},
  {"getParent", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_43getParent, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_42getParent},
  {"getData", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_45getData, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_44getData},
  {"setData", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_47setData, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_46setData},
  {"wordCount", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_49wordCount, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_48wordCount},
  {"isLeaf", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_51isLeaf, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_50isLeaf},
  {"isDummyNode", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_53isDummyNode, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_52isDummyNode},
  {"moveLeft", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_57moveLeft, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_56moveLeft},
  {"moveRight", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_59moveRight, METH_O, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_58moveRight},
  {"ancestorString", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_61ancestorString, METH_NOARGS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_60ancestorString},
  {"constituentSpanList", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_63constituentSpanList, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9ParseTree_9ParseNode_9ParseNode_62constituentSpanList},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_65__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9ParseTree_9ParseNode_9ParseNode_67__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9ParseTree_9ParseNode_ParseNode = {
  PyVarObject_HEAD_INIT(0, 0)
  "ParseTree.ParseNode.ParseNode", /*tp_name*/
  sizeof(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9ParseTree_9ParseNode_ParseNode, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_9ParseTree_9ParseNode_9ParseNode_55__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9ParseTree_9ParseNode_ParseNode, /*tp_traverse*/
  __pyx_tp_clear_9ParseTree_9ParseNode_ParseNode, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9ParseTree_9ParseNode_ParseNode, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9ParseTree_9ParseNode_9ParseNode_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9ParseTree_9ParseNode_ParseNode, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_ParseNode(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_ParseNode},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "ParseNode",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_kp_u_0, __pyx_k_0, sizeof(__pyx_k_0), 0, 1, 0, 0},
  {&__pyx_n_s_ADJP, __pyx_k_ADJP, sizeof(__pyx_k_ADJP), 0, 0, 1, 1},
  {&__pyx_n_u_ADJP, __pyx_k_ADJP, sizeof(__pyx_k_ADJP), 0, 1, 0, 1},
  {&__pyx_n_s_ADVP, __pyx_k_ADVP, sizeof(__pyx_k_ADVP), 0, 0, 1, 1},
  {&__pyx_n_u_ADVP, __pyx_k_ADVP, sizeof(__pyx_k_ADVP), 0, 1, 0, 1},
  {&__pyx_n_u_CC, __pyx_k_CC, sizeof(__pyx_k_CC), 0, 1, 0, 1},
  {&__pyx_n_u_CD, __pyx_k_CD, sizeof(__pyx_k_CD), 0, 1, 0, 1},
  {&__pyx_n_s_CONJP, __pyx_k_CONJP, sizeof(__pyx_k_CONJP), 0, 0, 1, 1},
  {&__pyx_n_u_CONJP, __pyx_k_CONJP, sizeof(__pyx_k_CONJP), 0, 1, 0, 1},
  {&__pyx_n_u_DT, __pyx_k_DT, sizeof(__pyx_k_DT), 0, 1, 0, 1},
  {&__pyx_n_u_EX, __pyx_k_EX, sizeof(__pyx_k_EX), 0, 1, 0, 1},
  {&__pyx_n_s_FRAG, __pyx_k_FRAG, sizeof(__pyx_k_FRAG), 0, 0, 1, 1},
  {&__pyx_n_u_FRAG, __pyx_k_FRAG, sizeof(__pyx_k_FRAG), 0, 1, 0, 1},
  {&__pyx_n_u_FW, __pyx_k_FW, sizeof(__pyx_k_FW), 0, 1, 0, 1},
  {&__pyx_n_u_IN, __pyx_k_IN, sizeof(__pyx_k_IN), 0, 1, 0, 1},
  {&__pyx_n_s_INTJ, __pyx_k_INTJ, sizeof(__pyx_k_INTJ), 0, 0, 1, 1},
  {&__pyx_n_u_INTJ, __pyx_k_INTJ, sizeof(__pyx_k_INTJ), 0, 1, 0, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x67, __pyx_k_Incompatible_checksums_s_vs_0x67, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x67), 0, 0, 1, 0},
  {&__pyx_n_u_JJ, __pyx_k_JJ, sizeof(__pyx_k_JJ), 0, 1, 0, 1},
  {&__pyx_n_u_JJR, __pyx_k_JJR, sizeof(__pyx_k_JJR), 0, 1, 0, 1},
  {&__pyx_n_u_JJS, __pyx_k_JJS, sizeof(__pyx_k_JJS), 0, 1, 0, 1},
  {&__pyx_n_s_LEFT, __pyx_k_LEFT, sizeof(__pyx_k_LEFT), 0, 0, 1, 1},
  {&__pyx_n_u_LS, __pyx_k_LS, sizeof(__pyx_k_LS), 0, 1, 0, 1},
  {&__pyx_n_s_LST, __pyx_k_LST, sizeof(__pyx_k_LST), 0, 0, 1, 1},
  {&__pyx_n_u_LST, __pyx_k_LST, sizeof(__pyx_k_LST), 0, 1, 0, 1},
  {&__pyx_n_u_MD, __pyx_k_MD, sizeof(__pyx_k_MD), 0, 1, 0, 1},
  {&__pyx_n_s_NAC, __pyx_k_NAC, sizeof(__pyx_k_NAC), 0, 0, 1, 1},
  {&__pyx_n_u_NAC, __pyx_k_NAC, sizeof(__pyx_k_NAC), 0, 1, 0, 1},
  {&__pyx_n_u_NCD, __pyx_k_NCD, sizeof(__pyx_k_NCD), 0, 1, 0, 1},
  {&__pyx_n_u_NN, __pyx_k_NN, sizeof(__pyx_k_NN), 0, 1, 0, 1},
  {&__pyx_n_u_NNP, __pyx_k_NNP, sizeof(__pyx_k_NNP), 0, 1, 0, 1},
  {&__pyx_n_u_NNPS, __pyx_k_NNPS, sizeof(__pyx_k_NNPS), 0, 1, 0, 1},
  {&__pyx_n_u_NNS, __pyx_k_NNS, sizeof(__pyx_k_NNS), 0, 1, 0, 1},
  {&__pyx_kp_u_NONE, __pyx_k_NONE, sizeof(__pyx_k_NONE), 0, 1, 0, 0},
  {&__pyx_n_u_NP, __pyx_k_NP, sizeof(__pyx_k_NP), 0, 1, 0, 1},
  {&__pyx_n_s_NP1, __pyx_k_NP1, sizeof(__pyx_k_NP1), 0, 0, 1, 1},
  {&__pyx_n_s_NP2, __pyx_k_NP2, sizeof(__pyx_k_NP2), 0, 0, 1, 1},
  {&__pyx_n_s_NP3, __pyx_k_NP3, sizeof(__pyx_k_NP3), 0, 0, 1, 1},
  {&__pyx_n_s_NP4, __pyx_k_NP4, sizeof(__pyx_k_NP4), 0, 0, 1, 1},
  {&__pyx_n_s_NP5, __pyx_k_NP5, sizeof(__pyx_k_NP5), 0, 0, 1, 1},
  {&__pyx_n_u_NX, __pyx_k_NX, sizeof(__pyx_k_NX), 0, 1, 0, 1},
  {&__pyx_n_u_POS, __pyx_k_POS, sizeof(__pyx_k_POS), 0, 1, 0, 1},
  {&__pyx_n_s_PP, __pyx_k_PP, sizeof(__pyx_k_PP), 0, 0, 1, 1},
  {&__pyx_n_u_PP, __pyx_k_PP, sizeof(__pyx_k_PP), 0, 1, 0, 1},
  {&__pyx_n_s_PRN, __pyx_k_PRN, sizeof(__pyx_k_PRN), 0, 0, 1, 1},
  {&__pyx_n_u_PRN, __pyx_k_PRN, sizeof(__pyx_k_PRN), 0, 1, 0, 1},
  {&__pyx_n_u_PRP, __pyx_k_PRP, sizeof(__pyx_k_PRP), 0, 1, 0, 1},
  {&__pyx_n_s_PRT, __pyx_k_PRT, sizeof(__pyx_k_PRT), 0, 0, 1, 1},
  {&__pyx_n_u_PRT, __pyx_k_PRT, sizeof(__pyx_k_PRT), 0, 1, 0, 1},
  {&__pyx_n_s_ParseNode, __pyx_k_ParseNode, sizeof(__pyx_k_ParseNode), 0, 0, 1, 1},
  {&__pyx_n_s_ParseTree_ParseNode, __pyx_k_ParseTree_ParseNode, sizeof(__pyx_k_ParseTree_ParseNode), 0, 0, 1, 1},
  {&__pyx_n_s_ParseTree_SearchDirectionType, __pyx_k_ParseTree_SearchDirectionType, sizeof(__pyx_k_ParseTree_SearchDirectionType), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_QP, __pyx_k_QP, sizeof(__pyx_k_QP), 0, 0, 1, 1},
  {&__pyx_n_u_QP, __pyx_k_QP, sizeof(__pyx_k_QP), 0, 1, 0, 1},
  {&__pyx_n_u_RB, __pyx_k_RB, sizeof(__pyx_k_RB), 0, 1, 0, 1},
  {&__pyx_n_u_RBR, __pyx_k_RBR, sizeof(__pyx_k_RBR), 0, 1, 0, 1},
  {&__pyx_n_u_RBS, __pyx_k_RBS, sizeof(__pyx_k_RBS), 0, 1, 0, 1},
  {&__pyx_n_s_RIGHT, __pyx_k_RIGHT, sizeof(__pyx_k_RIGHT), 0, 0, 1, 1},
  {&__pyx_n_u_RP, __pyx_k_RP, sizeof(__pyx_k_RP), 0, 1, 0, 1},
  {&__pyx_n_s_RRC, __pyx_k_RRC, sizeof(__pyx_k_RRC), 0, 0, 1, 1},
  {&__pyx_n_u_RRC, __pyx_k_RRC, sizeof(__pyx_k_RRC), 0, 1, 0, 1},
  {&__pyx_n_s_S, __pyx_k_S, sizeof(__pyx_k_S), 0, 0, 1, 1},
  {&__pyx_n_u_S, __pyx_k_S, sizeof(__pyx_k_S), 0, 1, 0, 1},
  {&__pyx_n_s_SBAR, __pyx_k_SBAR, sizeof(__pyx_k_SBAR), 0, 0, 1, 1},
  {&__pyx_n_u_SBAR, __pyx_k_SBAR, sizeof(__pyx_k_SBAR), 0, 1, 0, 1},
  {&__pyx_n_s_SBARQ, __pyx_k_SBARQ, sizeof(__pyx_k_SBARQ), 0, 0, 1, 1},
  {&__pyx_n_u_SBARQ, __pyx_k_SBARQ, sizeof(__pyx_k_SBARQ), 0, 1, 0, 1},
  {&__pyx_n_s_SINV, __pyx_k_SINV, sizeof(__pyx_k_SINV), 0, 0, 1, 1},
  {&__pyx_n_u_SINV, __pyx_k_SINV, sizeof(__pyx_k_SINV), 0, 1, 0, 1},
  {&__pyx_n_s_SQ, __pyx_k_SQ, sizeof(__pyx_k_SQ), 0, 0, 1, 1},
  {&__pyx_n_u_SQ, __pyx_k_SQ, sizeof(__pyx_k_SQ), 0, 1, 0, 1},
  {&__pyx_n_s_SearchDirectionType, __pyx_k_SearchDirectionType, sizeof(__pyx_k_SearchDirectionType), 0, 0, 1, 1},
  {&__pyx_n_u_TO, __pyx_k_TO, sizeof(__pyx_k_TO), 0, 1, 0, 1},
  {&__pyx_n_s_UCP, __pyx_k_UCP, sizeof(__pyx_k_UCP), 0, 0, 1, 1},
  {&__pyx_n_u_UCP, __pyx_k_UCP, sizeof(__pyx_k_UCP), 0, 1, 0, 1},
  {&__pyx_n_u_VB, __pyx_k_VB, sizeof(__pyx_k_VB), 0, 1, 0, 1},
  {&__pyx_n_u_VBD, __pyx_k_VBD, sizeof(__pyx_k_VBD), 0, 1, 0, 1},
  {&__pyx_n_u_VBG, __pyx_k_VBG, sizeof(__pyx_k_VBG), 0, 1, 0, 1},
  {&__pyx_n_u_VBN, __pyx_k_VBN, sizeof(__pyx_k_VBN), 0, 1, 0, 1},
  {&__pyx_n_u_VBP, __pyx_k_VBP, sizeof(__pyx_k_VBP), 0, 1, 0, 1},
  {&__pyx_n_u_VBZ, __pyx_k_VBZ, sizeof(__pyx_k_VBZ), 0, 1, 0, 1},
  {&__pyx_n_s_VP, __pyx_k_VP, sizeof(__pyx_k_VP), 0, 0, 1, 1},
  {&__pyx_n_u_VP, __pyx_k_VP, sizeof(__pyx_k_VP), 0, 1, 0, 1},
  {&__pyx_n_u_WDT, __pyx_k_WDT, sizeof(__pyx_k_WDT), 0, 1, 0, 1},
  {&__pyx_n_s_WHADJP, __pyx_k_WHADJP, sizeof(__pyx_k_WHADJP), 0, 0, 1, 1},
  {&__pyx_n_u_WHADJP, __pyx_k_WHADJP, sizeof(__pyx_k_WHADJP), 0, 1, 0, 1},
  {&__pyx_n_s_WHADVP, __pyx_k_WHADVP, sizeof(__pyx_k_WHADVP), 0, 0, 1, 1},
  {&__pyx_n_u_WHADVP, __pyx_k_WHADVP, sizeof(__pyx_k_WHADVP), 0, 1, 0, 1},
  {&__pyx_n_s_WHNP, __pyx_k_WHNP, sizeof(__pyx_k_WHNP), 0, 0, 1, 1},
  {&__pyx_n_u_WHNP, __pyx_k_WHNP, sizeof(__pyx_k_WHNP), 0, 1, 0, 1},
  {&__pyx_n_s_WHPP, __pyx_k_WHPP, sizeof(__pyx_k_WHPP), 0, 0, 1, 1},
  {&__pyx_n_u_WHPP, __pyx_k_WHPP, sizeof(__pyx_k_WHPP), 0, 1, 0, 1},
  {&__pyx_n_u_WP, __pyx_k_WP, sizeof(__pyx_k_WP), 0, 1, 0, 1},
  {&__pyx_kp_u_WP_2, __pyx_k_WP_2, sizeof(__pyx_k_WP_2), 0, 1, 0, 0},
  {&__pyx_n_u_WRB, __pyx_k_WRB, sizeof(__pyx_k_WRB), 0, 1, 0, 1},
  {&__pyx_n_u_X, __pyx_k_X, sizeof(__pyx_k_X), 0, 1, 0, 1},
  {&__pyx_kp_u__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 1, 0, 0},
  {&__pyx_kp_u__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 1, 0, 0},
  {&__pyx_kp_u__4, __pyx_k__4, sizeof(__pyx_k__4), 0, 1, 0, 0},
  {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
  {&__pyx_kp_u__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0, 0},
  {&__pyx_kp_u__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 1, 0, 0},
  {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
  {&__pyx_n_u_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 1, 0, 1},
  {&__pyx_n_s_addChild, __pyx_k_addChild, sizeof(__pyx_k_addChild), 0, 0, 1, 1},
  {&__pyx_n_u_an, __pyx_k_an, sizeof(__pyx_k_an), 0, 1, 0, 1},
  {&__pyx_n_s_ancestorString, __pyx_k_ancestorString, sizeof(__pyx_k_ancestorString), 0, 0, 1, 1},
  {&__pyx_n_u_and, __pyx_k_and, sizeof(__pyx_k_and), 0, 1, 0, 1},
  {&__pyx_n_u_are, __pyx_k_are, sizeof(__pyx_k_are), 0, 1, 0, 1},
  {&__pyx_n_u_as, __pyx_k_as, sizeof(__pyx_k_as), 0, 1, 0, 1},
  {&__pyx_n_u_at, __pyx_k_at, sizeof(__pyx_k_at), 0, 1, 0, 1},
  {&__pyx_n_u_be, __pyx_k_be, sizeof(__pyx_k_be), 0, 1, 0, 1},
  {&__pyx_n_u_been, __pyx_k_been, sizeof(__pyx_k_been), 0, 1, 0, 1},
  {&__pyx_n_u_by, __pyx_k_by, sizeof(__pyx_k_by), 0, 1, 0, 1},
  {&__pyx_n_u_can, __pyx_k_can, sizeof(__pyx_k_can), 0, 1, 0, 1},
  {&__pyx_n_s_child, __pyx_k_child, sizeof(__pyx_k_child), 0, 0, 1, 1},
  {&__pyx_n_s_children, __pyx_k_children, sizeof(__pyx_k_children), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_constituentSpanList, __pyx_k_constituentSpanList, sizeof(__pyx_k_constituentSpanList), 0, 0, 1, 1},
  {&__pyx_n_s_correctParents, __pyx_k_correctParents, sizeof(__pyx_k_correctParents), 0, 0, 1, 1},
  {&__pyx_n_u_could, __pyx_k_could, sizeof(__pyx_k_could), 0, 1, 0, 1},
  {&__pyx_n_s_dataOrParent, __pyx_k_dataOrParent, sizeof(__pyx_k_dataOrParent), 0, 0, 1, 1},
  {&__pyx_n_s_defaultCase, __pyx_k_defaultCase, sizeof(__pyx_k_defaultCase), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_direction, __pyx_k_direction, sizeof(__pyx_k_direction), 0, 0, 1, 1},
  {&__pyx_n_u_do, __pyx_k_do, sizeof(__pyx_k_do), 0, 1, 0, 1},
  {&__pyx_n_u_down, __pyx_k_down, sizeof(__pyx_k_down), 0, 1, 0, 1},
  {&__pyx_n_u_few, __pyx_k_few, sizeof(__pyx_k_few), 0, 1, 0, 1},
  {&__pyx_n_s_firstChild, __pyx_k_firstChild, sizeof(__pyx_k_firstChild), 0, 0, 1, 1},
  {&__pyx_n_u_for, __pyx_k_for, sizeof(__pyx_k_for), 0, 1, 0, 1},
  {&__pyx_n_s_getChild, __pyx_k_getChild, sizeof(__pyx_k_getChild), 0, 0, 1, 1},
  {&__pyx_n_s_getChildIndex, __pyx_k_getChildIndex, sizeof(__pyx_k_getChildIndex), 0, 0, 1, 1},
  {&__pyx_n_s_getData, __pyx_k_getData, sizeof(__pyx_k_getData), 0, 0, 1, 1},
  {&__pyx_n_s_getName, __pyx_k_getName, sizeof(__pyx_k_getName), 0, 0, 1, 1},
  {&__pyx_n_s_getParent, __pyx_k_getParent, sizeof(__pyx_k_getParent), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_u_had, __pyx_k_had, sizeof(__pyx_k_had), 0, 1, 0, 1},
  {&__pyx_n_u_has, __pyx_k_has, sizeof(__pyx_k_has), 0, 1, 0, 1},
  {&__pyx_n_u_have, __pyx_k_have, sizeof(__pyx_k_have), 0, 1, 0, 1},
  {&__pyx_n_s_headChild, __pyx_k_headChild, sizeof(__pyx_k_headChild), 0, 0, 1, 1},
  {&__pyx_n_s_headLeaf, __pyx_k_headLeaf, sizeof(__pyx_k_headLeaf), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_u_in, __pyx_k_in, sizeof(__pyx_k_in), 0, 1, 0, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_u_is, __pyx_k_is, sizeof(__pyx_k_is), 0, 1, 0, 1},
  {&__pyx_n_s_isDescendant, __pyx_k_isDescendant, sizeof(__pyx_k_isDescendant), 0, 0, 1, 1},
  {&__pyx_n_s_isDummyNode, __pyx_k_isDummyNode, sizeof(__pyx_k_isDummyNode), 0, 0, 1, 1},
  {&__pyx_n_s_isLastChild, __pyx_k_isLastChild, sizeof(__pyx_k_isLastChild), 0, 0, 1, 1},
  {&__pyx_n_s_isLeaf, __pyx_k_isLeaf, sizeof(__pyx_k_isLeaf), 0, 0, 1, 1},
  {&__pyx_n_s_isPunctuationSymbol, __pyx_k_isPunctuationSymbol, sizeof(__pyx_k_isPunctuationSymbol), 0, 0, 1, 1},
  {&__pyx_n_s_lastChild, __pyx_k_lastChild, sizeof(__pyx_k_lastChild), 0, 0, 1, 1},
  {&__pyx_n_s_leafCount, __pyx_k_leafCount, sizeof(__pyx_k_leafCount), 0, 0, 1, 1},
  {&__pyx_n_s_leftOrLine, __pyx_k_leftOrLine, sizeof(__pyx_k_leftOrLine), 0, 0, 1, 1},
  {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_u_may, __pyx_k_may, sizeof(__pyx_k_may), 0, 1, 0, 1},
  {&__pyx_n_u_might, __pyx_k_might, sizeof(__pyx_k_might), 0, 1, 0, 1},
  {&__pyx_n_s_moveLeft, __pyx_k_moveLeft, sizeof(__pyx_k_moveLeft), 0, 0, 1, 1},
  {&__pyx_n_s_moveRight, __pyx_k_moveRight, sizeof(__pyx_k_moveRight), 0, 0, 1, 1},
  {&__pyx_kp_u_n_t, __pyx_k_n_t, sizeof(__pyx_k_n_t), 0, 1, 0, 0},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_nextSibling, __pyx_k_nextSibling, sizeof(__pyx_k_nextSibling), 0, 0, 1, 1},
  {&__pyx_n_s_nodeCount, __pyx_k_nodeCount, sizeof(__pyx_k_nodeCount), 0, 0, 1, 1},
  {&__pyx_n_s_nodeCountWithMultipleChildren, __pyx_k_nodeCountWithMultipleChildren, sizeof(__pyx_k_nodeCountWithMultipleChildren), 0, 0, 1, 1},
  {&__pyx_n_u_not, __pyx_k_not, sizeof(__pyx_k_not), 0, 1, 0, 1},
  {&__pyx_n_s_numberOfChildren, __pyx_k_numberOfChildren, sizeof(__pyx_k_numberOfChildren), 0, 0, 1, 1},
  {&__pyx_n_u_of, __pyx_k_of, sizeof(__pyx_k_of), 0, 1, 0, 1},
  {&__pyx_n_u_off, __pyx_k_off, sizeof(__pyx_k_off), 0, 1, 0, 1},
  {&__pyx_n_u_on, __pyx_k_on, sizeof(__pyx_k_on), 0, 1, 0, 1},
  {&__pyx_n_u_or, __pyx_k_or, sizeof(__pyx_k_or), 0, 1, 0, 1},
  {&__pyx_n_s_parent, __pyx_k_parent, sizeof(__pyx_k_parent), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_previousSibling, __pyx_k_previousSibling, sizeof(__pyx_k_previousSibling), 0, 0, 1, 1},
  {&__pyx_n_s_priorityList, __pyx_k_priorityList, sizeof(__pyx_k_priorityList), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ParseNode, __pyx_k_pyx_unpickle_ParseNode, sizeof(__pyx_k_pyx_unpickle_ParseNode), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_kp_u_re, __pyx_k_re, sizeof(__pyx_k_re), 0, 1, 0, 0},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_remove, __pyx_k_remove, sizeof(__pyx_k_remove), 0, 0, 1, 1},
  {&__pyx_n_s_removeChild, __pyx_k_removeChild, sizeof(__pyx_k_removeChild), 0, 0, 1, 1},
  {&__pyx_n_s_removeXNodes, __pyx_k_removeXNodes, sizeof(__pyx_k_removeXNodes), 0, 0, 1, 1},
  {&__pyx_n_s_rightOrIsLeaf, __pyx_k_rightOrIsLeaf, sizeof(__pyx_k_rightOrIsLeaf), 0, 0, 1, 1},
  {&__pyx_n_s_rindex, __pyx_k_rindex, sizeof(__pyx_k_rindex), 0, 0, 1, 1},
  {&__pyx_kp_u_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 1, 0, 0},
  {&__pyx_n_s_searchHeadChild, __pyx_k_searchHeadChild, sizeof(__pyx_k_searchHeadChild), 0, 0, 1, 1},
  {&__pyx_n_s_setChild, __pyx_k_setChild, sizeof(__pyx_k_setChild), 0, 0, 1, 1},
  {&__pyx_n_s_setData, __pyx_k_setData, sizeof(__pyx_k_setData), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_startIndex, __pyx_k_startIndex, sizeof(__pyx_k_startIndex), 0, 0, 1, 1},
  {&__pyx_n_s_startswith, __pyx_k_startswith, sizeof(__pyx_k_startswith), 0, 0, 1, 1},
  {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_u_than, __pyx_k_than, sizeof(__pyx_k_than), 0, 1, 0, 1},
  {&__pyx_n_u_the, __pyx_k_the, sizeof(__pyx_k_the), 0, 1, 0, 1},
  {&__pyx_n_u_there, __pyx_k_there, sizeof(__pyx_k_there), 0, 1, 0, 1},
  {&__pyx_n_u_to, __pyx_k_to, sizeof(__pyx_k_to), 0, 1, 0, 1},
  {&__pyx_n_u_up, __pyx_k_up, sizeof(__pyx_k_up), 0, 1, 0, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_u_was, __pyx_k_was, sizeof(__pyx_k_was), 0, 1, 0, 1},
  {&__pyx_n_u_were, __pyx_k_were, sizeof(__pyx_k_were), 0, 1, 0, 1},
  {&__pyx_n_u_will, __pyx_k_will, sizeof(__pyx_k_will), 0, 1, 0, 1},
  {&__pyx_n_u_with, __pyx_k_with, sizeof(__pyx_k_with), 0, 1, 0, 1},
  {&__pyx_n_s_wordCount, __pyx_k_wordCount, sizeof(__pyx_k_wordCount), 0, 0, 1, 1},
  {&__pyx_n_u_would, __pyx_k_would, sizeof(__pyx_k_would), 0, 1, 0, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 83, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":1
 * def __pyx_unpickle_ParseNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__9 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ParseNode, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyUnicode_Type_strip.type = (PyObject*)&PyUnicode_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_108795736 = PyInt_FromLong(108795736L); if (unlikely(!__pyx_int_108795736)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_9ParseTree_9ParseNode_ParseNode = &__pyx_vtable_9ParseTree_9ParseNode_ParseNode;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.__pyx___searchHeadChild = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, PyObject *, PyObject *, int, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode___searchHeadChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.headLeaf = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_headLeaf;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.headChild = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_headChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.addChild = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch, struct __pyx_opt_args_9ParseTree_9ParseNode_9ParseNode_addChild *__pyx_optional_args))__pyx_f_9ParseTree_9ParseNode_9ParseNode_addChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.correctParents = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_correctParents;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.removeXNodes = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_removeXNodes;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.setChild = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_setChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.removeChild = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_removeChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.leafCount = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_leafCount;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.nodeCount = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCount;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.nodeCountWithMultipleChildren = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_nodeCountWithMultipleChildren;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.numberOfChildren = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_numberOfChildren;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.getChild = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_getChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.firstChild = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_firstChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.lastChild = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_lastChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.isLastChild = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_isLastChild;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.getChildIndex = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_getChildIndex;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.isDescendant = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_isDescendant;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.previousSibling = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_previousSibling;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.nextSibling = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_nextSibling;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.getParent = (struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_getParent;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.getData = (struct __pyx_obj_9ParseTree_6Symbol_Symbol *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_getData;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.setData = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_6Symbol_Symbol *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_setData;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.wordCount = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_wordCount;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.isLeaf = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_isLeaf;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.isDummyNode = (int (*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_isDummyNode;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.moveLeft = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_moveLeft;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.moveRight = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_moveRight;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.ancestorString = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_ancestorString;
  __pyx_vtable_9ParseTree_9ParseNode_ParseNode.constituentSpanList = (PyObject *(*)(struct __pyx_obj_9ParseTree_9ParseNode_ParseNode *, int, PyObject *, int __pyx_skip_dispatch))__pyx_f_9ParseTree_9ParseNode_9ParseNode_constituentSpanList;
  if (PyType_Ready(&__pyx_type_9ParseTree_9ParseNode_ParseNode) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_9ParseTree_9ParseNode_ParseNode.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_9ParseTree_9ParseNode_ParseNode.tp_dictoffset && __pyx_type_9ParseTree_9ParseNode_ParseNode.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_9ParseTree_9ParseNode_ParseNode.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_9ParseTree_9ParseNode_ParseNode, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 7, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode___init__.doc = __pyx_doc_9ParseTree_9ParseNode_9ParseNode___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode___init__;
    }
  }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_9ParseTree_9ParseNode_ParseNode, "__str__"); if (unlikely(!wrapper)) __PYX_ERR(0, 7, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode_54__str__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode_54__str__.doc = __pyx_doc_9ParseTree_9ParseNode_9ParseNode_54__str__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_9ParseTree_9ParseNode_9ParseNode_54__str__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_9ParseTree_9ParseNode_ParseNode.tp_dict, __pyx_vtabptr_9ParseTree_9ParseNode_ParseNode) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ParseNode, (PyObject *)&__pyx_type_9ParseTree_9ParseNode_ParseNode) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9ParseTree_9ParseNode_ParseNode) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __pyx_ptype_9ParseTree_9ParseNode_ParseNode = &__pyx_type_9ParseTree_9ParseNode_ParseNode;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("Dictionary.Word"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_10Dictionary_4Word_Word = __Pyx_ImportType(__pyx_t_1, "Dictionary.Word", "Word", sizeof(struct __pyx_obj_10Dictionary_4Word_Word), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_10Dictionary_4Word_Word) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_vtabptr_10Dictionary_4Word_Word = (struct __pyx_vtabstruct_10Dictionary_4Word_Word*)__Pyx_GetVtable(__pyx_ptype_10Dictionary_4Word_Word->tp_dict); if (unlikely(!__pyx_vtabptr_10Dictionary_4Word_Word)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ParseTree.Symbol"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_9ParseTree_6Symbol_Symbol = __Pyx_ImportType(__pyx_t_1, "ParseTree.Symbol", "Symbol", sizeof(struct __pyx_obj_9ParseTree_6Symbol_Symbol), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_9ParseTree_6Symbol_Symbol) __PYX_ERR(3, 4, __pyx_L1_error)
  __pyx_vtabptr_9ParseTree_6Symbol_Symbol = (struct __pyx_vtabstruct_9ParseTree_6Symbol_Symbol*)__Pyx_GetVtable(__pyx_ptype_9ParseTree_6Symbol_Symbol->tp_dict); if (unlikely(!__pyx_vtabptr_9ParseTree_6Symbol_Symbol)) __PYX_ERR(3, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("ParseTree.ConstituentSpan"); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_9ParseTree_15ConstituentSpan_ConstituentSpan = __Pyx_ImportType(__pyx_t_1, "ParseTree.ConstituentSpan", "ConstituentSpan", sizeof(struct __pyx_obj_9ParseTree_15ConstituentSpan_ConstituentSpan), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_9ParseTree_15ConstituentSpan_ConstituentSpan) __PYX_ERR(4, 4, __pyx_L1_error)
  __pyx_vtabptr_9ParseTree_15ConstituentSpan_ConstituentSpan = (struct __pyx_vtabstruct_9ParseTree_15ConstituentSpan_ConstituentSpan*)__Pyx_GetVtable(__pyx_ptype_9ParseTree_15ConstituentSpan_ConstituentSpan->tp_dict); if (unlikely(!__pyx_vtabptr_9ParseTree_15ConstituentSpan_ConstituentSpan)) __PYX_ERR(4, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initParseNode(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initParseNode(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_ParseNode(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_ParseNode(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_ParseNode(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'ParseNode' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_ParseNode(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("ParseNode", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ParseTree__ParseNode) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ParseTree.ParseNode")) {
      if (unlikely(PyDict_SetItemString(modules, "ParseTree.ParseNode", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ParseTree/ParseNode.pyx":4
 * 
 * from ParseTree.ConstituentSpan cimport ConstituentSpan
 * from ParseTree.SearchDirectionType import SearchDirectionType             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_SearchDirectionType);
  __Pyx_GIVEREF(__pyx_n_s_SearchDirectionType);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_SearchDirectionType);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_ParseTree_SearchDirectionType, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_SearchDirectionType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SearchDirectionType, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ParseTree/ParseNode.pyx":9
 * cdef class ParseNode:
 * 
 *     ADJP = ["NNS", "QP", "NN", "$", "ADVP", "JJ", "VBN", "VBG", "ADJP", "JJR", "NP", "JJS", "DT", "FW", "RBR", "RBS",             # <<<<<<<<<<<<<<
 *             "SBAR", "RB"]
 *     ADVP = ["RB", "RBR", "RBS", "FW", "ADVP", "TO", "CD", "JJR", "JJ", "IN", "NP", "JJS", "NN"]
 */
  __pyx_t_2 = PyList_New(18); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_NNS);
  __Pyx_GIVEREF(__pyx_n_u_NNS);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_NNS);
  __Pyx_INCREF(__pyx_n_u_QP);
  __Pyx_GIVEREF(__pyx_n_u_QP);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_QP);
  __Pyx_INCREF(__pyx_n_u_NN);
  __Pyx_GIVEREF(__pyx_n_u_NN);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_NN);
  __Pyx_INCREF(__pyx_kp_u__6);
  __Pyx_GIVEREF(__pyx_kp_u__6);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_kp_u__6);
  __Pyx_INCREF(__pyx_n_u_ADVP);
  __Pyx_GIVEREF(__pyx_n_u_ADVP);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_ADVP);
  __Pyx_INCREF(__pyx_n_u_JJ);
  __Pyx_GIVEREF(__pyx_n_u_JJ);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_JJ);
  __Pyx_INCREF(__pyx_n_u_VBN);
  __Pyx_GIVEREF(__pyx_n_u_VBN);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_VBN);
  __Pyx_INCREF(__pyx_n_u_VBG);
  __Pyx_GIVEREF(__pyx_n_u_VBG);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_VBG);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_JJR);
  __Pyx_GIVEREF(__pyx_n_u_JJR);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_JJR);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_NP);
  __Pyx_INCREF(__pyx_n_u_JJS);
  __Pyx_GIVEREF(__pyx_n_u_JJS);
  PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_u_JJS);
  __Pyx_INCREF(__pyx_n_u_DT);
  __Pyx_GIVEREF(__pyx_n_u_DT);
  PyList_SET_ITEM(__pyx_t_2, 12, __pyx_n_u_DT);
  __Pyx_INCREF(__pyx_n_u_FW);
  __Pyx_GIVEREF(__pyx_n_u_FW);
  PyList_SET_ITEM(__pyx_t_2, 13, __pyx_n_u_FW);
  __Pyx_INCREF(__pyx_n_u_RBR);
  __Pyx_GIVEREF(__pyx_n_u_RBR);
  PyList_SET_ITEM(__pyx_t_2, 14, __pyx_n_u_RBR);
  __Pyx_INCREF(__pyx_n_u_RBS);
  __Pyx_GIVEREF(__pyx_n_u_RBS);
  PyList_SET_ITEM(__pyx_t_2, 15, __pyx_n_u_RBS);
  __Pyx_INCREF(__pyx_n_u_SBAR);
  __Pyx_GIVEREF(__pyx_n_u_SBAR);
  PyList_SET_ITEM(__pyx_t_2, 16, __pyx_n_u_SBAR);
  __Pyx_INCREF(__pyx_n_u_RB);
  __Pyx_GIVEREF(__pyx_n_u_RB);
  PyList_SET_ITEM(__pyx_t_2, 17, __pyx_n_u_RB);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_ADJP, __pyx_t_2) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":11
 *     ADJP = ["NNS", "QP", "NN", "$", "ADVP", "JJ", "VBN", "VBG", "ADJP", "JJR", "NP", "JJS", "DT", "FW", "RBR", "RBS",
 *             "SBAR", "RB"]
 *     ADVP = ["RB", "RBR", "RBS", "FW", "ADVP", "TO", "CD", "JJR", "JJ", "IN", "NP", "JJS", "NN"]             # <<<<<<<<<<<<<<
 *     CONJP = ["CC", "RB", "IN"]
 *     FRAG = []
 */
  __pyx_t_2 = PyList_New(13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_RB);
  __Pyx_GIVEREF(__pyx_n_u_RB);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_RB);
  __Pyx_INCREF(__pyx_n_u_RBR);
  __Pyx_GIVEREF(__pyx_n_u_RBR);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_RBR);
  __Pyx_INCREF(__pyx_n_u_RBS);
  __Pyx_GIVEREF(__pyx_n_u_RBS);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_RBS);
  __Pyx_INCREF(__pyx_n_u_FW);
  __Pyx_GIVEREF(__pyx_n_u_FW);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_FW);
  __Pyx_INCREF(__pyx_n_u_ADVP);
  __Pyx_GIVEREF(__pyx_n_u_ADVP);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_ADVP);
  __Pyx_INCREF(__pyx_n_u_TO);
  __Pyx_GIVEREF(__pyx_n_u_TO);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_TO);
  __Pyx_INCREF(__pyx_n_u_CD);
  __Pyx_GIVEREF(__pyx_n_u_CD);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_CD);
  __Pyx_INCREF(__pyx_n_u_JJR);
  __Pyx_GIVEREF(__pyx_n_u_JJR);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_JJR);
  __Pyx_INCREF(__pyx_n_u_JJ);
  __Pyx_GIVEREF(__pyx_n_u_JJ);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_JJ);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_IN);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_NP);
  __Pyx_INCREF(__pyx_n_u_JJS);
  __Pyx_GIVEREF(__pyx_n_u_JJS);
  PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_u_JJS);
  __Pyx_INCREF(__pyx_n_u_NN);
  __Pyx_GIVEREF(__pyx_n_u_NN);
  PyList_SET_ITEM(__pyx_t_2, 12, __pyx_n_u_NN);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_ADVP, __pyx_t_2) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":12
 *             "SBAR", "RB"]
 *     ADVP = ["RB", "RBR", "RBS", "FW", "ADVP", "TO", "CD", "JJR", "JJ", "IN", "NP", "JJS", "NN"]
 *     CONJP = ["CC", "RB", "IN"]             # <<<<<<<<<<<<<<
 *     FRAG = []
 *     INTJ = []
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_CC);
  __Pyx_GIVEREF(__pyx_n_u_CC);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_CC);
  __Pyx_INCREF(__pyx_n_u_RB);
  __Pyx_GIVEREF(__pyx_n_u_RB);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_RB);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_IN);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_CONJP, __pyx_t_2) < 0) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":13
 *     ADVP = ["RB", "RBR", "RBS", "FW", "ADVP", "TO", "CD", "JJR", "JJ", "IN", "NP", "JJS", "NN"]
 *     CONJP = ["CC", "RB", "IN"]
 *     FRAG = []             # <<<<<<<<<<<<<<
 *     INTJ = []
 *     LST = ["LS", ":"]
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_FRAG, __pyx_t_2) < 0) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":14
 *     CONJP = ["CC", "RB", "IN"]
 *     FRAG = []
 *     INTJ = []             # <<<<<<<<<<<<<<
 *     LST = ["LS", ":"]
 *     NAC = ["NN", "NNS", "NNP", "NNPS", "NP", "NAC", "EX", "$", "CD", "QP", "PRP", "VBG", "JJ", "JJS", "JJR", "ADJP",
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_INTJ, __pyx_t_2) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":15
 *     FRAG = []
 *     INTJ = []
 *     LST = ["LS", ":"]             # <<<<<<<<<<<<<<
 *     NAC = ["NN", "NNS", "NNP", "NNPS", "NP", "NAC", "EX", "$", "CD", "QP", "PRP", "VBG", "JJ", "JJS", "JJR", "ADJP",
 *            "FW"]
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_LS);
  __Pyx_GIVEREF(__pyx_n_u_LS);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_LS);
  __Pyx_INCREF(__pyx_kp_u__8);
  __Pyx_GIVEREF(__pyx_kp_u__8);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_kp_u__8);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_LST, __pyx_t_2) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":16
 *     INTJ = []
 *     LST = ["LS", ":"]
 *     NAC = ["NN", "NNS", "NNP", "NNPS", "NP", "NAC", "EX", "$", "CD", "QP", "PRP", "VBG", "JJ", "JJS", "JJR", "ADJP",             # <<<<<<<<<<<<<<
 *            "FW"]
 *     PP = ["IN", "TO", "VBG", "VBN", "RP", "FW"]
 */
  __pyx_t_2 = PyList_New(17); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_NN);
  __Pyx_GIVEREF(__pyx_n_u_NN);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_NN);
  __Pyx_INCREF(__pyx_n_u_NNS);
  __Pyx_GIVEREF(__pyx_n_u_NNS);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_NNS);
  __Pyx_INCREF(__pyx_n_u_NNP);
  __Pyx_GIVEREF(__pyx_n_u_NNP);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_NNP);
  __Pyx_INCREF(__pyx_n_u_NNPS);
  __Pyx_GIVEREF(__pyx_n_u_NNPS);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_NNPS);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_NP);
  __Pyx_INCREF(__pyx_n_u_NAC);
  __Pyx_GIVEREF(__pyx_n_u_NAC);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_NAC);
  __Pyx_INCREF(__pyx_n_u_EX);
  __Pyx_GIVEREF(__pyx_n_u_EX);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_EX);
  __Pyx_INCREF(__pyx_kp_u__6);
  __Pyx_GIVEREF(__pyx_kp_u__6);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_kp_u__6);
  __Pyx_INCREF(__pyx_n_u_CD);
  __Pyx_GIVEREF(__pyx_n_u_CD);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_CD);
  __Pyx_INCREF(__pyx_n_u_QP);
  __Pyx_GIVEREF(__pyx_n_u_QP);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_QP);
  __Pyx_INCREF(__pyx_n_u_PRP);
  __Pyx_GIVEREF(__pyx_n_u_PRP);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_PRP);
  __Pyx_INCREF(__pyx_n_u_VBG);
  __Pyx_GIVEREF(__pyx_n_u_VBG);
  PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_u_VBG);
  __Pyx_INCREF(__pyx_n_u_JJ);
  __Pyx_GIVEREF(__pyx_n_u_JJ);
  PyList_SET_ITEM(__pyx_t_2, 12, __pyx_n_u_JJ);
  __Pyx_INCREF(__pyx_n_u_JJS);
  __Pyx_GIVEREF(__pyx_n_u_JJS);
  PyList_SET_ITEM(__pyx_t_2, 13, __pyx_n_u_JJS);
  __Pyx_INCREF(__pyx_n_u_JJR);
  __Pyx_GIVEREF(__pyx_n_u_JJR);
  PyList_SET_ITEM(__pyx_t_2, 14, __pyx_n_u_JJR);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 15, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_FW);
  __Pyx_GIVEREF(__pyx_n_u_FW);
  PyList_SET_ITEM(__pyx_t_2, 16, __pyx_n_u_FW);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_NAC, __pyx_t_2) < 0) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":18
 *     NAC = ["NN", "NNS", "NNP", "NNPS", "NP", "NAC", "EX", "$", "CD", "QP", "PRP", "VBG", "JJ", "JJS", "JJR", "ADJP",
 *            "FW"]
 *     PP = ["IN", "TO", "VBG", "VBN", "RP", "FW"]             # <<<<<<<<<<<<<<
 *     PRN = []
 *     PRT = ["RP"]
 */
  __pyx_t_2 = PyList_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_IN);
  __Pyx_INCREF(__pyx_n_u_TO);
  __Pyx_GIVEREF(__pyx_n_u_TO);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_TO);
  __Pyx_INCREF(__pyx_n_u_VBG);
  __Pyx_GIVEREF(__pyx_n_u_VBG);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_VBG);
  __Pyx_INCREF(__pyx_n_u_VBN);
  __Pyx_GIVEREF(__pyx_n_u_VBN);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_VBN);
  __Pyx_INCREF(__pyx_n_u_RP);
  __Pyx_GIVEREF(__pyx_n_u_RP);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_RP);
  __Pyx_INCREF(__pyx_n_u_FW);
  __Pyx_GIVEREF(__pyx_n_u_FW);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_FW);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_PP, __pyx_t_2) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":19
 *            "FW"]
 *     PP = ["IN", "TO", "VBG", "VBN", "RP", "FW"]
 *     PRN = []             # <<<<<<<<<<<<<<
 *     PRT = ["RP"]
 *     QP = ["$", "IN", "NNS", "NN", "JJ", "RB", "DT", "CD", "NCD", "QP", "JJR", "JJS"]
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_PRN, __pyx_t_2) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":20
 *     PP = ["IN", "TO", "VBG", "VBN", "RP", "FW"]
 *     PRN = []
 *     PRT = ["RP"]             # <<<<<<<<<<<<<<
 *     QP = ["$", "IN", "NNS", "NN", "JJ", "RB", "DT", "CD", "NCD", "QP", "JJR", "JJS"]
 *     RRC = ["VP", "NP", "ADVP", "ADJP", "PP"]
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_RP);
  __Pyx_GIVEREF(__pyx_n_u_RP);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_RP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_PRT, __pyx_t_2) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":21
 *     PRN = []
 *     PRT = ["RP"]
 *     QP = ["$", "IN", "NNS", "NN", "JJ", "RB", "DT", "CD", "NCD", "QP", "JJR", "JJS"]             # <<<<<<<<<<<<<<
 *     RRC = ["VP", "NP", "ADVP", "ADJP", "PP"]
 *     S = ["TO", "IN", "VP", "S", "SBAR", "ADJP", "UCP", "NP"]
 */
  __pyx_t_2 = PyList_New(12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_kp_u__6);
  __Pyx_GIVEREF(__pyx_kp_u__6);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u__6);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_IN);
  __Pyx_INCREF(__pyx_n_u_NNS);
  __Pyx_GIVEREF(__pyx_n_u_NNS);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_NNS);
  __Pyx_INCREF(__pyx_n_u_NN);
  __Pyx_GIVEREF(__pyx_n_u_NN);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_NN);
  __Pyx_INCREF(__pyx_n_u_JJ);
  __Pyx_GIVEREF(__pyx_n_u_JJ);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_JJ);
  __Pyx_INCREF(__pyx_n_u_RB);
  __Pyx_GIVEREF(__pyx_n_u_RB);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_RB);
  __Pyx_INCREF(__pyx_n_u_DT);
  __Pyx_GIVEREF(__pyx_n_u_DT);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_DT);
  __Pyx_INCREF(__pyx_n_u_CD);
  __Pyx_GIVEREF(__pyx_n_u_CD);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_CD);
  __Pyx_INCREF(__pyx_n_u_NCD);
  __Pyx_GIVEREF(__pyx_n_u_NCD);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_NCD);
  __Pyx_INCREF(__pyx_n_u_QP);
  __Pyx_GIVEREF(__pyx_n_u_QP);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_QP);
  __Pyx_INCREF(__pyx_n_u_JJR);
  __Pyx_GIVEREF(__pyx_n_u_JJR);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_JJR);
  __Pyx_INCREF(__pyx_n_u_JJS);
  __Pyx_GIVEREF(__pyx_n_u_JJS);
  PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_u_JJS);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_QP, __pyx_t_2) < 0) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":22
 *     PRT = ["RP"]
 *     QP = ["$", "IN", "NNS", "NN", "JJ", "RB", "DT", "CD", "NCD", "QP", "JJR", "JJS"]
 *     RRC = ["VP", "NP", "ADVP", "ADJP", "PP"]             # <<<<<<<<<<<<<<
 *     S = ["TO", "IN", "VP", "S", "SBAR", "ADJP", "UCP", "NP"]
 *     SBAR = ["WHNP", "WHPP", "WHADVP", "WHADJP", "IN", "DT", "S", "SQ", "SINV", "SBAR", "FRAG"]
 */
  __pyx_t_2 = PyList_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_VP);
  __Pyx_GIVEREF(__pyx_n_u_VP);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_VP);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_NP);
  __Pyx_INCREF(__pyx_n_u_ADVP);
  __Pyx_GIVEREF(__pyx_n_u_ADVP);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_ADVP);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_PP);
  __Pyx_GIVEREF(__pyx_n_u_PP);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_PP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_RRC, __pyx_t_2) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":23
 *     QP = ["$", "IN", "NNS", "NN", "JJ", "RB", "DT", "CD", "NCD", "QP", "JJR", "JJS"]
 *     RRC = ["VP", "NP", "ADVP", "ADJP", "PP"]
 *     S = ["TO", "IN", "VP", "S", "SBAR", "ADJP", "UCP", "NP"]             # <<<<<<<<<<<<<<
 *     SBAR = ["WHNP", "WHPP", "WHADVP", "WHADJP", "IN", "DT", "S", "SQ", "SINV", "SBAR", "FRAG"]
 *     SBARQ = ["SQ", "S", "SINV", "SBARQ", "FRAG"]
 */
  __pyx_t_2 = PyList_New(8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_TO);
  __Pyx_GIVEREF(__pyx_n_u_TO);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_TO);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_IN);
  __Pyx_INCREF(__pyx_n_u_VP);
  __Pyx_GIVEREF(__pyx_n_u_VP);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_VP);
  __Pyx_INCREF(__pyx_n_u_S);
  __Pyx_GIVEREF(__pyx_n_u_S);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_S);
  __Pyx_INCREF(__pyx_n_u_SBAR);
  __Pyx_GIVEREF(__pyx_n_u_SBAR);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_SBAR);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_UCP);
  __Pyx_GIVEREF(__pyx_n_u_UCP);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_UCP);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_NP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_S, __pyx_t_2) < 0) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":24
 *     RRC = ["VP", "NP", "ADVP", "ADJP", "PP"]
 *     S = ["TO", "IN", "VP", "S", "SBAR", "ADJP", "UCP", "NP"]
 *     SBAR = ["WHNP", "WHPP", "WHADVP", "WHADJP", "IN", "DT", "S", "SQ", "SINV", "SBAR", "FRAG"]             # <<<<<<<<<<<<<<
 *     SBARQ = ["SQ", "S", "SINV", "SBARQ", "FRAG"]
 *     SINV = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "S", "SINV", "ADJP", "NP"]
 */
  __pyx_t_2 = PyList_New(11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_WHNP);
  __Pyx_GIVEREF(__pyx_n_u_WHNP);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_WHNP);
  __Pyx_INCREF(__pyx_n_u_WHPP);
  __Pyx_GIVEREF(__pyx_n_u_WHPP);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_WHPP);
  __Pyx_INCREF(__pyx_n_u_WHADVP);
  __Pyx_GIVEREF(__pyx_n_u_WHADVP);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_WHADVP);
  __Pyx_INCREF(__pyx_n_u_WHADJP);
  __Pyx_GIVEREF(__pyx_n_u_WHADJP);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_WHADJP);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_IN);
  __Pyx_INCREF(__pyx_n_u_DT);
  __Pyx_GIVEREF(__pyx_n_u_DT);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_DT);
  __Pyx_INCREF(__pyx_n_u_S);
  __Pyx_GIVEREF(__pyx_n_u_S);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_S);
  __Pyx_INCREF(__pyx_n_u_SQ);
  __Pyx_GIVEREF(__pyx_n_u_SQ);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_SQ);
  __Pyx_INCREF(__pyx_n_u_SINV);
  __Pyx_GIVEREF(__pyx_n_u_SINV);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_SINV);
  __Pyx_INCREF(__pyx_n_u_SBAR);
  __Pyx_GIVEREF(__pyx_n_u_SBAR);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_SBAR);
  __Pyx_INCREF(__pyx_n_u_FRAG);
  __Pyx_GIVEREF(__pyx_n_u_FRAG);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_FRAG);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_SBAR, __pyx_t_2) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":25
 *     S = ["TO", "IN", "VP", "S", "SBAR", "ADJP", "UCP", "NP"]
 *     SBAR = ["WHNP", "WHPP", "WHADVP", "WHADJP", "IN", "DT", "S", "SQ", "SINV", "SBAR", "FRAG"]
 *     SBARQ = ["SQ", "S", "SINV", "SBARQ", "FRAG"]             # <<<<<<<<<<<<<<
 *     SINV = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "S", "SINV", "ADJP", "NP"]
 *     SQ = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "SQ"]
 */
  __pyx_t_2 = PyList_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_SQ);
  __Pyx_GIVEREF(__pyx_n_u_SQ);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_SQ);
  __Pyx_INCREF(__pyx_n_u_S);
  __Pyx_GIVEREF(__pyx_n_u_S);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_S);
  __Pyx_INCREF(__pyx_n_u_SINV);
  __Pyx_GIVEREF(__pyx_n_u_SINV);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_SINV);
  __Pyx_INCREF(__pyx_n_u_SBARQ);
  __Pyx_GIVEREF(__pyx_n_u_SBARQ);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_SBARQ);
  __Pyx_INCREF(__pyx_n_u_FRAG);
  __Pyx_GIVEREF(__pyx_n_u_FRAG);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_FRAG);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_SBARQ, __pyx_t_2) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":26
 *     SBAR = ["WHNP", "WHPP", "WHADVP", "WHADJP", "IN", "DT", "S", "SQ", "SINV", "SBAR", "FRAG"]
 *     SBARQ = ["SQ", "S", "SINV", "SBARQ", "FRAG"]
 *     SINV = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "S", "SINV", "ADJP", "NP"]             # <<<<<<<<<<<<<<
 *     SQ = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "SQ"]
 *     UCP = []
 */
  __pyx_t_2 = PyList_New(10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_VBZ);
  __Pyx_GIVEREF(__pyx_n_u_VBZ);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_VBZ);
  __Pyx_INCREF(__pyx_n_u_VBD);
  __Pyx_GIVEREF(__pyx_n_u_VBD);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_VBD);
  __Pyx_INCREF(__pyx_n_u_VBP);
  __Pyx_GIVEREF(__pyx_n_u_VBP);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_VBP);
  __Pyx_INCREF(__pyx_n_u_VB);
  __Pyx_GIVEREF(__pyx_n_u_VB);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_VB);
  __Pyx_INCREF(__pyx_n_u_MD);
  __Pyx_GIVEREF(__pyx_n_u_MD);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_MD);
  __Pyx_INCREF(__pyx_n_u_VP);
  __Pyx_GIVEREF(__pyx_n_u_VP);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_VP);
  __Pyx_INCREF(__pyx_n_u_S);
  __Pyx_GIVEREF(__pyx_n_u_S);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_S);
  __Pyx_INCREF(__pyx_n_u_SINV);
  __Pyx_GIVEREF(__pyx_n_u_SINV);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_SINV);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_NP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_SINV, __pyx_t_2) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":27
 *     SBARQ = ["SQ", "S", "SINV", "SBARQ", "FRAG"]
 *     SINV = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "S", "SINV", "ADJP", "NP"]
 *     SQ = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "SQ"]             # <<<<<<<<<<<<<<
 *     UCP = []
 *     VP = ["TO", "VBD", "VBN", "MD", "VBZ", "VB", "VBG", "VBP", "VP", "ADJP", "NN", "NNS", "NP"]
 */
  __pyx_t_2 = PyList_New(7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_VBZ);
  __Pyx_GIVEREF(__pyx_n_u_VBZ);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_VBZ);
  __Pyx_INCREF(__pyx_n_u_VBD);
  __Pyx_GIVEREF(__pyx_n_u_VBD);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_VBD);
  __Pyx_INCREF(__pyx_n_u_VBP);
  __Pyx_GIVEREF(__pyx_n_u_VBP);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_VBP);
  __Pyx_INCREF(__pyx_n_u_VB);
  __Pyx_GIVEREF(__pyx_n_u_VB);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_VB);
  __Pyx_INCREF(__pyx_n_u_MD);
  __Pyx_GIVEREF(__pyx_n_u_MD);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_MD);
  __Pyx_INCREF(__pyx_n_u_VP);
  __Pyx_GIVEREF(__pyx_n_u_VP);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_VP);
  __Pyx_INCREF(__pyx_n_u_SQ);
  __Pyx_GIVEREF(__pyx_n_u_SQ);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_SQ);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_SQ, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":28
 *     SINV = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "S", "SINV", "ADJP", "NP"]
 *     SQ = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "SQ"]
 *     UCP = []             # <<<<<<<<<<<<<<
 *     VP = ["TO", "VBD", "VBN", "MD", "VBZ", "VB", "VBG", "VBP", "VP", "ADJP", "NN", "NNS", "NP"]
 *     WHADJP = ["CC", "WRB", "JJ", "ADJP"]
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_UCP, __pyx_t_2) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":29
 *     SQ = ["VBZ", "VBD", "VBP", "VB", "MD", "VP", "SQ"]
 *     UCP = []
 *     VP = ["TO", "VBD", "VBN", "MD", "VBZ", "VB", "VBG", "VBP", "VP", "ADJP", "NN", "NNS", "NP"]             # <<<<<<<<<<<<<<
 *     WHADJP = ["CC", "WRB", "JJ", "ADJP"]
 *     WHADVP = ["CC", "WRB"]
 */
  __pyx_t_2 = PyList_New(13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_TO);
  __Pyx_GIVEREF(__pyx_n_u_TO);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_TO);
  __Pyx_INCREF(__pyx_n_u_VBD);
  __Pyx_GIVEREF(__pyx_n_u_VBD);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_VBD);
  __Pyx_INCREF(__pyx_n_u_VBN);
  __Pyx_GIVEREF(__pyx_n_u_VBN);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_VBN);
  __Pyx_INCREF(__pyx_n_u_MD);
  __Pyx_GIVEREF(__pyx_n_u_MD);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_MD);
  __Pyx_INCREF(__pyx_n_u_VBZ);
  __Pyx_GIVEREF(__pyx_n_u_VBZ);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_VBZ);
  __Pyx_INCREF(__pyx_n_u_VB);
  __Pyx_GIVEREF(__pyx_n_u_VB);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_VB);
  __Pyx_INCREF(__pyx_n_u_VBG);
  __Pyx_GIVEREF(__pyx_n_u_VBG);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_VBG);
  __Pyx_INCREF(__pyx_n_u_VBP);
  __Pyx_GIVEREF(__pyx_n_u_VBP);
  PyList_SET_ITEM(__pyx_t_2, 7, __pyx_n_u_VBP);
  __Pyx_INCREF(__pyx_n_u_VP);
  __Pyx_GIVEREF(__pyx_n_u_VP);
  PyList_SET_ITEM(__pyx_t_2, 8, __pyx_n_u_VP);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 9, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_NN);
  __Pyx_GIVEREF(__pyx_n_u_NN);
  PyList_SET_ITEM(__pyx_t_2, 10, __pyx_n_u_NN);
  __Pyx_INCREF(__pyx_n_u_NNS);
  __Pyx_GIVEREF(__pyx_n_u_NNS);
  PyList_SET_ITEM(__pyx_t_2, 11, __pyx_n_u_NNS);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 12, __pyx_n_u_NP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_VP, __pyx_t_2) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":30
 *     UCP = []
 *     VP = ["TO", "VBD", "VBN", "MD", "VBZ", "VB", "VBG", "VBP", "VP", "ADJP", "NN", "NNS", "NP"]
 *     WHADJP = ["CC", "WRB", "JJ", "ADJP"]             # <<<<<<<<<<<<<<
 *     WHADVP = ["CC", "WRB"]
 *     WHNP = ["WDT", "WP", "WP$", "WHADJP", "WHPP", "WHNP"]
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_CC);
  __Pyx_GIVEREF(__pyx_n_u_CC);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_CC);
  __Pyx_INCREF(__pyx_n_u_WRB);
  __Pyx_GIVEREF(__pyx_n_u_WRB);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_WRB);
  __Pyx_INCREF(__pyx_n_u_JJ);
  __Pyx_GIVEREF(__pyx_n_u_JJ);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_JJ);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_ADJP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_WHADJP, __pyx_t_2) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":31
 *     VP = ["TO", "VBD", "VBN", "MD", "VBZ", "VB", "VBG", "VBP", "VP", "ADJP", "NN", "NNS", "NP"]
 *     WHADJP = ["CC", "WRB", "JJ", "ADJP"]
 *     WHADVP = ["CC", "WRB"]             # <<<<<<<<<<<<<<
 *     WHNP = ["WDT", "WP", "WP$", "WHADJP", "WHPP", "WHNP"]
 *     WHPP = ["IN", "TO", "FW"]
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_CC);
  __Pyx_GIVEREF(__pyx_n_u_CC);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_CC);
  __Pyx_INCREF(__pyx_n_u_WRB);
  __Pyx_GIVEREF(__pyx_n_u_WRB);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_WRB);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_WHADVP, __pyx_t_2) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":32
 *     WHADJP = ["CC", "WRB", "JJ", "ADJP"]
 *     WHADVP = ["CC", "WRB"]
 *     WHNP = ["WDT", "WP", "WP$", "WHADJP", "WHPP", "WHNP"]             # <<<<<<<<<<<<<<
 *     WHPP = ["IN", "TO", "FW"]
 *     NP1 = ["NN", "NNP", "NNPS", "NNS", "NX", "POS", "JJR"]
 */
  __pyx_t_2 = PyList_New(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_WDT);
  __Pyx_GIVEREF(__pyx_n_u_WDT);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_WDT);
  __Pyx_INCREF(__pyx_n_u_WP);
  __Pyx_GIVEREF(__pyx_n_u_WP);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_WP);
  __Pyx_INCREF(__pyx_kp_u_WP_2);
  __Pyx_GIVEREF(__pyx_kp_u_WP_2);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_WP_2);
  __Pyx_INCREF(__pyx_n_u_WHADJP);
  __Pyx_GIVEREF(__pyx_n_u_WHADJP);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_WHADJP);
  __Pyx_INCREF(__pyx_n_u_WHPP);
  __Pyx_GIVEREF(__pyx_n_u_WHPP);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_WHPP);
  __Pyx_INCREF(__pyx_n_u_WHNP);
  __Pyx_GIVEREF(__pyx_n_u_WHNP);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_WHNP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_WHNP, __pyx_t_2) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":33
 *     WHADVP = ["CC", "WRB"]
 *     WHNP = ["WDT", "WP", "WP$", "WHADJP", "WHPP", "WHNP"]
 *     WHPP = ["IN", "TO", "FW"]             # <<<<<<<<<<<<<<
 *     NP1 = ["NN", "NNP", "NNPS", "NNS", "NX", "POS", "JJR"]
 *     NP2 = ["NP"]
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_IN);
  __Pyx_GIVEREF(__pyx_n_u_IN);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_IN);
  __Pyx_INCREF(__pyx_n_u_TO);
  __Pyx_GIVEREF(__pyx_n_u_TO);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_TO);
  __Pyx_INCREF(__pyx_n_u_FW);
  __Pyx_GIVEREF(__pyx_n_u_FW);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_FW);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_WHPP, __pyx_t_2) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":34
 *     WHNP = ["WDT", "WP", "WP$", "WHADJP", "WHPP", "WHNP"]
 *     WHPP = ["IN", "TO", "FW"]
 *     NP1 = ["NN", "NNP", "NNPS", "NNS", "NX", "POS", "JJR"]             # <<<<<<<<<<<<<<
 *     NP2 = ["NP"]
 *     NP3 = ["$", "ADJP", "PRN"]
 */
  __pyx_t_2 = PyList_New(7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_NN);
  __Pyx_GIVEREF(__pyx_n_u_NN);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_NN);
  __Pyx_INCREF(__pyx_n_u_NNP);
  __Pyx_GIVEREF(__pyx_n_u_NNP);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_NNP);
  __Pyx_INCREF(__pyx_n_u_NNPS);
  __Pyx_GIVEREF(__pyx_n_u_NNPS);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_NNPS);
  __Pyx_INCREF(__pyx_n_u_NNS);
  __Pyx_GIVEREF(__pyx_n_u_NNS);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_NNS);
  __Pyx_INCREF(__pyx_n_u_NX);
  __Pyx_GIVEREF(__pyx_n_u_NX);
  PyList_SET_ITEM(__pyx_t_2, 4, __pyx_n_u_NX);
  __Pyx_INCREF(__pyx_n_u_POS);
  __Pyx_GIVEREF(__pyx_n_u_POS);
  PyList_SET_ITEM(__pyx_t_2, 5, __pyx_n_u_POS);
  __Pyx_INCREF(__pyx_n_u_JJR);
  __Pyx_GIVEREF(__pyx_n_u_JJR);
  PyList_SET_ITEM(__pyx_t_2, 6, __pyx_n_u_JJR);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_NP1, __pyx_t_2) < 0) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":35
 *     WHPP = ["IN", "TO", "FW"]
 *     NP1 = ["NN", "NNP", "NNPS", "NNS", "NX", "POS", "JJR"]
 *     NP2 = ["NP"]             # <<<<<<<<<<<<<<
 *     NP3 = ["$", "ADJP", "PRN"]
 *     NP4 = ["CD"]
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_NP);
  __Pyx_GIVEREF(__pyx_n_u_NP);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_NP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_NP2, __pyx_t_2) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":36
 *     NP1 = ["NN", "NNP", "NNPS", "NNS", "NX", "POS", "JJR"]
 *     NP2 = ["NP"]
 *     NP3 = ["$", "ADJP", "PRN"]             # <<<<<<<<<<<<<<
 *     NP4 = ["CD"]
 *     NP5 = ["JJ", "JJS", "RB", "QP"]
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_kp_u__6);
  __Pyx_GIVEREF(__pyx_kp_u__6);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u__6);
  __Pyx_INCREF(__pyx_n_u_ADJP);
  __Pyx_GIVEREF(__pyx_n_u_ADJP);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_ADJP);
  __Pyx_INCREF(__pyx_n_u_PRN);
  __Pyx_GIVEREF(__pyx_n_u_PRN);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_PRN);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_NP3, __pyx_t_2) < 0) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":37
 *     NP2 = ["NP"]
 *     NP3 = ["$", "ADJP", "PRN"]
 *     NP4 = ["CD"]             # <<<<<<<<<<<<<<
 *     NP5 = ["JJ", "JJS", "RB", "QP"]
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_CD);
  __Pyx_GIVEREF(__pyx_n_u_CD);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_CD);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_NP4, __pyx_t_2) < 0) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "ParseTree/ParseNode.pyx":38
 *     NP3 = ["$", "ADJP", "PRN"]
 *     NP4 = ["CD"]
 *     NP5 = ["JJ", "JJS", "RB", "QP"]             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, dataOrParent, leftOrLine=None, rightOrIsLeaf=None):
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_u_JJ);
  __Pyx_GIVEREF(__pyx_n_u_JJ);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_u_JJ);
  __Pyx_INCREF(__pyx_n_u_JJS);
  __Pyx_GIVEREF(__pyx_n_u_JJS);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_u_JJS);
  __Pyx_INCREF(__pyx_n_u_RB);
  __Pyx_GIVEREF(__pyx_n_u_RB);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_u_RB);
  __Pyx_INCREF(__pyx_n_u_QP);
  __Pyx_GIVEREF(__pyx_n_u_QP);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_u_QP);
  if (PyDict_SetItem((PyObject *)__pyx_ptype_9ParseTree_9ParseNode_ParseNode->tp_dict, __pyx_n_s_NP5, __pyx_t_2) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  PyType_Modified(__pyx_ptype_9ParseTree_9ParseNode_ParseNode);

  /* "(tree fragment)":1
 * def __pyx_unpickle_ParseNode(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9ParseTree_9ParseNode_1__pyx_unpickle_ParseNode, NULL, __pyx_n_s_ParseTree_ParseNode); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ParseNode, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ParseTree/ParseNode.pyx":1
 * from Dictionary.Word cimport Word             # <<<<<<<<<<<<<<
 * 
 * from ParseTree.ConstituentSpan cimport ConstituentSpan
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ParseTree.ParseNode", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ParseTree.ParseNode");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, int inplace, int zerodivision_check) {
    (void)inplace;
    (void)zerodivision_check;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                    CYTHON_FALLTHROUGH;
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* pop_index */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix) {
    PyObject *r;
    if (unlikely(!py_ix)) return NULL;
    r = __Pyx__PyObject_PopIndex(L, py_ix);
    Py_DECREF(py_ix);
    return r;
}
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix) {
    return __Pyx_PyObject_CallMethod1(L, __pyx_n_s_pop, py_ix);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix) {
    Py_ssize_t size = PyList_GET_SIZE(L);
    if (likely(size > (((PyListObject*)L)->allocated >> 1))) {
        Py_ssize_t cix = ix;
        if (cix < 0) {
            cix += size;
        }
        if (likely(__Pyx_is_valid_index(cix, size))) {
            PyObject* v = PyList_GET_ITEM(L, cix);
            __Pyx_SET_SIZE(L, Py_SIZE(L) - 1);
            size -= 1;
            memmove(&PyList_GET_ITEM(L, cix), &PyList_GET_ITEM(L, cix+1), (size_t)(size-cix)*sizeof(PyObject*));
            return v;
        }
    }
    if (py_ix == Py_None) {
        return __Pyx__PyObject_PopNewIndex(L, PyInt_FromSsize_t(ix));
    } else {
        return __Pyx__PyObject_PopIndex(L, py_ix);
    }
}
#endif

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* None */
static CYTHON_INLINE Py_ssize_t __Pyx_mod_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t r = a % b;
    r += ((r != 0) & ((r ^ b) < 0)) * b;
    return r;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) ((int) 0 - (int) 1), const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) ((long) 0 - (long) 1), const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
